<?xml version="1.0" encoding="ISO-8859-1"?><xsl:stylesheet version="1.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:SMLNL="http://www.shef.ac.uk/SpineMLNetworkLayer"  xmlns:SMLLOWNL="http://www.shef.ac.uk/SpineMLLowLevelNetworkLayer" xmlns:SMLCL="http://www.shef.ac.uk/SpineMLComponentLayer" xmlns:NMLEX="http://www.shef.ac.uk/SpineMLExperimentLayer" xmlns:fn="http://www.w3.org/2005/xpath-functions">
<xsl:output method="text" version="1.0" encoding="UTF-8" indent="yes"/>

<!-- INCLUDE THE HELPER TEMPLATES -->
<xsl:include href="SpineML_helpers.xsl"/>

<!-- MAIN TEMPLATE -->
<xsl:template match="/"> <!-- mode="prototypes"-->

<!-- CREATE REFERENCES TO THE FILES USED IN THIS SCRIPT -->
<xsl:variable name="expt_file" select="/"/>
<xsl:variable name="model_file" select="document(concat('../model/', $expt_file//NMLEX:Model/@network_layer_url))"/>
<xsl:variable name="split_model_file" select="document(concat('../model/split_', $expt_file//NMLEX:Model/@network_layer_url))"/>

<!-- MOVE INTO THE ORIGINAL MODEL FILE -->
<xsl:for-each select="$model_file">

<!-- SET UP SOME FIXED VALUES FOR MAXIMUM CONNECTIVITY -->
<xsl:variable name="max_hash_size" select="'80'"/><!-- REPLACE LATER -->
<xsl:variable name="max_hash_columns" select="'3'"/>
<xsl:variable name="max_total_inputs" select="'80'"/><!-- REPLACE LATER -->
<xsl:variable name="inputs_per_port" select="'80'"/><!-- REPLACE LATER -->
<xsl:variable name="number_of_input_ports" select='80'/><!-- REPLACE LATER -->
<xsl:variable name="max_pop_size" select="'100'"/><!-- REPLACE LATER -->

#timestamp on

#monitor off

#define e 2.71828183
#define EXP_MAX 10.397

<!-- DO ALL THE #DEFINES -->
#define INPUTS_PER_PORT <xsl:value-of select="$inputs_per_port"/>
//#define MAX_NUMBER_OF_PORTS 10
#define MAX_TOTAL_INPUTS <xsl:value-of select="$max_total_inputs"/>;
#define MAX_HASH_SIZE <xsl:value-of select="$max_hash_size"/>;
#define NUM_NODES <xsl:value-of select="count($split_model_file//SMLLOWNL:Population)"/>

<!-- DEFINE THE REGIMES GLOBALLY OUTSIDE THE NODE TO AVOID ERRORS DUE TO REPEATS -->
// regime definitions:
<xsl:for-each select="document(//@url)">
	<xsl:for-each select="//SMLCL:Dynamics">
		<xsl:apply-templates select="SMLCL:Regime" mode="defineRegimes"/>
	</xsl:for-each>
</xsl:for-each>

// Neuron numbers for each population component type
<xsl:for-each select="document(//@url)">
	<xsl:for-each select="//SMLCL:ComponentClass">
#define N_<xsl:value-of select="translate(@name, ' -', 'oo')"/><xsl:text> </xsl:text>10 <!-- UPDATE WITH VALUE FROM CONSTRAINTS FILE -->
	</xsl:for-each>
</xsl:for-each>



<!-- EACH POPULATION IN THE ORIGINAL FILE WILL BE AN INDIVIDUAL NODE -->
<xsl:for-each select="/SMLLOWNL:SpineML/SMLLOWNL:Population">

<!-- SET UP OFTEN USED VARIBLES -->
<xsl:variable name="curr_pop" select="."/>
<xsl:variable name="curr_pop_name" select="SMLLOWNL:Neuron/@name"/>
<xsl:variable name="curr_pop_name_sanitised" select="translate(SMLLOWNL:Neuron/@name,' ', 'o')"/>
<xsl:variable name="curr_pop_size" select="SMLLOWNL:Neuron/@size"/>
<xsl:variable name="curr_pop_component_file" select="document(./SMLLOWNL:Neuron/@url)"/>

<xsl:variable name="nrn_per_core" select="$curr_pop_size"/><!-- REPLACE LATER -->

<!-- GET A LIST OF THE PROJECTIONS THAT TARGET THIS POPULATION-->
<xsl:variable name="proj_inputs" select="/SMLLOWNL:SpineML/SMLLOWNL:Population/SMLLOWNL:Projection[@dst_population=$curr_pop_name]/SMLLOWNL:Synapse"/>

<!-- GET A LIST OF ALL THE COMPONENTS IN THE NRN AND ITS PROJECTIONS -->
<xsl:variable name="all_components" select="$proj_inputs/SMLLOWNL:WeightUpdate/@url | $proj_inputs/SMLLOWNL:PostSynapse/@url | $curr_pop/SMLLOWNL:Neuron/@url"/>

<!--  -->

/*
   Autogenerated DAMSON file from SpineML description.
   Engine: XSLT
   Engine Author: Alex Cope 2012
   Node name: <xsl:value-of select="$curr_pop_name"/>
*/

#node <xsl:value-of select="$curr_pop_name_sanitised"/>

// number of neurons for this instance
int num_neurons_actual;

// counter for init
int nodes_to_see = NUM_NODES;

// array for which ports should be output
int portOn[<xsl:value-of select="$max_total_inputs"/>];

// the counter for clock interrupts (i.e. number of iterations)
int timerCounter = 0;
float t;

float dt = <xsl:value-of select="$expt_file//NMLEX:Simulation/*/@dt"/>;
float inv_dt = <xsl:value-of select="1.0 div number($expt_file//NMLEX:Simulation/*/@dt)"/>;

int hashLookupTable[MAX_HASH_SIZE];
int hashSwitchTable[MAX_HASH_SIZE];
int hashInputIndexTable[MAX_HASH_SIZE];

int hashCreation[MAX_TOTAL_INPUTS][<xsl:value-of select="$max_hash_columns"/>];

// for logging events
int current_neuron_index;

// helper functions #############################################

////////////////////////////////// Integrate
float solveDE(float,float);

////////////////////////////////// RNG

float convert_no_cast(float);

int seed = 11;
int a_RNG = 1103515245;
int c_RNG = 12345;
float uniformRNG(float, float);

float pow(float, int);

float exp(float);

////////////////////////////////// #1
int hashSwitch(int);

////////////////////////////////// #2
int hashInputIndex(int);

// SYNAPSES DECLARATIONS //////////////////////////////////
<!-- FOR EACH OF THE OBJECTS CONNECTING INTO THE CURRENT NEURON BODY -->
<xsl:for-each select="$proj_inputs">
	<!-- IF IT IS A SYNAPSE -->
        <xsl:apply-templates select="SMLLOWNL:WeightUpdate" mode="defineAll">
		<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
		<xsl:with-param name="split_model_file" select="$split_model_file"/>
		<xsl:with-param name="n_dst" select="$max_pop_size"/>
		<xsl:with-param name="n_src" select="$max_pop_size"/>
	</xsl:apply-templates>
</xsl:for-each>

// PostSynapse DECLARATIONS ///////////////////////////////////////
<!-- FOR EACH OF THE OBJECTS CONNECTING INTO THE CURRENT NEURON BODY -->
<xsl:for-each select="$proj_inputs">
	<!-- IF IT IS A PostSynapse -->
        <xsl:apply-templates select="SMLLOWNL:PostSynapse" mode="defineAll">
		<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
		<xsl:with-param name="split_model_file" select="$split_model_file"/>
		<xsl:with-param name="n_dst" select="$max_pop_size"/>
		<xsl:with-param name="n_src" select="$max_pop_size"/>
	</xsl:apply-templates>
</xsl:for-each>


// NEURON DECLARATIONS ////////////////////////////////////
<!-- DEFINE ALL FOR THE CURRENT NEURON BODY -->
<xsl:for-each select="$curr_pop">
        <xsl:apply-templates select="SMLLOWNL:Neuron" mode="defineAll">
		<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
		<xsl:with-param name="split_model_file" select="$split_model_file"/>
		<xsl:with-param name="n_dst" select="$max_pop_size"/>
		<xsl:with-param name="n_src" select="$max_pop_size"/>
	</xsl:apply-templates>
</xsl:for-each>

// GENERIC INPUTS /////////////////////////////////////////
///////////////////////////////////////////////////////////

// SYNAPSE GENERIC INPUTS /////////////////////////////////
<!-- FOR EACH OF THE OBJECTS CONNECTING INTO THE CURRENT NEURON BODY -->
<xsl:for-each select="$proj_inputs">
        <xsl:if test="count(SMLLOWNL:WeightUpdate/SMLLOWNL:Input)>0">
		<!---->!!! Error: learing not implemented yet - generic inputs to synapses are not permitted !!!
<!---->
	</xsl:if>
</xsl:for-each>

// PostSynapse GENERIC INPUTS /////////////////////////////////////
<!-- FOR EACH OF THE OBJECTS CONNECTING INTO THE CURRENT NEURON BODY -->
<xsl:for-each select="$proj_inputs">
        <xsl:apply-templates select="SMLLOWNL:PostSynapse/SMLLOWNL:Input" mode="definePostSynapseInput">
		<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
		<xsl:with-param name="split_model_file" select="$split_model_file"/>
		<xsl:with-param name="n_dst" select="$max_pop_size"/>
		<xsl:with-param name="n_src" select="$max_pop_size"/>
	</xsl:apply-templates>
</xsl:for-each>

// NRN GENERIC INPUTS /////////////////////////////////////
<!-- FOR EACH OF THE OBJECTS CONNECTING INTO THE CURRENT NEURON BODY -->
<xsl:apply-templates select="SMLLOWNL:Neuron/SMLLOWNL:Input" mode="defineNRNInput">
	<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
	<xsl:with-param name="split_model_file" select="$split_model_file"/>
	<xsl:with-param name="n_dst" select="$max_pop_size"/>
	<xsl:with-param name="n_src" select="$max_pop_size"/>
</xsl:apply-templates>


// PROTOTYPES FOR THE INTERRUPTS
int event_int(int,int,float,int);
void clock_int(int,int,int);
// FUNCTIONS FOR THE PostSynapse COMPONENTS
<xsl:for-each select="document($proj_inputs/SMLLOWNL:PostSynapse/@url)">
<!---->// <xsl:value-of select=".//SMLCL:ComponentClass/@name"/> //
<!----><!-- FUNCTION DEFINITION -->
<!---->void <xsl:value-of select="translate(.//SMLCL:ComponentClass/@name,' -', '__')"/>
	<!---->(int i,<!---->
	<xsl:for-each select="//SMLCL:Parameter">
		<!---->float <!---->
		<xsl:value-of select="@name"/>
		<!---->,<!---->
	</xsl:for-each>
	<xsl:for-each select="//SMLCL:StateVariable | //SMLCL:AnalogReceivePort | //SMLCL:AnalogReducePort">
		<!---->float <!---->
		<xsl:value-of select="@name"/>
		<!---->INST[<!---->
		<xsl:value-of select="$max_pop_size"/>
		<!---->],<!---->
	</xsl:for-each>
	<xsl:apply-templates select="//SMLCL:Alias" mode="passPortAliases">
		<xsl:with-param name="suffix" select="concat(concat('INST[', $max_pop_size),']')"/>
	</xsl:apply-templates>
	<!---->, int regimeINST[<!---->
	<xsl:value-of select="$max_pop_size"/>
	<!---->]);
</xsl:for-each>
// FUNCTION FOR THE NEURON BODY COMPONENT
<xsl:for-each select="document(./SMLLOWNL:Neuron/@url)">
<!---->// <xsl:value-of select=".//SMLCL:ComponentClass/@name"/> //
<!----><!-- FUNCTION DEFINITION -->
<!---->void <xsl:value-of select="translate(.//SMLCL:ComponentClass/@name,' -', '__')"/>
	<!---->(int i,<!---->
	<xsl:for-each select="//SMLCL:Parameter">
		<!---->float <!---->
		<xsl:value-of select="@name"/>
		<!---->,<!---->
	</xsl:for-each>
	<xsl:for-each select="//SMLCL:StateVariable | //SMLCL:AnalogReceivePort | //SMLCL:AnalogReducePort">
		<!---->float <!---->
		<xsl:value-of select="@name"/>
		<!---->INST[<!---->
		<xsl:value-of select="$max_pop_size"/>
		<!---->],<!---->
	</xsl:for-each>
	<xsl:apply-templates select="//SMLCL:Alias" mode="passPortAliases">
		<xsl:with-param name="suffix" select="concat(concat('INST[', $max_pop_size),']')"/>
	</xsl:apply-templates>
	<!---->int regimeINST[<!---->
	<xsl:value-of select="$max_pop_size"/>
	<!---->]);
</xsl:for-each>
	
<xsl:for-each select="$proj_inputs/SMLLOWNL:PostSynapse">
	<xsl:variable name="component" select="document(@url)"/>
	<xsl:variable name="dst_port" select="@input_dst_port"/>
	<xsl:variable name="suffix">
		<xsl:value-of select="translate(@name,' -','__')"/><!-- NEW SUFFIX -->
	</xsl:variable>
// IMPULSE HANDLER
<!----><xsl:if test="count($component//SMLCL:OnImpulse[@src_port=$dst_port])=1">
		void HANDLE_IMPULSE_<xsl:value-of select="$suffix"/>_<xsl:value-of select="$dst_port"/>(int i, float <xsl:value-of select="$dst_port"/>);
	</xsl:if>
</xsl:for-each>
	


<!-- MAIN -->
// MAIN FUNCTION
int main() 
{

// DEFINITIONS
int i;
int j;
int k;
int index;
int input;
int counter;
// temporary storage for generated values
int tempInts[<xsl:value-of select="$curr_pop_size"/>]; // this population's size
float tempFloats[<xsl:value-of select="$curr_pop_size"/>]; // this population's size
int connRowLengths[100]; // a temporary array for probabalistic connections that stores row lengths for initialising Pars SVs etc... since we don't know the number of nrns in the src the num of elements is quite large 

<!-- DEBUGGING -->//printf("IN MAIN\n");<!-- /DEBUGGING -->

// set the tickrate for the simulation
tickrate(1000000000);

// SET INITIAL HASH TABLE VALUES
for (i = 0; i &lt; MAX_HASH_SIZE; i+=1) {
	hashLookupTable[i]=-200;
}
<xsl:variable name="all_components_comps" select="$all_components/.."/>
<xsl:if test="count($all_components/..//SMLLOWNL:Input | $all_components_comps[local-name(.)='WeightUpdate'] )>0">
// GENERATE THE HASH TABLES
for (i = 0; i &lt; MAX_TOTAL_INPUTS; i+=1) {

	if (hashCreation[i][0] != 0) { 
		// add value into first hash table
		input = hashCreation[i][0];
		index = (input * 137 + 92731) % MAX_HASH_SIZE;
		while (1==1){
		    if(hashLookupTable[index]==-200)
			{
				hashLookupTable[index]=input;
				hashSwitchTable[index]=hashCreation[i][1];
				hashInputIndexTable[index]=hashCreation[i][2];
				break;
			}
		    index += 1;
		    if(index >= MAX_HASH_SIZE)
		        index = 0;
		}
    }
}
</xsl:if>

<!-- DEBUGGING -->// HACK
seed = 123;<!-- /DEBUGGING -->

//////////////////////////////////////////////////////
// 			BEGIN INITIALISATION OF DATA			//
//////////////////////////////////////////////////////

////// COMPONENTS ####################################
<xsl:apply-templates select="$proj_inputs/SMLLOWNL:WeightUpdate | $proj_inputs/SMLLOWNL:PostSynapse | SMLLOWNL:Neuron" mode="initData">
	<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
	<xsl:with-param name="split_model_file" select="$split_model_file"/>
	<xsl:with-param name="n_dst" select="$max_pop_size"/>
	<xsl:with-param name="n_src" select="$max_pop_size"/>
</xsl:apply-templates>

////// INPUTS #######################################

<xsl:apply-templates select="$proj_inputs/SMLLOWNL:PostSynapse | SMLLOWNL:Neuron" mode="initInputData">
	<xsl:with-param name="split_model_file" select="$split_model_file"/>
	<xsl:with-param name="n_dst" select="$max_pop_size"/>
	<xsl:with-param name="n_src" select="$max_pop_size"/>
</xsl:apply-templates>

//////////////////////////////////////////////////////
// 			END INITIALISATION OF DATA				//
//////////////////////////////////////////////////////



<!-- DEBUGGING --> printf("MAIN COMPLETE\n");<!-- /DEBUGGING -->
 
 <!-- SEND TO SAY THAT MAIN IS COMPLETE, THIS IS AN ISSUE AS TO WORK IT REQUIRES COMPLETE CONNECTIVITY -->
 syncnodes();
 return 0;
}

// helper functions #############################################

////////////////////////////////// Integrate
float solveDE(float x,float fx) {
  return x + fx*dt;
}

float convert_no_cast(float val) {
	return val;
}

////////////////////////////////// RNG
float uniformRNG(float ub, float lb) {
	float val;
	float seed2;
	seed = abs(seed*a_RNG+c_RNG);
	seed2 = convert_no_cast(seed>>15);
	val = (seed2)*(ub-lb)+lb;
	return val;
}

////////////////////////////////// POW
float pow(float value, int power) {

	int i;
	float result = 1;
	for (i = 0; i &lt; power; i += 1) {
		result *= value;
	}
	return result;
}

////////////////////////////////// EXP
float exp(float x) {
	
	int   n = 1;	//expansion step
	float f = 1.0;	//factorial n!
	float r = 1.0;	//result is sum of expansion steps
	float s = 1.0;	//expansion step value is x^n/n!
	float p = x;	//power x^n
	
	if (x>EXP_MAX)	//will overflow so return max float or error
		return 32768.0;	
	
	if (x>1)		//recursive step exploits exponential indentity (exp(1+x) = E*exp(x))
		return e*exp(x-1.0);
	else			//Taylor expansion
	{

		
		while (s > 0){	//while step value is within fixed point range
			f = f*n;
			s = (p/f);
			r = r + s;
			p = p * x;
			n += 1;
		}
		
		return r;
	}

}

////////////////////////////////// #1
int hashSwitch(int input) {

	int index;

    index = (input * 137 + 92731) % MAX_HASH_SIZE;
    while (hashLookupTable[index] != -200){
        if(hashLookupTable[index]==input)
			return hashSwitchTable[index];
		if(hashLookupTable[index]==-200)
			return -1;
        index += 1;
        if(index >= MAX_HASH_SIZE)
            index = 0;
    }
    return -1;
}

////////////////////////////////// #2
int hashInputIndex(int input) {

	int index;

    index = (input * 137 + 92731) % MAX_HASH_SIZE;
    while (hashLookupTable[index] != -200){
        if(hashLookupTable[index]==input)
			return hashInputIndexTable[index];
		if(hashLookupTable[index]==-200)
			return -1;
        index += 1;
        if(index >= MAX_HASH_SIZE)
            index = 0;
    }
    return -1;
}

//////////////////////////////////// CLOCK INTERRUPT - THIS IS USED TO UPDATE THE SYSTEM EACH ITERATION
void clock_int(int not_using_this_1, int not_using_this_2, int t_not_used) {

	// declare variables
	int i;
	int t2; // used for rolling buffers
	float sum;
	int portNum;
	int portID;

	// make time available
	t = (float) timerCounter * dt;
		
	<xsl:for-each select="$all_components">
	// engage semaphores:
<!--					 -->
	<xsl:variable name="currComponent" select="."/>
    <xsl:variable name="componentName" select="../@name"/>
        <xsl:if test="not(local-name(..)='WeightUpdate')">
            <xsl:for-each select="..//SMLLOWNL:Input">
            	<xsl:variable name="suffix_name">
            		<xsl:value-of select="translate($componentName,' -','__')"/><!-- NEW SUFFIX -->
            		<!---->_input_<!---->
            		<xsl:value-of select="position()"/>
				</xsl:variable>
                <xsl:variable name="dstPortName" select="@dst_port"/>
                <!-- DON'T DO CONDUCTANCE BASED SYNAPSE INPUTS -->
				<xsl:if test="not(local-name(..)='PostSynapse' and @src=../../../@dst_population and count(SMLNL:OneToOneConnection)=1)">
					<xsl:if test="count(document($currComponent)//SMLCL:AnalogReducePort[@name=$dstPortName] | document($currComponent)//SMLCL:AnalogReceivePort[@name=$dstPortName])=1">
<!---->					<!---->	wait(&amp;<xsl:value-of select="$suffix_name"/>_semaphore);
<!---->				</xsl:if>
				</xsl:if>
			</xsl:for-each>
		</xsl:if>
	</xsl:for-each>
	
	for (i=0; i &lt; num_neurons_actual; i+=1) {
	
		// current neuron index for logging purposes
		current_neuron_index = i;
<!---->	 
		<!-- UPDATE THE DYNAMICS OF THE PostSynapseSs -->
		// do PostSynapses ##########################################################
		<xsl:for-each select="$proj_inputs">
		
		    <xsl:variable name="suffix_name">
                                <xsl:value-of select="translate(SMLLOWNL:PostSynapse/@name,' -','__')"/><!-- NEW SUFFIX -->
	    	</xsl:variable>
		
                        <xsl:for-each select="SMLLOWNL:PostSynapse/SMLLOWNL:Input">
				<xsl:if test="@src=$curr_pop_name and count(SMLNL:OneToOneConnection)=1">
					<!---->// NRN -> PostSynapse (Conductance-based synapses)
		<!---->		<xsl:value-of select="@dst_port"/><xsl:value-of select="$suffix_name"/>[i] = <xsl:value-of select="@src_port"/><xsl:value-of select="translate(@src,' -','__')"/>[i];
		<!---->	</xsl:if>
			</xsl:for-each>
		
                        <xsl:variable name="curr_PostSynapse" select="SMLLOWNL:PostSynapse"/>
                <xsl:for-each select="document(SMLLOWNL:PostSynapse/@url)">
                                <xsl:value-of select="translate(.//SMLCL:ComponentClass/@name,' -', '__')"/>
	 			<!---->(i,<!---->
	 			<xsl:for-each select="//SMLCL:Parameter">
	 				<!-- IF IS A FIXED VALUE -->
	 				<xsl:variable name="parName" select="@name"/>
	 				<xsl:if test="count($curr_PostSynapse/SMLNL:Property[@name=$parName]/SMLNL:FixedValue)=1">
		 				<xsl:value-of select="@name"/>
		 				<xsl:value-of select="$suffix_name"/>
		 			</xsl:if>
	 				<!-- IF IS NOT A FIXED VALUE -->
	 				<xsl:if test="count($curr_PostSynapse/SMLNL:Property[@name=$parName]/SMLNL:FixedValue)=0">
		 				<xsl:value-of select="@name"/>
			 			<xsl:value-of select="$suffix_name"/>
			 			<!---->[i]<!---->
			 		</xsl:if>
	 				<!---->,<!---->
	 			</xsl:for-each>
	 			<xsl:for-each select="//SMLCL:StateVariable | //SMLCL:AnalogReceivePort | //SMLCL:AnalogReducePort">
	 				<xsl:value-of select="@name"/>
	 				<xsl:value-of select="$suffix_name"/>
	 				<!---->,<!---->
	 			</xsl:for-each>
	 			<xsl:apply-templates select="//SMLCL:Alias" mode="passPortAliases">
	 				<xsl:with-param name="suffix" select="$suffix_name"/>
	 			</xsl:apply-templates>
	 			<!--xsl:if test="count(//SMLCL:Regime)>1"-->
	 				<!---->regimeCurr<!---->
	 				<xsl:value-of select="$suffix_name"/>
	 			<!--/xsl:if-->
	 			<!---->);
<!---->
			</xsl:for-each>
		
		<!---->		// PostSynapse -> NRN
<!---->	<!-- CONNECT THE PostSynapse TO THE NRN USING THE PROJECTION OUTPUT MAPPING -->
        <xsl:variable name="dstPortName" select=".//SMLLOWNL:PostSynapse/@output_dst_port"/>
    	<xsl:variable name="dstPop" select="../@dst_population"/>
        <xsl:value-of select="$dstPortName"/><xsl:value-of select="translate($dstPop,' -','__')"/>[i] <xsl:value-of select="document(//SMLLOWNL:Neuron[@name=$dstPop]/@url)//SMLCL:AnalogReducePort[@name=$dstPortName]/@reduce_op"/>= <xsl:value-of select="../..//SMLLOWNL:PostSynapse/@output_src_port"/><xsl:value-of select="$suffix_name"/>[i];
    	
    	
	</xsl:for-each>

	<!-- UPDATE THE DYNAMICS OF THE NRN BODY -->
	 	// do NRN ##########################################################
        <xsl:for-each select="document(.//SMLLOWNL:Neuron/@url)">
                        <xsl:value-of select="translate(.//SMLCL:ComponentClass/@name,' -', '__')"/>
 			<!---->(i,<!---->
 			<xsl:for-each select="//SMLCL:Parameter">
 				<!-- IF IS A FIXED VALUE -->
 				<xsl:variable name="parName" select="@name"/>
                                <xsl:if test="count($curr_pop/SMLLOWNL:Neuron/SMLNL:Property[@name=$parName]/SMLNL:FixedValue)=1">
	 				<xsl:value-of select="@name"/>
                                        <xsl:value-of select="translate($curr_pop/SMLLOWNL:Neuron/@name,' -','__')"/>
	 			</xsl:if>
 				<!-- IF IS NOT A FIXED VALUE -->
                                <xsl:if test="count($curr_pop/SMLLOWNL:Neuron/SMLNL:Property[@name=$parName]/SMLNL:FixedValue)=0">
	 				<xsl:value-of select="@name"/>
                                        <xsl:value-of select="translate($curr_pop/SMLLOWNL:Neuron/@name,' -','__')"/>
		 			<!---->[i]<!---->
		 		</xsl:if>
 				<!---->,<!---->
 			</xsl:for-each>
 			<xsl:for-each select="//SMLCL:StateVariable | //SMLCL:AnalogReceivePort | //SMLCL:AnalogReducePort">
 				<xsl:value-of select="@name"/>
                                <xsl:value-of select="translate($curr_pop/SMLLOWNL:Neuron/@name,' -','__')"/>
 				<!---->,<!---->
 			</xsl:for-each>
 			<xsl:apply-templates select="//SMLCL:Alias" mode="passPortAliases">
                                <xsl:with-param name="suffix" select="translate($curr_pop/SMLLOWNL:Neuron/@name,' -','__')"/>
 			</xsl:apply-templates>
 			<!--xsl:if test="count(//SMLCL:Regime)>1"-->
 				<!----> regimeCurr<!---->
                                <xsl:value-of select="translate($curr_pop/SMLLOWNL:Neuron/@name,' -','__')"/>
 			<!--/xsl:if-->
 			<!---->);
		</xsl:for-each>
	
	
		<!-- ZERO THE ANALOG INPUT PORTS -->
		// zero analog input ports
<!---->	<xsl:for-each select="$all_components">
  		<xsl:variable name="currComponent" select="."/>
    	    <xsl:variable name="componentName" select="../@name"/>
    	    <xsl:variable name="suffix_name">
				<xsl:value-of select="translate($componentName,' -','__')"/><!-- NEW SUFFIX -->
     	   </xsl:variable>
      	  <xsl:variable name="curr" select="."/>
      		  	<!-- FOR EACH ANALOGPORT IN THE COMPONENT THAT IS REFERENCED IN AN INPUT AS THE DST PORT -->
                                <xsl:for-each select="document($currComponent)//SMLCL:AnalogReducePort[@name=$curr/..//SMLLOWNL:Input/@dst_port] | document($currComponent)//SMLCL:AnalogReceivePort[@name=$curr/..//SMLLOWNL:Input/@dst_port]">
					<xsl:text>		</xsl:text>
					<xsl:value-of select="@name"/><xsl:value-of select="$suffix_name"/>[i] = 0;
<!---->			</xsl:for-each>
		</xsl:for-each>
		
		<!-- ZERO THE ANALOG INPUT PORTS FOR SYNAPSE -> PostSynapse -->
                <xsl:for-each select="$proj_inputs/SMLLOWNL:PostSynapse">
    	    <xsl:variable name="componentName" select="@name"/>
    	    <xsl:variable name="suffix">
 				<xsl:value-of select="translate($componentName,' -','__')"/><!-- NEW SUFFIX -->
     	   </xsl:variable>
  		  	<!-- FOR EACH ANALOGPORT FROM THE SYNAPSE -->
			<xsl:text>		</xsl:text>
			<xsl:value-of select="@input_dst_port"/><xsl:value-of select="$suffix"/>[i] = 0;
<!---->	</xsl:for-each>
	
	}
	
	<!-- DISENGAGE THE SEMAPHORES FOR ANALOG PORTS -->
	<xsl:for-each select="$all_components">
	// disengage semaphores:
<!--					 -->
	<xsl:variable name="currComponent" select="."/>
    <xsl:variable name="componentName" select="../@name"/>
        <xsl:if test="not(local-name(..)='WeightUpdate')">
            <xsl:for-each select="../SMLLOWNL:Input">
            	<xsl:variable name="suffix_name">
					<xsl:value-of select="translate($componentName,' -','__')"/><!-- NEW SUFFIX -->
					<!---->_input_<!---->
					<xsl:value-of select="position()"/>
				</xsl:variable>
                <xsl:variable name="dstPortName" select="@dst_port"/>
                <!-- DON'T DO CONDUCTANCE BASED SYNAPSE INPUTS -->
				<xsl:if test="not(local-name(..)='PostSynapse' and @src=../../../@dst_population and count(SMLNL:OneToOneConnection)=1)">
					<xsl:if test="count(document($currComponent)//SMLCL:AnalogReducePort[@name=$dstPortName] | document($currComponent)//SMLCL:AnalogReceivePort[@name=$dstPortName])=1">
<!---->					<!---->	signal(&amp;<xsl:value-of select="$suffix_name"/>_semaphore);
<!---->				</xsl:if>
				</xsl:if>
			</xsl:for-each>
		</xsl:if>
	</xsl:for-each>

	
	timerCounter += 1;
    if ( timerCounter >= <xsl:value-of select="number($expt_file//NMLEX:Simulation/@duration)*1000 div $expt_file//NMLEX:Simulation/*/@dt"/>)
    {
        exit(1);
    }
}

<!-- THREAD FUNCTIONS -->

//////////////////////////////////////
//									//
//			THREAD FUNCTIONS		//
//									//
//////////////////////////////////////

// synapses //////////////////////////
<xsl:for-each select="$proj_inputs">
	<!-- IF IT IS A SYNAPSE -->
        <xsl:apply-templates select="SMLLOWNL:WeightUpdate" mode="threadFuncs">
		<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
		<xsl:with-param name="split_model_file" select="$split_model_file"/>
		<xsl:with-param name="n_dst" select="$max_pop_size"/>
		<xsl:with-param name="n_src" select="$max_pop_size"/>
	</xsl:apply-templates>
</xsl:for-each>

// inputs ////////////////////////////
<!-- DO THE INPUTS FOR NEURONS AND PostSynapses (NO INPUTS FOR SYNAPSES CURRENTLY) -->
<xsl:apply-templates select="$proj_inputs/SMLLOWNL:PostSynapse | $curr_pop/SMLLOWNL:Neuron" mode="defineFullInputFuncs">
	<xsl:with-param name="split_model_file" select="$split_model_file"/>
	<xsl:with-param name="n_dst" select="$max_pop_size"/>
	<xsl:with-param name="n_src" select="$max_pop_size"/>
</xsl:apply-templates>

// DO EVENTS ##################################################

int event_int(int the_src_node_number, int the_port_and_Neuron_number, float the_packet_payload, int the_current_clock_tick) {

 int thread;

 // switch between inputs - are they synaptic, or inputs to other components?

 // for now we will only use 7-5 split of the 12 bits of port (7 for Neuron num, 5 for port ID)
 
 int the_port_number = 0xf&amp;(the_port_and_Neuron_number);
 int the_nrn_number = 0x7f&amp;(the_port_and_Neuron_number&gt;&gt;4);
  
 int switch_value = hashSwitch(the_src_node_number);
 int split_value = hashInputIndex(the_src_node_number);

 // startup:
 //if (nodes_to_see > 0 &amp;&amp; the_port_and_Neuron_number==0) {

	//printf("Nodes to see = %d\n", nodes_to_see);
        //nodes_to_see -= 1;
        //return 0;

 //}
 

<!-- SWITCH ON THE INPUT NUMBER (SYNAPSE OR GENERIC) -->
 // switch on input number
 switch (switch_value) {
        <xsl:for-each select="$all_components/../../SMLLOWNL:WeightUpdate/../../../SMLLOWNL:Neuron | //SMLLOWNL:Neuron[@name=$all_components/../SMLLOWNL:Input/@src]">
		<xsl:variable name="curr_nrn" select="."/>
		<!---->	case <xsl:value-of select="position()"/>:
<!---->	<!-- SWITCH ON THE PORT ON THE SRC -->
<!---->	<!---->		// switch on port on input src
<!---->	<!---->		switch (the_port_number) {
<!---->	<!--xsl:for-each select="document(@url)//SMLCL:AnalogSendPort[@name=$all_components/../../SMLLOWNL:WeightUpdate/@input_src_port | @name=$all_components/../SMLLOWNL:Input/@src_port] | document(@url)//SMLCL:EventSendPort[@name=$all_components/../../SMLLOWNL:WeightUpdate/@input_src_port | @name=$all_components/../SMLLOWNL:Input/@src_port]"-->
<!---->	<xsl:for-each select="document(@url)//SMLCL:AnalogSendPort | document(@url)//SMLCL:EventSendPort | document(@url)//SMLCL:ImpulseSendPort">
			<xsl:if test="local-name(.)='AnalogSendPort'">
				<xsl:variable name="curr_port" select="."/>
				<!---->			case <xsl:value-of select="position()-1"/>:
<!---->			<!---->				<!-- INPUTS FROM THIS PORT -->
<!---->			<!---->				// inputs
<!---->			<!---->				<xsl:for-each select="$all_components/../SMLLOWNL:Input[@src=$curr_nrn/@name and @src_port=$curr_port/@name]">
										<!-- GENERATE SUFFIX NAME -->
										<xsl:variable name="dst_suffix">
											<xsl:value-of select="translate(../@name,' -','__')"/><!-- NEW SUFFIX -->
											<!---->_input_<!---->
											<xsl:variable name="id" select="generate-id(.)"/>
                                                                                        <xsl:for-each select="../SMLLOWNL:Input">
												<xsl:if test="generate-id(.)=$id">
													<xsl:value-of select="position()"/>
												</xsl:if>
											</xsl:for-each>
										</xsl:variable>
										<xsl:if test="not(local-name(..)='PostSynapse' and @src=../../../@dst_population and count(SMLNL:OneToOneConnection)=1)">
<!---->			<!---->					thread = createthread(<xsl:value-of select="$dst_suffix"/>_schedule, 1000, the_nrn_number, split_value, the_packet_payload);
<!---->			<!---->					</xsl:if>
<!---->			<!---->				</xsl:for-each>
<!---->			<!---->				break;			
<!---->		</xsl:if>
			<xsl:if test="local-name(.)='EventSendPort'">
				<xsl:variable name="curr_port" select="."/>
				<!---->			case <xsl:value-of select="position()-1"/>:
<!---->			<!---->				<!-- PROJECTIONS FROM THIS PORT -->
<!---->			<!---->				// projections
<!---->			<!---->				<xsl:for-each select="$all_components/../../SMLLOWNL:WeightUpdate[../../../SMLLOWNL:Neuron/@name=$curr_nrn/@name and @input_src_port=$curr_port/@name]">
<!---->				<!---->				thread = createthread(<xsl:value-of select="translate(@name, ' -', 'oo')"/>_schedule, 1000, the_nrn_number, split_value);
<!---->			<!---->				</xsl:for-each>
<!---->			<!---->				<!-- INPUTS FROM THIS PORT -->
<!---->			<!---->				// inputs
<!---->			<!---->				<xsl:for-each select="$all_components/../SMLLOWNL:Input[@src=$curr_nrn/@name and @src_port=$curr_port/@name]">
										<!-- GENERATE SUFFIX NAME -->
										<xsl:variable name="dst_suffix">
											<xsl:value-of select="translate(../@name,' -','__')"/><!-- NEW SUFFIX -->
											<!---->_input_<!---->
											<xsl:variable name="id" select="generate-id(.)"/>
                                                                                        <xsl:for-each select="../SMLLOWNL:Input">
												<xsl:if test="generate-id(.)=$id">
													<xsl:value-of select="position()"/>
												</xsl:if>
											</xsl:for-each>
										</xsl:variable>
<!---->			<!---->					thread = createthread(<xsl:value-of select="$dst_suffix"/>_schedule, 1000, the_nrn_number, split_value);
<!---->			<!---->				</xsl:for-each>
<!---->			<!---->				break;			
<!---->		</xsl:if>	
			<xsl:if test="local-name(.)='ImpulseSendPort'">
				<xsl:variable name="curr_port" select="."/>
				<!---->			case <xsl:value-of select="position()-1"/>:
<!---->			<!---->				<!-- PROJECTIONS FROM THIS PORT -->
<!---->			<!---->				// projections
<!---->			<!---->				<xsl:for-each select="$all_components/../../SMLLOWNL:WeightUpdate[../../../SMLLOWNL:Neuron/@name=$curr_nrn/@name and @input_src_port=$curr_port/@name]">
<!---->				<!---->				thread = createthread(<xsl:value-of select="translate(@name, ' -', 'oo')"/>_schedule, 1000, the_nrn_number, split_value, the_packet_payload);
<!---->			<!---->				</xsl:for-each>
<!---->			<!---->				<!-- INPUTS FROM THIS PORT -->
<!---->			<!---->				// inputs
<!---->			<!---->				<xsl:for-each select="$all_components/../SMLLOWNL:Input[@src=$curr_nrn/@name and @src_port=$curr_port/@name]">
										<!-- GENERATE SUFFIX NAME -->
										<xsl:variable name="dst_suffix">
											<xsl:value-of select="translate(../@name,' -','__')"/><!-- NEW SUFFIX -->
											<!---->_input_<!---->
											<xsl:variable name="id" select="generate-id(.)"/>
                                                                                        <xsl:for-each select="../SMLLOWNL:Input">
												<xsl:if test="generate-id(.)=$id">
													<xsl:value-of select="position()"/>
												</xsl:if>
											</xsl:for-each>
										</xsl:variable>
<!---->			<!---->					thread = createthread(<xsl:value-of select="$dst_suffix"/>_schedule, 1000, the_nrn_number, split_value, the_packet_payload);
<!---->			<!---->				</xsl:for-each>
<!---->			<!---->				break;			
<!---->		</xsl:if>		
<!---->	</xsl:for-each>
<!---->	<!---->			default:
<!---->	<!---->				//printf("Warning - unhandled packet!\n"); //exit(-1);
<!---->	<!---->		}
<!---->	<!---->		break;		
	</xsl:for-each>
	default:
		//printf("Warning - unhandled packet!\n"); //exit(-1);
 }	
 return 1;
}



// FUNCTIONS FOR THE PostSynapse COMPONENTS
// impulse handlers for PS ////////////
<xsl:for-each select="$proj_inputs/SMLLOWNL:PostSynapse">
	<xsl:variable name="component" select="document(@url)"/>
	<xsl:variable name="dst_port" select="@input_dst_port"/>
	<xsl:variable name="suffix">
		<xsl:value-of select="translate(@name,' -','__')"/><!-- NEW SUFFIX -->
	</xsl:variable>
	<xsl:if test="count($component//SMLCL:OnImpulse[@src_port=$dst_port])=1">
		<!---->void HANDLE_IMPULSE_<xsl:value-of select="$suffix"/>_<xsl:value-of select="$dst_port"/>(int i, float <xsl:value-of select="$dst_port"/>)
<!---->{
			
<!---->	// define parameter handlers
		<xsl:for-each select="$component//SMLCL:Parameter">
			<!---->	float <xsl:value-of select="@name"/>;	
		</xsl:for-each>
		
<!---->	// define aliases that are not port aliases... 
		<xsl:apply-templates select="//SMLCL:Alias" mode="defineAlias">
			<xsl:with-param name="suffix" select="$suffix"/>
		</xsl:apply-templates>

<!---->	// assign non-fixed parameter handlers
		<xsl:for-each select="SMLNL:Property[count(SMLNL:FixedValue)=0 and @name=$component//SMLCL:Parameter/@name]">
			<!----> <xsl:value-of select="@name"/> = <xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>[i];
		</xsl:for-each>
<!---->	// assign fixed parameter handlers
		<xsl:for-each select="SMLNL:Property[count(SMLNL:FixedValue)=1 and @name=$component//SMLCL:Parameter/@name]">
			<!----> <xsl:value-of select="@name"/> = <xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>;
		</xsl:for-each>

<!---->	// assign aliases that are not port aliases... 
		<xsl:apply-templates select="//SMLCL:Alias" mode="doAll">
			<xsl:with-param name="suffix" select="$suffix"/>
		</xsl:apply-templates>
		
<!---->	// do state assignments
		<xsl:apply-templates select="$component//SMLCL:OnImpulse[@src_port=$dst_port]" mode="doSA">
			<xsl:with-param name="suffix" select="$suffix"/>
		</xsl:apply-templates>
			
<!---->}
	</xsl:if>
</xsl:for-each>

// main functions for PS //////////////
<xsl:for-each select="document($proj_inputs/SMLLOWNL:PostSynapse/@url)">
<!---->// <xsl:value-of select=".//SMLCL:ComponentClass/@name"/> //
<!----><!-- FUNCTION DEFINITION -->
<!---->void <xsl:value-of select="translate(.//SMLCL:ComponentClass/@name,' -', '__')"/>
	<!---->(int i,<!---->
	<xsl:for-each select="//SMLCL:Parameter">
		<!---->float <!---->
		<xsl:value-of select="@name"/>
		<!---->,<!---->
	</xsl:for-each>
	<xsl:for-each select="//SMLCL:StateVariable | //SMLCL:AnalogReceivePort | //SMLCL:AnalogReducePort">
		<!---->float <!---->
		<xsl:value-of select="@name"/>
		<!---->INST[<!---->
		<xsl:value-of select="$max_pop_size"/>
		<!---->],<!---->
	</xsl:for-each>
	<xsl:apply-templates select="//SMLCL:Alias" mode="passPortAliases">
		<xsl:with-param name="suffix" select="concat(concat('INST[', $max_pop_size),']')"/>
	</xsl:apply-templates>
	<!---->int regimeINST[<!---->
	<xsl:value-of select="$max_pop_size"/>
	<!---->]) {

<!---->	int regimeNext;
<!---->	int transition;

<!---->	// define temp variables for time derivatives 
<!---->	<xsl:apply-templates select="//SMLCL:StateVariable" mode="defineTempVars"/>

<!---->	// define aliases that are not port aliases... 
		<xsl:apply-templates select="//SMLCL:Alias" mode="defineAlias">
			<xsl:with-param name="suffix" select="'INST'"/>
		</xsl:apply-templates>

<!---->	// assign aliases that are not port aliases... 
		<xsl:apply-templates select="//SMLCL:Alias" mode="doAll">
			<xsl:with-param name="suffix" select="'INST'"/>
		</xsl:apply-templates>			
			
		<!-- DO THE DYNAMICS OF THE MODEL - USES HIERARCHICAL SpineML TEMPLATES -->
		<xsl:apply-templates select="//SMLCL:ComponentClass" mode="doIter">
			<xsl:with-param name="suffix" select="'INST'"/>
		</xsl:apply-templates>
		<!-- UPDATE PORT ALIASES FOR OUTPUT - USES HIERARCHICAL SpineML TEMPLATES -->
		<xsl:apply-templates select="//SMLCL:ComponentClass" mode="doPortAlias">
			<xsl:with-param name="suffix" select="'INST'"/>
		</xsl:apply-templates>
		<!-- ZERO ANALOG INPUTS -->
<!---->	// zero analog ports
<!---->	<xsl:for-each select="//SMLCL:AnalogReceivePort | //SMLCL:AnalogReducePort">
			<xsl:text>	</xsl:text>
			<xsl:value-of select="@name"/><xsl:value-of select="'INST'"/>[i] = 0;
<!---->	</xsl:for-each>

<!---->}
</xsl:for-each>

// FUNCTION FOR THE NEURON BODY COMPONENT
<xsl:for-each select="document(./SMLLOWNL:Neuron/@url)">
<!---->// <xsl:value-of select=".//SMLCL:ComponentClass/@name"/> //
<!----><!-- FUNCTION DEFINITION -->
<!---->void <xsl:value-of select="translate(.//SMLCL:ComponentClass/@name,' -', '__')"/>
	<!---->(int i,<!---->
	<xsl:for-each select="//SMLCL:Parameter">
		<!---->float <!---->
		<xsl:value-of select="@name"/>
		<!---->,<!---->
	</xsl:for-each>
	<xsl:for-each select="//SMLCL:StateVariable | //SMLCL:AnalogReceivePort | //SMLCL:AnalogReducePort">
		<!---->float <!---->
		<xsl:value-of select="@name"/>
		<!---->INST[<!---->
		<xsl:value-of select="$max_pop_size"/>
		<!---->],<!---->
	</xsl:for-each>
	<xsl:apply-templates select="//SMLCL:Alias" mode="passPortAliases">
		<xsl:with-param name="suffix" select="concat(concat('INST[', $max_pop_size),']')"/>
	</xsl:apply-templates>
	<!---->int regimeINST[<!---->
	<xsl:value-of select="$max_pop_size"/>
	<!---->]) {
<!---->
<!---->	int j;
<!---->	int portID;
<!---->	int regimeNext;
<!---->	int transition;

<!---->	// define temp variables for time derivatives 
<!---->	<xsl:apply-templates select="//SMLCL:StateVariable" mode="defineTempVars"/>

<!---->	// define aliases that are not port aliases... 
		<xsl:apply-templates select="//SMLCL:Alias" mode="defineAlias">
			<xsl:with-param name="suffix" select="'INST'"/>
		</xsl:apply-templates>

<!---->	// assign aliases that are not port aliases... 
		<xsl:apply-templates select="//SMLCL:Alias" mode="doAll">
			<xsl:with-param name="suffix" select="'INST'"/>
		</xsl:apply-templates>
		
		<!-- DO THE DYNAMICS OF THE MODEL - USES HIERARCHICAL SpineML TEMPLATES -->
		<xsl:apply-templates select="//SMLCL:ComponentClass" mode="doIter">
			<xsl:with-param name="suffix" select="'INST'"/>
                        <xsl:with-param name="src" select="$curr_pop/SMLLOWNL:Neuron"/>
		</xsl:apply-templates>
		<!-- UPDATE PORT ALIASES FOR OUTPUT - USES HIERARCHICAL SpineML TEMPLATES -->
		<xsl:apply-templates select="//SMLCL:ComponentClass" mode="doPortAlias">
			<xsl:with-param name="suffix" select="'INST'"/>
		</xsl:apply-templates>

<!---->	// output any analog ports
<!---->	<!-- OUTPUT ANALOG PORTS - USES HIERARCHICAL SpineML TEMPLATES -->
		<xsl:for-each select="//SMLCL:ComponentClass//SMLCL:AnalogSendPort | //SMLCL:ComponentClass//SMLNL:EventSendPort ">
			<xsl:variable name="index" select="number(position())-1"/>
			<xsl:apply-templates select="." mode="doOutput">
				<xsl:with-param name="suffix" select="'INST'"/>
                                <xsl:with-param name="size" select="concat('N_', translate(//SMLCL:ComponentClass/@name, ' -', 'oo'))"/>
				<xsl:with-param name="index" select="$index"/>
			</xsl:apply-templates>
		</xsl:for-each>
		<!-- ZERO ANALOG INPUTS -->
<!---->	// zero analog ports
<!---->	<xsl:for-each select="//SMLCL:AnalogReceivePort | //SMLCL:AnalogReducePort">
			<xsl:text>	</xsl:text>
			<xsl:value-of select="@name"/><xsl:value-of select="'INST'"/>[i] = 0;
<!---->	</xsl:for-each>

<!---->}
</xsl:for-each>


<!--</xsl:if> = for doing the nodes one by one, rather than in a long list -->


</xsl:for-each> <!-- end of nodes -->


<!--DO ALIASES-->

/// ############### ALIASES #############

#include "data.d"


</xsl:for-each> <!-- END FOR-EACH FOR MODEL XML FILE -->

</xsl:template>

<xsl:include href="SpineML_Component.xsl"/>
<xsl:include href="SpineML_Dynamics.xsl"/>
<xsl:include href="SpineML_Regime.xsl"/>
<xsl:include href="SpineML_StateVariable.xsl"/>
<xsl:include href="SpineML_Parameter.xsl"/>
<xsl:include href="SpineML_AnalogPort.xsl"/>
<xsl:include href="SpineML_EventPort.xsl"/>
<xsl:include href="SpineML_Alias.xsl"/>
<xsl:include href="SpineML_Input.xsl"/>
<xsl:include href="SpineML_Property.xsl"/>
<xsl:include href="SpineML_WeightUpdate.xsl"/>
<xsl:include href="SpineML_Postsynapse.xsl"/>
<xsl:include href="SpineML_Neuron.xsl"/>

</xsl:stylesheet>


