<?xml version="1.0" encoding="ISO-8859-1"?><xsl:stylesheet version="1.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:SMLNL="http://www.shef.ac.uk/SpineMLNetworkLayer" xmlns:SMLLOWNL="http://www.shef.ac.uk/SpineMLLowLevelNetworkLayer" xmlns:SMLCL="http://www.shef.ac.uk/SpineMLComponentLayer"xmlns:fn="http://www.w3.org/2005/xpath-functions">
<xsl:output method="text" version="1.0" encoding="UTF-8" indent="yes"/>

<!-- ####################################################
#														#
#			PostSynapse INPUT PROTOTYPES						#
#														#
######################################################-->

<xsl:template match="SMLLOWNL:Input" mode="definePostSynapseInput">
	<xsl:param name="split_model_file"/>
	<xsl:param name="n_dst"/>
	<xsl:param name="n_src"/>

	<!-- DON'T DO CONDUCTANCE BASED SYNAPSE INPUTS -->
	<xsl:if test="not(local-name(..)='PostSynapse' and @src=../../../@dst_population and count(SMLNL:OneToOneConnection)=1)">

	<!-- FOR NOW JUST FIND THE NUMBER OF SPLITS IN THE SOURCE -->
	<xsl:variable name="num_split_src_pops">
		<xsl:variable name="src" select="@src"/>
		<xsl:value-of select="count($split_model_file//SMLLOWNL:Population/SMLLOWNL:Neuron[substring-before(@name,'_sub')=$src])"/>
	</xsl:variable>
	
	<!-- GET THE CONNECTION TYPE (EVENT OR ANALOG) -->
	<xsl:variable name="conn_type">
		<xsl:variable name="dst_port" select="@dst_port"/>
		<xsl:if test="count(document(../@url)//SMLCL:EventReceivePort[@name=$dst_port])=1">
			<!---->EVENT<!---->
		</xsl:if>
		<xsl:if test="count(document(../@url)//SMLCL:AnalogReceivePort[@name=$dst_port] | document(../@url)//SMLCL:AnalogReducePort[@name=$dst_port])=1">
			<!---->ANALOG<!---->
		</xsl:if>
	</xsl:variable>
	
	<!-- GENERATE SUFFIX NAME -->
	<xsl:variable name="suffix">
		<xsl:value-of select="translate(../@name,' -','__')"/><!-- NEW SUFFIX -->
		<!---->_input_<!---->
		<xsl:variable name="id" select="generate-id(.)"/>
		<xsl:for-each select="../SMLLOWNL:Input">
			<xsl:if test="generate-id(.)=$id">
				<xsl:value-of select="position()"/>
			</xsl:if>
		</xsl:for-each>
	</xsl:variable>
	
	
<!---->// input to <xsl:value-of select="../@name"/>: <xsl:value-of select="$suffix"/> ///////////////

	<!-- HANDLE THE DIFFERENT CONNECTION TYPES -->
	<!-- EVENT      CONNECTION TYPE ############################################ -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->
	<xsl:if test="$conn_type='EVENT'">
		<xsl:choose>
			<!-- ONE TO ONE CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:OneToOneConnection)=1">
				<!-- ONE TO ONES DO NOT NEED A CONNECTION -->
				<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
					<!---->// delay
<!---->				<!---->float delay<xsl:value-of select="$suffix"/> = <xsl:value-of select=".//SMLNL:FixedValue/@value"/>;
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int nrn_index, int split_index);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
					<!---->// delay 
<!---->				<!---->float delay<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_dst"/>];
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE EXPLICIT (THIS IS NOT PERMITTED BECAUSE IT IS SILLY) -->
				<xsl:if test="count(.//@delay)>0">
					<!---->!!! Error - Non-Explicit List inputs with Explicit List delays are not supported (<xsl:value-of select="../@name"/>) !!!
<!---->			</xsl:if>
			</xsl:when>
			<!-- ALL TO ALL CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->	
			<xsl:when test="count(SMLNL:AllToAllConnection)=1">
				<!-- ALL TO ALLS DO NOT NEED EXTERNAL CONNECTIVITY PATTERNS SO NONE HERE -->
				<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
					<!---->// delay
<!---->				<!---->float delay<xsl:value-of select="$suffix"/> = <xsl:value-of select=".//SMLNL:FixedValue/@value"/>;
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
					<!---->// delay 
<!---->				<!---->extern float delaySD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>];
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE EXPLICIT (THIS IS NOT PERMITTED BECAUSE IT IS SILLY) -->
				<xsl:if test="count(.//@delay)>0">
					!!! Error - Non-Explicit List inputs with Explicit List delays are not supported (<xsl:value-of select="../@name"/>) !!!
				</xsl:if>
			</xsl:when>	
			<!-- EXPLICIT LIST CONNECTION TYPE ######################################### -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:ConnectionList)=1">
				<!---->//connection
<!---->			<!---->extern int connectionSD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>];
<!---->			<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
					<!---->// delay
<!---->				<!---->float delay<xsl:value-of select="$suffix"/> = <xsl:value-of select=".//SMLNL:FixedValue/@value"/>;
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
					<!---->// delay 
<!---->				<!---->extern float delaySD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>];
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE EXPLICIT -->
				<xsl:if test="count(.//@delay)>0">
					<!---->// delay 
<!---->				<!---->extern float delaySD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>];
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index);
<!---->			</xsl:if>
			</xsl:when>
			<!-- FIXED PROB CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:FixedProbabilityConnection)=1">
				<!---->//connection
<!---->			<!---->extern int connectionSD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/><!-- p would need to be very low for there to not be at least 1 connection from each src split pop -->][<xsl:value-of select="$n_src"/>][<xsl:value-of select="ceiling(6.0*number($n_dst)*number(SMLNL:FixedProbabilityConnection/@probability))"/>];
<!---->			<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
					<!---->// delay
<!---->				<!---->float delay<xsl:value-of select="$suffix"/> = <xsl:value-of select=".//SMLNL:FixedValue/@value"/>;
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
					<!---->// delay 
<!---->				<!---->extern float delaySD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="ceiling(6.0*number($n_dst)*number(SMLNL:FixedProbabilityConnection/@probability))"/>];
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE EXPLICIT (THIS IS NOT PERMITTED BECAUSE IT IS SILLY) -->
				<xsl:if test="count(.//@delay)>0">
					<!---->!!! Error - Non-Explicit List inputs with Explicit List delays are not supported (<xsl:value-of select="../@name"/>) !!!
<!---->			</xsl:if>
			</xsl:when>
			<xsl:otherwise>
				<!---->!!! Error - connection type not recognised (<xsl:value-of select="../@name"/>, src=<xsl:value-of select="@src"/>, src_port=<xsl:value-of select="@src_port"/>, dst_port=<xsl:value-of select="@dst_port"/>)
<!---->		</xsl:otherwise>
		</xsl:choose>
	</xsl:if>
	
	<!-- ANALOG      CONNECTION TYPE ########################################### -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->	
	<xsl:if test="$conn_type='ANALOG'">
	<!-- CREATE SEMAPHORE VARIABLE -->
	<!---->//semaphore:
<!--	-->int <xsl:value-of select="$suffix"/>_semaphore = 1;
<!---->	<xsl:choose>
			<!-- ONE TO ONE CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:OneToOneConnection)=1">
				<!-- ONE TO ONES DO NOT NEED A CONNECTION -->
				<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
					<!---->// delay (121 fixed)
<!---->				<!---->float delay<xsl:value-of select="$suffix"/> = <xsl:value-of select=".//SMLNL:FixedValue/@value"/>;
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index, float value);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
					<!---->// delay (121 prob)
<!---->				<!---->float delay<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_dst"/>];
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index, float value);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE EXPLICIT (THIS IS NOT PERMITTED BECAUSE IT IS SILLY) -->
				<xsl:if test="count(.//@delay)>0">
					<!---->!!! Error - Non-Explicit List inputs with Explicit List delays are not supported (<xsl:value-of select="../@name"/>) !!!
<!---->			</xsl:if>
			</xsl:when>
			<!-- ALL TO ALL CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->	
			<xsl:when test="count(SMLNL:AllToAllConnection)=1">
				<!-- ALL TO ALLS DO NOT NEED EXTERNAL CONNECTIVITY PATTERNS SO NONE HERE -->
				<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
					<!---->// delay (A2A fixed)
<!---->				<!---->float delay<xsl:value-of select="$suffix"/> = <xsl:value-of select=".//SMLNL:FixedValue/@value"/>; 
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index, float value);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
					<!---->// delay (A2A prob)
<!---->				<!---->extern float delaySD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>];
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index, float value);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE EXPLICIT (THIS IS NOT PERMITTED BECAUSE IT IS SILLY) -->
				<xsl:if test="count(.//@delay)>0">
					<!---->!!! Error - Non-Explicit List inputs with Explicit List delays are not supported (<xsl:value-of select="../@name"/>) !!!
<!---->			</xsl:if>
			</xsl:when>	
			<!-- EXPLICIT LIST CONNECTION TYPE ######################################### -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:ConnectionList)=1">
				<!---->//connection
<!---->			<!---->extern int connectionSD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>];
<!---->			<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
					<!---->// delay (el fixed)
<!---->				<!---->float delay<xsl:value-of select="$suffix"/> = <xsl:value-of select=".//SMLNL:FixedValue/@value"/>;
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->	// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index, float value);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
					<!---->// delay (el prob)
<!---->				<!---->extern float delaySD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>];
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index, float value);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE EXPLICIT -->
				<xsl:if test="count(.//@delay)>0">
					<!---->// delay (el expl)
<!---->				<!---->extern float delaySD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>];
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay, float value);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index, float value);
<!---->			</xsl:if>
			</xsl:when>
			<!-- FIXED PROB CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:FixedProbabilityConnection)=1">
				<!---->//connection
<!---->			<!---->extern int connectionSD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/><!-- p would need to be very low for there to not be at least 1 connection from each src split pop -->][<xsl:value-of select="$n_src"/>][<xsl:value-of select="ceiling(6.0*number($n_dst)*number(SMLNL:FixedProbabilityConnection/@probability))"/>];
<!---->			<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
					<!---->// delay (fp fixed)
<!---->				<!---->float delay<xsl:value-of select="$suffix"/> = <xsl:value-of select=".//SMLNL:FixedValue/@value"/>;
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index, float value);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
					<!---->// delay (fp prob)
<!---->				<!---->extern float delaySD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="ceiling(6.0*number($n_dst)*number(SMLNL:FixedProbabilityConnection/@probability))"/>];
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index, float value);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE EXPLICIT (THIS IS NOT PERMITTED BECAUSE IT IS SILLY) -->
				<xsl:if test="count(.//@delay)>0">
				<!---->!!! Error - Non-Explicit List inputs with Explicit List delays are not supported (<xsl:value-of select="../@name"/>) !!!
<!---->			</xsl:if>
			</xsl:when>
			<xsl:otherwise>
				<!---->!!! Error - connection type not recognised (<xsl:value-of select="../@name"/>, src=<xsl:value-of select="@src"/>, src_port=<xsl:value-of select="@src_port"/>, dst_port=<xsl:value-of select="@dst_port"/>)
<!---->		</xsl:otherwise>
		</xsl:choose>
	</xsl:if>	
	
	<!-- /DON'T DO CONDUCTANCE BASED SYNAPSE INPUTS -->
	</xsl:if>
	
</xsl:template>

<!-- ####################################################
#														#
#			NEURON INPUT PROTOTYPES						#
#														#
######################################################-->

<xsl:template match="SMLLOWNL:Input" mode="defineNRNInput">
	<xsl:param name="split_model_file"/>
	<xsl:param name="n_dst"/>
	<xsl:param name="n_src"/>

	<!-- FOR NOW JUST FIND THE NUMBER OF SPLITS IN THE SOURCE -->
	<xsl:variable name="num_split_src_pops">
		<xsl:variable name="src" select="@src"/>
		<xsl:value-of select="count($split_model_file//SMLLOWNL:Population/SMLLOWNL:Neuron[substring-before(@name,'_sub')=$src])"/>
	</xsl:variable>
	
	<!-- GET THE CONNECTION TYPE (EVENT OR ANALOG) -->
	<xsl:variable name="conn_type">
		<xsl:variable name="dst_port" select="@dst_port"/>		
		<xsl:if test="count(document(../@url)//SMLCL:EventReceivePort[@name=$dst_port])=1">
			<!---->EVENT<!---->
		</xsl:if>
		<xsl:if test="count(document(../@url)//SMLCL:AnalogReceivePort[@name=$dst_port] | document(../@url)//SMLCL:AnalogReducePort[@name=$dst_port])=1">
			<!---->ANALOG<!---->
		</xsl:if>
	</xsl:variable>
	
	<!-- GENERATE SUFFIX NAME -->
	<xsl:variable name="suffix">
		<xsl:value-of select="translate(../@name,' -','__')"/><!-- NEW SUFFIX -->
		<!---->_input_<!---->
		<xsl:variable name="id" select="generate-id(.)"/>
		<xsl:for-each select="../SMLLOWNL:Input">
			<xsl:if test="generate-id(.)=$id">
				<xsl:value-of select="position()"/>
			</xsl:if>
		</xsl:for-each>
	</xsl:variable>
<!---->	
// input to <xsl:value-of select="../@name"/>: <xsl:value-of select="$suffix"/> ///////////////
<!---->
	<!-- HANDLE THE DIFFERENT CONNECTION TYPES -->
	<!-- EVENT      CONNECTION TYPE ############################################ -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->
	<xsl:if test="$conn_type='EVENT'">
		<xsl:choose>
			<!-- ONE TO ONE CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:OneToOneConnection)=1">
				<!-- ONE TO ONES DO NOT NEED A CONNECTION -->
				<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
					<!---->// delay
<!---->				<!---->float delay<xsl:value-of select="$suffix"/> = <xsl:value-of select=".//SMLNL:FixedValue/@value"/>;
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int nrn_index);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
					<!---->// delay 
<!---->				<!---->float delay<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_dst"/>];
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE EXPLICIT (THIS IS NOT PERMITTED BECAUSE IT IS SILLY) -->
				<xsl:if test="count(.//@delay)>0">
					<!---->!!! Error - Non-Explicit List inputs with Explicit List delays are not supported (<xsl:value-of select="../@name"/>) !!!
<!---->			</xsl:if>
			</xsl:when>
			<!-- ALL TO ALL CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->	
			<xsl:when test="count(SMLNL:AllToAllConnection)=1">
				<!-- ALL TO ALLS DO NOT NEED EXTERNAL CONNECTIVITY PATTERNS SO NONE HERE -->
				<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
					<!---->// delay
<!---->				<!---->float delay<xsl:value-of select="$suffix"/> = <xsl:value-of select=".//SMLNL:FixedValue/@value"/>;
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
					<!---->// delay 
<!---->				<!---->extern float delaySD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>];
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE EXPLICIT (THIS IS NOT PERMITTED BECAUSE IT IS SILLY) -->
				<xsl:if test="count(.//@delay)>0">
					!!! Error - Non-Explicit List inputs with Explicit List delays are not supported (<xsl:value-of select="../@name"/>) !!!
				</xsl:if>
			</xsl:when>	
			<!-- EXPLICIT LIST CONNECTION TYPE ######################################### -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:ConnectionList)=1">
				<!---->//connection
<!---->			<!---->extern int connectionSD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>];
<!---->			<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
					<!---->// delay
<!---->				<!---->float delay<xsl:value-of select="$suffix"/> = <xsl:value-of select=".//SMLNL:FixedValue/@value"/>;
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
					<!---->// delay 
<!---->				<!---->extern float delaySD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>];
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE EXPLICIT -->
				<xsl:if test="count(.//@delay)>0">
					<!---->// delay 
<!---->				<!---->extern float delaySD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>];
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index);
<!---->			</xsl:if>
			</xsl:when>
			<!-- FIXED PROB CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:FixedProbabilityConnection)=1">
				<!---->//connection
<!---->			<!---->extern int connectionSD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/><!-- p would need to be very low for there to not be at least 1 connection from each src split pop -->][<xsl:value-of select="$n_src"/>][<xsl:value-of select="ceiling(6.0*number($n_dst)*number(SMLNL:FixedProbabilityConnection/@probability))"/>];
<!---->			<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
					<!---->// delay
<!---->				<!---->float delay<xsl:value-of select="$suffix"/> = <xsl:value-of select=".//SMLNL:FixedValue/@value"/>;
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
					<!---->// delay 
<!---->				<!---->extern float delaySD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="ceiling(6.0*number($n_dst)*number(SMLNL:FixedProbabilityConnection/@probability))"/>];
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE EXPLICIT (THIS IS NOT PERMITTED BECAUSE IT IS SILLY) -->
				<xsl:if test="count(.//@delay)>0">
					<!---->!!! Error - Non-Explicit List inputs with Explicit List delays are not supported (<xsl:value-of select="../@name"/>) !!!
<!---->			</xsl:if>
			</xsl:when>
			<xsl:otherwise>
				<!---->!!! Error - connection type not recognised (<xsl:value-of select="../@name"/>, src=<xsl:value-of select="@src"/>, src_port=<xsl:value-of select="@src_port"/>, dst_port=<xsl:value-of select="@dst_port"/>)
<!---->		</xsl:otherwise>
		</xsl:choose>
	</xsl:if>
	
	<!-- ANALOG      CONNECTION TYPE ########################################### -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->	
	<xsl:if test="$conn_type='ANALOG'">
	<!-- CREATE SEMAPHORE VARIABLE -->
	<!---->//semaphore:
<!--	-->int <xsl:value-of select="$suffix"/>_semaphore = 1;
<!---->	<xsl:choose>
			<!-- ONE TO ONE CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:OneToOneConnection)=1">
				<!-- ONE TO ONES DO NOT NEED A CONNECTION -->
				<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
					<!---->// delay (121 fixed)
<!---->				<!---->float delay<xsl:value-of select="$suffix"/> = <xsl:value-of select=".//SMLNL:FixedValue/@value"/>;
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index, float value);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
					<!---->// delay (121 prob)
<!---->				<!---->float delay<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_dst"/>];
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index, float value);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE EXPLICIT (THIS IS NOT PERMITTED BECAUSE IT IS SILLY) -->
				<xsl:if test="count(.//@delay)>0">
					<!---->!!! Error - Non-Explicit List inputs with Explicit List delays are not supported (<xsl:value-of select="../@name"/>) !!!
<!---->			</xsl:if>
			</xsl:when>
			<!-- ALL TO ALL CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->	
			<xsl:when test="count(SMLNL:AllToAllConnection)=1">
				<!-- ALL TO ALLS DO NOT NEED EXTERNAL CONNECTIVITY PATTERNS SO NONE HERE -->
				<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
					<!---->// delay (A2A fixed)
<!---->				<!---->float delay<xsl:value-of select="$suffix"/> = <xsl:value-of select=".//SMLNL:FixedValue/@value"/>; 
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index, float value);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
					<!---->// delay (A2A prob)
<!---->				<!---->extern float delaySD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>];
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index, float value);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE EXPLICIT (THIS IS NOT PERMITTED BECAUSE IT IS SILLY) -->
				<xsl:if test="count(.//@delay)>0">
					<!---->!!! Error - Non-Explicit List inputs with Explicit List delays are not supported (<xsl:value-of select="../@name"/>) !!!
<!---->			</xsl:if>
			</xsl:when>	
			<!-- EXPLICIT LIST CONNECTION TYPE ######################################### -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:ConnectionList)=1">
				<!---->//connection
<!---->			<!---->extern int connectionSD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>];
<!---->			<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
					<!---->// delay (el fixed)
<!---->				<!---->float delay<xsl:value-of select="$suffix"/> = <xsl:value-of select=".//SMLNL:FixedValue/@value"/>;
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->	// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int split_index, float value);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index, float value);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
					<!---->// delay (el prob)
<!---->				<!---->extern float delaySD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>];
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index, float value);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE EXPLICIT -->
				<xsl:if test="count(.//@delay)>0">
					<!---->// delay (el expl)
<!---->				<!---->extern float delaySD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>];
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay, float value);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index, float value);
<!---->			</xsl:if>
			</xsl:when>
			<!-- FIXED PROB CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:FixedProbabilityConnection)=1">
				<!---->//connection
<!---->			<!---->extern int connectionSD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/><!-- p would need to be very low for there to not be at least 1 connection from each src split pop -->][<xsl:value-of select="$n_src"/>][<xsl:value-of select="ceiling(6.0*number($n_dst)*number(SMLNL:FixedProbabilityConnection/@probability))"/>];
<!---->			<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
					<!---->// delay (fp fixed)
<!---->				<!---->float delay<xsl:value-of select="$suffix"/> = <xsl:value-of select=".//SMLNL:FixedValue/@value"/>;
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index, float value);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
					<!---->// delay (fp prob)
<!---->				<!---->extern float delaySD<xsl:value-of select="$suffix"/>[<xsl:value-of select="$num_split_src_pops"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="ceiling(6.0*number($n_dst)*number(SMLNL:FixedProbabilityConnection/@probability))"/>];
<!---->				<!-- THREAD FUNCTION PROTOTYPES -->	
					<!---->// prototypes for thread functions
<!---->				<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay);
<!---->				<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index, float value);
<!---->			</xsl:if>
				<!-- IF DELAYS ARE EXPLICIT (THIS IS NOT PERMITTED BECAUSE IT IS SILLY) -->
				<xsl:if test="count(.//@delay)>0">
				<!---->!!! Error - Non-Explicit List inputs with Explicit List delays are not supported (<xsl:value-of select="../@name"/>) !!!
<!---->			</xsl:if>
			</xsl:when>
			<xsl:otherwise>
				<!---->!!! Error - connection type not recognised (<xsl:value-of select="../@name"/>, src=<xsl:value-of select="@src"/>, src_port=<xsl:value-of select="@src_port"/>, dst_port=<xsl:value-of select="@dst_port"/>)
<!---->		</xsl:otherwise>
		</xsl:choose>
	</xsl:if>	
</xsl:template>


<!-- ####################################################
#														#
#			PostSynapse & NRN INPUT INITIALISATION				#
#														#
######################################################-->

<xsl:template match="SMLLOWNL:Input" mode="initInputData">
	<xsl:param name="split_model_file"/>
	<xsl:param name="n_dst"/>
	<xsl:param name="n_src"/>
	
	<!-- DON'T DO CONDUCTANCE BASED SYNAPSE INPUTS -->
	<xsl:if test="not(local-name(..)='PostSynapse' and @src=../../../@dst_population and count(SMLNL:OneToOneConnection)=1)">
	
<!---->
	<!-- FOR NOW JUST FIND THE NUMBER OF SPLITS IN THE SOURCE -->
	<xsl:variable name="num_split_src_pops">
		<xsl:variable name="src" select="@src"/>
		<xsl:value-of select="count($split_model_file//SMLLOWNL:Population/SMLLOWNL:Neuron[substring-before(@name,'_sub')=$src])"/>
	</xsl:variable>
	
	<!-- GET THE CONNECTION TYPE (EVENT OR ANALOG) -->
	<xsl:variable name="conn_type">
		<xsl:variable name="dst_port" select="@dst_port"/>
		<xsl:if test="count(document(../@url)//SMLCL:EventReceivePort[@name=$dst_port])=1">
			<!---->EVENT<!---->
		</xsl:if>
		<xsl:if test="count(document(../@url)//SMLCL:AnalogReceivePort[@name=$dst_port] | document(../@url)//SMLCL:AnalogReducePort[@name=$dst_port])=1">
			<!---->ANALOG<!---->
		</xsl:if>
	</xsl:variable>
	
	<!-- GENERATE SUFFIX NAME -->
	<xsl:variable name="suffix">
		<xsl:value-of select="translate(../@name,' -','__')"/><!-- NEW SUFFIX -->
		<!---->_input_<!---->
		<xsl:variable name="id" select="generate-id(.)"/>
		<xsl:for-each select="../SMLLOWNL:Input">
			<xsl:if test="generate-id(.)=$id">
				<xsl:value-of select="position()"/>
			</xsl:if>
		</xsl:for-each>
	</xsl:variable>
	
<!---->// input to <xsl:value-of select="../@name"/>: <xsl:value-of select="$suffix"/> ///////////////

<!---->
	<!-- HANDLE THE DIFFERENT CONNECTION TYPES -->
	<!-- EVENT      CONNECTION TYPE ############################################ -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->	
	<xsl:if test="$conn_type='EVENT'">
		<xsl:choose>
			<!-- ONE TO ONE CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:OneToOneConnection)=1">
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:for-each select=".//SMLNL:UniformDistribution">
					<!---->	// delay 
<!---->				<!---->	for (i=0; i &lt; <xsl:value-of select="$num_split_src_pops"/>; i += 1) {
<!---->				<!---->		for (j=0; j &lt; <xsl:value-of select="$n_dst"/>; j += 1) {
<!---->				<!---->			delay<xsl:value-of select="$suffix"/>[i][j] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->				<!---->		}
<!---->				<!---->	}
<!---->			</xsl:for-each>
			</xsl:when>
			<!-- ALL TO ALL CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->	
			<xsl:when test="count(SMLNL:AllToAllConnection)=1">
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:for-each select=".//SMLNL:UniformDistribution">
					<!---->	// delay 
<!---->				<!---->	for (i=0; i &lt; <xsl:value-of select="$num_split_src_pops"/>; i += 1) {
<!---->				<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->				<!---->			for (k=0; k &lt; <xsl:value-of select="$n_dst"/>; k += 1) {
<!---->				<!---->				tempFloats[k] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->				<!---->			}
<!---->				<!---->			writesdram(&amp;delay<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempFloats, <xsl:value-of select="$n_dst"/>);
<!---->				<!---->		}
<!---->				<!---->	}
<!---->			</xsl:for-each>
			</xsl:when>	
			<!-- EXPLICIT LIST CONNECTION TYPE ######################################### -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:ConnectionList)=1">
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:for-each select=".//SMLNL:UniformDistribution">
					<!---->	// delay 
<!---->				<!---->	for (i=0; i &lt; <xsl:value-of select="$num_split_src_pops"/>; i += 1) {
<!---->				<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->				<!---->			for (k=0; k &lt; <xsl:value-of select="$n_dst"/>; k += 1) {
<!---->				<!---->				tempFloats[k] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->				<!---->			}
<!---->				<!---->			writesdram(&amp;delay<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempFloats, <xsl:value-of select="$n_dst"/>);
<!---->				<!---->		}
<!---->				<!---->	}
<!---->			</xsl:for-each>
			</xsl:when>
			<!-- FIXED PROB CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:FixedProbabilityConnection)=1">
				<!-- THIS ONE IS A BIT DIFFERENT, AS WE NEED TO FILL IN ALL THE SD 
				RAM ARRAYS WITH THE NUMBER OF DELAYS GENERATED FOR THE CONNECTION -->
				<!-- CONNECTIVITY -->
				<!---->	// connectivity
<!---->				<!---->	for (i=0; i &lt; <xsl:value-of select="$num_split_src_pops"/>; i += 1) {
<!---->				<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->				<!---->			counter = 0;
<!---->				<!---->			for (k=0; k &lt; <xsl:value-of select="ceiling(6.0*number($n_dst)*number(SMLNL:FixedProbabilityConnection/@probability))"/>; k += 1) {
<!---->				<!---->				if (uniformRNG(1.0, 0.0) &lt; <xsl:value-of select="SMLNL:FixedProbabilityConnection/@probability"/>) {	
<!---->				<!---->					tempInts[counter] = k;
<!---->				<!---->					counter += 1;
<!---->				<!---->				}
<!---->				<!---->			}
<!---->				<!---->			connRowLengths[j] = counter; // for use in pars init
<!---->				<!---->			tempInts[counter+1] = -1; // terminate list
<!---->				<!---->			writesdram(&amp;connectionSD<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempInts, <xsl:value-of select="ceiling(6.0*number($n_dst)*number(SMLNL:FixedProbabilityConnection/@probability))"/>);
<!---->				<!---->		}
<!---->				<!---->	}	
<!---->				<!---->	// delays
<!---->				<!-- IF DELAYS ARE PROBABALISTIC -->
<!---->				<xsl:for-each select="SMLNL:FixedProbabilityConnection//SMLNL:UniformDistribution">
						<!---->	for (i=0; i &lt; <xsl:value-of select="$num_split_src_pops"/>; i += 1) {
<!---->					<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->					<!---->			for (k=0; k &lt; connRowLengths[j]; k += 1) {
<!---->					<!---->				tempFloats[k] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->					<!---->			}
<!---->					<!---->			writesdram(&amp;delay<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempFloats, <xsl:value-of select="$n_dst"/>);
<!---->					<!---->		}
<!---->					<!---->	}
<!---->				</xsl:for-each>
			</xsl:when>
			<xsl:otherwise>
				<!---->!!! Error - connection type not recognised (<xsl:value-of select="../@name"/>, src=<xsl:value-of select="@src"/>, src_port=<xsl:value-of select="@src_port"/>, dst_port=<xsl:value-of select="@dst_port"/>)
			</xsl:otherwise>
		</xsl:choose>
	</xsl:if>
	
	<!-- ANALOG      CONNECTION TYPE ########################################### -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->	
	<xsl:if test="$conn_type='ANALOG'">
<!---->	<xsl:choose>
			<!-- ONE TO ONE CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:OneToOneConnection)=1">
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:for-each select=".//SMLNL:UniformDistribution">
					<!---->	// delay 
<!---->				<!---->	for (i=0; i &lt; <xsl:value-of select="$num_split_src_pops"/>; i += 1) {
<!---->				<!---->		for (j=0; j &lt; <xsl:value-of select="$n_dst"/>; j += 1) {
<!---->				<!---->			delay<xsl:value-of select="$suffix"/>[i][j] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->				<!---->		}
<!---->				<!---->	}
<!---->			</xsl:for-each>
			</xsl:when>
			<!-- ALL TO ALL CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->	
			<xsl:when test="count(SMLNL:AllToAllConnection)=1">
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:for-each select=".//SMLNL:UniformDistribution">
					<!---->	// delay 
<!---->				<!---->	for (i=0; i &lt; <xsl:value-of select="$num_split_src_pops"/>; i += 1) {
<!---->				<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->				<!---->			for (k=0; k &lt; <xsl:value-of select="$n_dst"/>; k += 1) {
<!---->				<!---->				tempFloats[k] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->				<!---->			}
<!---->				<!---->			writesdram(&amp;delay<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempFloats, <xsl:value-of select="$n_dst"/>);
<!---->				<!---->		}
<!---->				<!---->	}
<!---->			</xsl:for-each>
			</xsl:when>	
			<!-- EXPLICIT LIST CONNECTION TYPE ######################################### -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:ConnectionList)=1">
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:for-each select=".//SMLNL:UniformDistribution">
					<!---->	// delay 
<!---->				<!---->	for (i=0; i &lt; <xsl:value-of select="$num_split_src_pops"/>; i += 1) {
<!---->				<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->				<!---->			for (k=0; k &lt; <xsl:value-of select="$n_dst"/>; k += 1) {
<!---->				<!---->				tempFloats[k] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->				<!---->			}
<!---->				<!---->			writesdram(&amp;delay<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempFloats, <xsl:value-of select="$n_dst"/>);
<!---->				<!---->		}
<!---->				<!---->	}
<!---->			</xsl:for-each>
			</xsl:when>
<!-- FIXED PROB CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:FixedProbabilityConnection)=1">
				<!-- THIS ONE IS A BIT DIFFERENT, AS WE NEED TO FILL IN ALL THE SD 
				RAM ARRAYS WITH THE NUMBER OF DELAYS GENERATED FOR THE CONNECTION -->
				<!-- CONNECTIVITY -->
				<!---->	// connectivity
<!---->				<!---->	for (i=0; i &lt; <xsl:value-of select="$num_split_src_pops"/>; i += 1) {
<!---->				<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->				<!---->			counter = 0;
<!---->				<!---->			for (k=0; k &lt; <xsl:value-of select="ceiling(6.0*number($n_dst)*number(SMLNL:FixedProbabilityConnection/@probability))"/>; k += 1) {
<!---->				<!---->				if (uniformRNG(1.0, 0.0) &lt; <xsl:value-of select="SMLNL:FixedProbabilityConnection/@probability"/>) {	
<!---->				<!---->					tempInts[counter] = k;
<!---->				<!---->					counter += 1;
<!---->				<!---->				}
<!---->				<!---->			}
<!---->				<!---->			connRowLengths[j] = counter; // for use in pars init
<!---->				<!---->			tempInts[counter+1] = -1; // terminate list
<!---->				<!---->			writesdram(&amp;connectionSD<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempInts, <xsl:value-of select="ceiling(6.0*number($n_dst)*number(SMLNL:FixedProbabilityConnection/@probability))"/>);
<!---->				<!---->		}
<!---->				<!---->	}	
<!---->				<!---->	// delays
<!---->				<!-- IF DELAYS ARE PROBABALISTIC -->
<!---->				<xsl:for-each select="SMLNL:FixedProbabilityConnection//SMLNL:UniformDistribution">
						<!---->	for (i=0; i &lt; <xsl:value-of select="$num_split_src_pops"/>; i += 1) {
<!---->					<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->					<!---->			for (k=0; k &lt; connRowLengths[j]; k += 1) {
<!---->					<!---->				tempFloats[k] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->					<!---->			}
<!---->					<!---->			writesdram(&amp;delay<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempFloats, <xsl:value-of select="$n_dst"/>);
<!---->					<!---->		}
<!---->					<!---->	}
<!---->				</xsl:for-each>
			</xsl:when>
			<xsl:otherwise>
				<!---->!!! Error - connection type not recognised (<xsl:value-of select="../@name"/>, src=<xsl:value-of select="@src"/>, src_port=<xsl:value-of select="@src_port"/>, dst_port=<xsl:value-of select="@dst_port"/>)
			</xsl:otherwise>
		</xsl:choose>
	</xsl:if>	
	
	</xsl:if>
	
</xsl:template>

<!-- ####################################################
#														#
#			PostSynapse OR NRN INPUT FUNCTIONS					#
#														#
######################################################-->

<xsl:template match="SMLLOWNL:Input" mode="defineFullInputFuncs">
	<xsl:param name="split_model_file"/>
	<xsl:param name="n_dst"/>
	<xsl:param name="n_src"/>
	
	<!-- DON'T DO CONDUCTANCE BASED SYNAPSE INPUTS -->
	<xsl:if test="not(local-name(..)='PostSynapse' and @src=../../../@dst_population and count(SMLNL:OneToOneConnection)=1)">
	
	<!-- FOR NOW JUST FIND THE NUMBER OF SPLITS IN THE SOURCE -->
	<xsl:variable name="num_split_src_pops">
		<xsl:variable name="src" select="@src"/>
		<xsl:value-of select="count($split_model_file//SMLLOWNL:Population/SMLLOWNL:Neuron[substring-before(@name,'_sub')=$src])"/>
	</xsl:variable>
	
	<!-- GET THE CONNECTION TYPE (EVENT OR ANALOG) -->
	<xsl:variable name="conn_type">
		<xsl:variable name="dst_port" select="@dst_port"/>
		<xsl:if test="count(document(../@url)//SMLCL:EventReceivePort[@name=$dst_port])=1">
			<!---->EVENT<!---->
		</xsl:if>
		<xsl:if test="count(document(../@url)//SMLCL:AnalogReceivePort[@name=$dst_port] | document(../@url)//SMLCL:AnalogReducePort[@name=$dst_port])=1">
			<!---->ANALOG<!---->
		</xsl:if>
	</xsl:variable>
	
	<!-- GENERATE SUFFIX NAME -->
	<xsl:variable name="suffix">
		<xsl:value-of select="translate(../@name,' -','__')"/><!-- NEW SUFFIX -->
		<!---->_input_<!---->
		<xsl:variable name="id" select="generate-id(.)"/>
		<xsl:for-each select="../SMLLOWNL:Input">
			<xsl:if test="generate-id(.)=$id">
				<xsl:value-of select="position()"/>
			</xsl:if>
		</xsl:for-each>
	</xsl:variable>
	
	<!-- GENERATE COMPONENT SUFFIX NAME -->
	<xsl:variable name="suffix_comp">
		<xsl:value-of select="translate(../@name,' -','__')"/><!-- NEW SUFFIX -->
	</xsl:variable>
	
	<xsl:variable name="curr_input" select="."/>
	
<!---->// input to <xsl:value-of select="../@name"/>: <xsl:value-of select="$suffix"/> ///////////////
<!---->	

	<!-- HANDLE THE DIFFERENT CONNECTION TYPES -->
	<!-- EVENT      CONNECTION TYPE ############################################ -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->	
	<xsl:if test="$conn_type='EVENT'">
		<xsl:choose>
			<!-- ONE TO ONE CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:OneToOneConnection)=1">
				<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
<!---->					<!-- THREAD FUNCTIONS -->
						<!---->// function threads
<!---->					<!---->void <xsl:value-of select="$suffix"/>(int nrn_index) {
<!---->					<!---->	int i = nrn_index;
<!---->					<!---->	// run relevant OnEvent
						<xsl:for-each select="document($curr_input/../@url)//SMLCL:OnEvent[@src_port=$curr_input/@dst_port]">
							<xsl:apply-templates select="." mode="doOnEvent">
								<xsl:with-param name="suffix" select="$suffix_comp"/>
							</xsl:apply-templates>
						</xsl:for-each>
						
<!---->					<!---->}
<!---->
<!---->					<!---->void <xsl:value-of select="$suffix"/>_schedule(int nrn_index, int split_index) {
<!---->					<!---->	// no fetching, all we need to do is pass the delays to the threads
<!---->					<!---->	int thread;
<!---->					<!---->	delay(delay<xsl:value-of select="$suffix"/>*inv_dt);
<!---->					<!---->	thread = createthread(<xsl:value-of select="$suffix"/>, 100, nrn_index, split_index);
<!---->					<!---->}
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
<!---->					<!-- THREAD FUNCTIONS -->
						<!---->// function threads
<!---->					<!---->void <xsl:value-of select="$suffix"/>(int nrn_index, int split_index) {
<!---->					<!---->	int i = nrn_index;
<!---->					<!---->	delay(delay<xsl:value-of select="$suffix"/>[i]*inv_dt);
<!---->					<!---->	// run relevant OnEvent
						<xsl:for-each select="document($curr_input/../@url)//SMLCL:OnEvent[@src_port=$curr_input/@dst_port]">
							<xsl:apply-templates select="." mode="doOnEvent">
								<xsl:with-param name="suffix" select="$suffix_comp"/>
							</xsl:apply-templates>
						</xsl:for-each>
						
<!---->					<!---->}

<!---->					<!---->void <xsl:value-of select="$suffix"/>_schedule(int nrn_index, int split_index) {
<!---->					<!---->	// no fetching, all we need to do is pass the delays to the threads
<!---->					<!---->	int thread;
<!---->					<!---->	thread = createthread(<xsl:value-of select="$suffix"/>, 100, nrn_index, split_index);
<!---->					<!---->}
<!---->			</xsl:if>
			</xsl:when>
			<!-- ALL TO ALL CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->	
			<xsl:when test="count(SMLNL:AllToAllConnection)=1">
				<!-- ALL TO ALLS DO NOT NEED EXTERNAL CONNECTIVITY PATTERNS SO NONE HERE -->
				<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
<!---->					<!-- THREAD FUNCTIONS -->
						<!---->// function threads
<!---->					<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index) {
<!---->					<!---->	int i;
<!---->					<!---->	// run relevant OnEvent for each dst nrn
<!---->					<!---->	for (i = 0; i &lt; num_neurons_actual; i += 1) {						
						<xsl:for-each select="document($curr_input/../@url)//SMLCL:OnEvent[@src_port=$curr_input/@dst_port]">
							<xsl:apply-templates select="." mode="doOnEvent">
								<xsl:with-param name="suffix" select="$suffix_comp"/>
							</xsl:apply-templates>
						</xsl:for-each>
<!---->					<!---->	}						
<!---->					<!---->}

<!---->					<!---->void <xsl:value-of select="$suffix"/>_schedule(int nrn_index, int split_index) {
<!---->					<!---->	// no fetching, all we need to do is pass the delays to the threads
<!---->					<!---->	int thread;
<!---->					<!---->	delay(delay<xsl:value-of select="$suffix"/>*inv_dt);
<!---->					<!---->	thread = createthread(<xsl:value-of select="$suffix"/>, 100, src_nrn_index);
<!---->					<!---->}
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
<!---->					<!-- THREAD FUNCTIONS -->
						<!---->// function threads
<!---->					<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay) {
<!---->					<!---->	int i;
<!---->					<!---->	i = dst_nrn_index;
<!---->					<!---->	delay(delay*inv_dt);

						<xsl:for-each select="document($curr_input/../@url)//SMLCL:OnEvent[@src_port=$curr_input/@dst_port]">
							<xsl:apply-templates select="." mode="doOnEvent">
								<xsl:with-param name="suffix" select="$suffix_comp"/>
							</xsl:apply-templates>
						</xsl:for-each>
<!---->					<!---->	}						
<!---->					<!---->}

<!---->					<!---->void <xsl:value-of select="$suffix"/>_schedule(int nrn_index, int split_index) {
<!---->					<!---->	// no fetching, all we need to do is pass the delays to the threads
<!---->					<!---->	int thread;
<!---->					<!---->	int i;
<!---->					<!---->	float delay<xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>];
<!---->					<!---->	readsdram(&amp;delaySD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;delay<xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_dst"/>);
<!---->					<!---->	// run relevant OnEvent for each dst nrn
<!---->					<!---->	for (i = 0; i &lt; num_neurons_actual; i += 1)	{	
<!---->					<!---->		thread = createthread(<xsl:value-of select="$suffix"/>, 100, src_nrn_index, i, delay<xsl:value-of select="$suffix"/>[i]);
<!---->					<!---->	}
<!---->					<!---->}
<!---->			</xsl:if>
			</xsl:when>	
			<!-- EXPLICIT LIST CONNECTION TYPE ######################################### -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:ConnectionList)=1">
<!---->			<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
<!---->					<!-- THREAD FUNCTIONS -->
						<!---->// function threads
<!---->					<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int split_index) {
<!---->					<!---->	int i;
<!---->					<!---->	int connection<xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>];
<!---->					<!---->	readsdram(&amp;connectionSD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;connection<xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_dst"/>);
<!---->					<!---->	// run relevant OnEvent for each dst nrn
<!---->					<!---->	for (i = 0; i &lt; num_neurons_actual; i += 1) {			
<!---->					<!---->		if (connection<xsl:value-of select="$suffix"/>[i] == 0) continue;			
						<xsl:for-each select="document($curr_input/../@url)//SMLCL:OnEvent[@src_port=$curr_input/@dst_port]">
							<xsl:apply-templates select="." mode="doOnEvent">
								<xsl:with-param name="suffix" select="$suffix_comp"/>
							</xsl:apply-templates>
						</xsl:for-each>
<!---->					<!---->	}						
<!---->					<!---->}

<!---->					<!---->void <xsl:value-of select="$suffix"/>_schedule(int nrn_index, int split_index){
<!---->					<!---->	// no fetching, all we need to do is pass the delays to the threads
<!---->					<!---->	int thread;
<!---->					<!---->	delay(delay<xsl:value-of select="$suffix"/>*inv_dt);
<!---->					<!---->	thread = createthread(<xsl:value-of select="$suffix"/>, 1000, src_nrn_index, split_index);
<!---->					<!---->}
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC OR EXPLICIT -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution | .//@delay)>0">
<!---->					<!-- THREAD FUNCTIONS -->
						<!---->// function threads
<!---->					<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay) {
<!---->					<!---->	int i;
<!---->					<!---->	i = dst_nrn_index;
<!---->					<!---->	delay(delay*inv_dt);

						<xsl:for-each select="document($curr_input/../@url)//SMLCL:OnEvent[@src_port=$curr_input/@dst_port]">
							<xsl:apply-templates select="." mode="doOnEvent">
								<xsl:with-param name="suffix" select="$suffix_comp"/>
							</xsl:apply-templates>
						</xsl:for-each>						
<!---->					<!---->}

<!---->					<!---->void <xsl:value-of select="$suffix"/>_schedule(int nrn_index, int split_index){
<!---->					<!---->	// no fetching, all we need to do is pass the delays to the threads
<!---->					<!---->	int thread;
<!---->					<!---->	int i;
<!---->					<!---->	int connection<xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>];
<!---->					<!---->	float delay<xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>];
<!---->					<!---->	readsdram(&amp;connectionSD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;connection<xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_dst"/>);
<!---->					<!---->	readsdram(&amp;delaySD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;delay<xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_dst"/>);
<!---->					<!---->	// run relevant OnEvent for each dst nrn
<!---->					<!---->	for (i = 0; i &lt; num_neurons_actual; i += 1)	{	
<!---->					<!---->		if (connection<xsl:value-of select="$suffix"/>[i] == 0) continue;	
<!---->					<!---->		thread = createthread(<xsl:value-of select="$suffix"/>, 100, src_nrn_index, i, delay<xsl:value-of select="$suffix"/>[i]);
<!---->					<!---->	}
<!---->					<!---->}
<!---->			</xsl:if>

			</xsl:when>
			<!-- FIXED PROB CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:FixedProbabilityConnection)=1">
				<xsl:variable name="n_prob" select="ceiling(6.0*number($n_dst)*number(SMLNL:FixedProbabilityConnection/@probability))"/>
<!---->			<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
<!---->					<!-- THREAD FUNCTIONS -->
						<!---->// function threads
<!---->					<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int split_index) {
<!---->					<!---->	int i;
<!---->					<!---->	int index;
<!---->					<!---->	int connection<xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_prob"/>];
<!---->					<!---->	readsdram(&amp;connectionSD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;connection<xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_prob"/>);
<!---->					<!---->	// run relevant OnEvent for each dst nrn
<!---->					<!---->	index=0;
<!---->					<!---->	while (connection<xsl:value-of select="$suffix"/>[index] != -1) {			
<!---->					<!---->		index+=1;
<!---->					<!---->		i = connection<xsl:value-of select="$suffix"/>[index];
<!---->
						<xsl:for-each select="document($curr_input/../@url)//SMLCL:OnEvent[@src_port=$curr_input/@dst_port]">
							<xsl:apply-templates select="." mode="doOnEvent">
								<xsl:with-param name="suffix" select="$suffix_comp"/>
							</xsl:apply-templates>
						</xsl:for-each>
<!---->					<!---->	}						
<!---->					<!---->}

<!---->					<!---->void <xsl:value-of select="$suffix"/>_schedule(int nrn_index, int split_index){
<!---->					<!---->	// no fetching, all we need to do is pass the delays to the threads
<!---->					<!---->	int thread;
<!---->					<!---->	delay(delay<xsl:value-of select="$suffix"/>*inv_dt);
<!---->					<!---->	thread = createthread(<xsl:value-of select="$suffix"/>, 1000, src_nrn_index, split_index);
<!---->					<!---->}
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
<!---->					<!-- THREAD FUNCTIONS -->
						<!---->// function threads
<!---->					<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay) {
<!---->					<!---->	int i;
<!---->					<!---->	i = dst_nrn_index;
<!---->					<!---->	delay(delay*inv_dt);

						<xsl:for-each select="document($curr_input/../@url)//SMLCL:OnEvent[@src_port=$curr_input/@dst_port]">
							<xsl:apply-templates select="." mode="doOnEvent">
								<xsl:with-param name="suffix" select="$suffix_comp"/>
							</xsl:apply-templates>
						</xsl:for-each>
<!---->					<!---->	}						
<!---->					<!---->}

<!---->					<!---->void <xsl:value-of select="$suffix"/>_schedule(int nrn_index, int split_index){
<!---->					<!---->	// no fetching, all we need to do is pass the delays to the threads
<!---->					<!---->	int thread;
<!---->					<!---->	int index;
<!---->					<!---->	int connection<xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_prob"/>];
<!---->					<!---->	float delay<xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_prob"/>];
<!---->					<!---->	readsdram(&amp;connectionSD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;connection<xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_prob"/>);
<!---->					<!---->	readsdram(&amp;delaySD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;delay<xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_prob"/>);
<!---->					<!---->	// launch thread for each dst nrn
<!---->					<!---->	index=0;
<!---->					<!---->	while (connection<xsl:value-of select="$suffix"/>[index] != -1) {			
<!---->					<!---->		index+=1;
<!---->					<!---->		thread = createthread(<xsl:value-of select="$suffix"/>, 100, src_nrn_index, connection<xsl:value-of select="$suffix"/>[index], delay<xsl:value-of select="$suffix"/>[index]);
<!---->					<!---->	}
<!---->					<!---->}
<!---->			</xsl:if>
				<!-- IF DELAYS ARE EXPLICIT (THIS IS NOT PERMITTED BECAUSE IT IS SILLY) -->
				<xsl:if test="count(.//SMLNL:Delay)>0">
				!!! Error - Non-Explicit List inputs with Explicit List delays are not supported (<xsl:value-of select="../@name"/>) !!!
				</xsl:if>
			</xsl:when>
			<xsl:otherwise>
				<!---->!!! Error - connection type not recognised (<xsl:value-of select="../@name"/>, src=<xsl:value-of select="@src"/>, src_port=<xsl:value-of select="@src_port"/>, dst_port=<xsl:value-of select="@dst_port"/>)
			</xsl:otherwise>
		</xsl:choose>
	</xsl:if>
	
	<!-- ANALOG      CONNECTION TYPE ########################################### -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->
	<!-- ####################################################################### -->	
	<xsl:if test="$conn_type='ANALOG'">
<!---->	<xsl:choose>
			<!-- ONE TO ONE CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:OneToOneConnection)=1">
				<!-- ONE TO ONES DO NOT NEED A CONNECTION -->
				<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
<!---->					<!-- THREAD FUNCTION -->
<!---->					<!---->void <xsl:value-of select="$suffix"/>_schedule(int nrn_index, int split_index, float value) /* split index not used for 1-2-1 */ {
<!---->					<!---->	// no fetching, all we need to do is pass the delays to the threads
<!---->					<!---->	int thread;
<!---->					<!---->	delay(delay<xsl:value-of select="$suffix"/>*inv_dt);
<!---->					<!---->	// transfer data to input array
<!---->					<!---->	// semaphore:
<!---->					<!---->	wait(&amp;<xsl:value-of select="$suffix"/>_semaphore);	
<!---->					<!----><xsl:text>	</xsl:text><xsl:value-of select="@dst_port"/><xsl:value-of select="$suffix_comp"/>[nrn_index] += value;
<!---->					<!---->	signal(&amp;<xsl:value-of select="$suffix"/>_semaphore);
<!---->					<!---->}
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
<!---->					<!-- THREAD FUNCTION -->
<!---->					<!---->void <xsl:value-of select="$suffix"/>_schedule(int nrn_index, int split_index, float value) {
<!---->					<!---->	// no fetching, all we need to do is pass the delays to the threads
<!---->					<!---->	int thread;
<!---->					<!---->	delay(delay<xsl:value-of select="$suffix"/>[nrn_index]*inv_dt);
<!---->					<!---->	// transfer data to input array
<!---->					<!---->	// semaphore:
<!---->					<!---->	wait(&amp;<xsl:value-of select="$suffix"/>_semaphore);	
<!---->					<!---->	<xsl:value-of select="@dst_port"/><xsl:value-of select="$suffix_comp"/>[nrn_index] += value;
<!---->					<!---->	signal(&amp;<xsl:value-of select="$suffix"/>_semaphore);
<!---->					<!---->}
<!---->			</xsl:if>

			</xsl:when>
			<!-- ALL TO ALL CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->	
			<xsl:when test="count(SMLNL:AllToAllConnection)=1">
				<!-- ALL TO ALLS DO NOT NEED EXTERNAL CONNECTIVITY PATTERNS SO NONE HERE -->
				<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
<!---->					<!-- THREAD FUNCTION -->
<!---->					<!---->void <xsl:value-of select="$suffix"/>_schedule(int nrn_index, float value) {
<!---->					<!---->	// no fetching, all we need to do is pass the delays to the threads
<!---->					<!---->	int thread;
<!---->					<!---->	int i;
<!---->					<!---->	delay(delay<xsl:value-of select="$suffix"/>*inv_dt);
<!---->					<!---->	// transfer data to input array
<!---->					<!---->	for (i = 0; i &lt; num_neurons_actual; i += 1) {
<!---->					<!---->	// semaphore:
<!---->					<!---->	wait(&amp;<xsl:value-of select="$suffix"/>_semaphore);	
<!---->					<!----><xsl:text>		</xsl:text><xsl:value-of select="@dst_port"/><xsl:value-of select="$suffix_comp"/>[i] += value;
<!---->					<!---->	signal(&amp;<xsl:value-of select="$suffix"/>_semaphore);	
<!---->					<!---->	}					
<!---->					<!---->}
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
<!---->					<!-- THREAD FUNCTIONS -->
						<!---->// function threads
<!---->					<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay, float value) {
<!---->					<!---->	delay(delay*inv_dt);

<!---->					<!---->	// transfer data to input array
<!---->					<!---->	// semaphore:
<!---->					<!---->	wait(&amp;<xsl:value-of select="$suffix"/>_semaphore);	
<!---->					<!----><xsl:text>	</xsl:text><xsl:value-of select="@dst_port"/><xsl:value-of select="$suffix_comp"/>[dst_nrn_index] += value;
<!---->					<!---->	signal(&amp;<xsl:value-of select="$suffix"/>_semaphore);						
<!---->					<!---->}

<!---->					<!---->void <xsl:value-of select="$suffix"/>_schedule(int nrn_index, float value) {
<!---->					<!---->	// no fetching, all we need to do is pass the delays to the threads
<!---->					<!---->	int thread;
<!---->					<!---->	int i;
<!---->					<!---->	float delay<xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>];
<!---->					<!---->	readsdram(&amp;delaySD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;delay<xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_dst"/>);
<!---->					<!---->	// run relevant OnEvent for each dst nrn
<!---->					<!---->	for (i = 0; i &lt; num_neurons_actual; i += 1)	{	
<!---->					<!---->		thread = createthread(<xsl:value-of select="$suffix"/>, 100, src_nrn_index, i, delay<xsl:value-of select="$suffix"/>[i], value);
<!---->					<!---->	}
<!---->					<!---->}
<!---->			</xsl:if>
			</xsl:when>	
			<!-- EXPLICIT LIST CONNECTION TYPE ######################################### -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:ConnectionList)=1">
<!---->			<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
<!---->					<!-- THREAD FUNCTIONS -->
						<!---->// function threads
<!---->					<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int split_index, float value) {
<!---->					<!---->	int i;
<!---->					<!---->	int connection<xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>];
<!---->					<!---->	readsdram(&amp;connectionSD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;connection<xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_dst"/>);
<!---->					<!---->	// run relevant OnEvent for each dst nrn
<!---->					<!---->	for (i = 0; i &lt; num_neurons_actual; i += 1) {			
<!---->					<!---->		if (connection<xsl:value-of select="$suffix"/>[i] == 0) continue;	
<!---->					<!---->		// semaphore:
<!---->					<!---->		wait(&amp;<xsl:value-of select="$suffix"/>_semaphore);		
<!---->					<!----><xsl:text>		</xsl:text><xsl:value-of select="@dst_port"/><xsl:value-of select="$suffix_comp"/>[i] += value;
<!---->					<!---->		signal(&amp;<xsl:value-of select="$suffix"/>_semaphore);
<!---->					<!---->	}						
<!---->					<!---->}

<!---->					<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index, float value) {
<!---->					<!---->	// no fetching, all we need to do is pass the delays to the threads
<!---->					<!---->	int thread;
<!---->					<!---->	delay(delay<xsl:value-of select="$suffix"/>*inv_dt);
<!---->					<!---->	thread = createthread(<xsl:value-of select="$suffix"/>, 1000, src_nrn_index, split_index, value);
<!---->					<!---->}
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC OR EXPLICIT-->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution | .//@delay)>0">
<!---->					<!-- THREAD FUNCTIONS -->
						<!---->// function threads
<!---->					<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay, float value) {
<!---->					<!---->	int i;
<!---->					<!---->	i = dst_nrn_index;
<!---->					<!---->	delay(delay*inv_dt);
<!---->					<!---->		// semaphore:
<!---->					<!---->		wait(&amp;<xsl:value-of select="$suffix"/>_semaphore);
<!---->					<!----><xsl:text>	</xsl:text><xsl:value-of select="@dst_port"/><xsl:value-of select="$suffix_comp"/>[i] += value;					
<!---->					<!---->		signal(&amp;<xsl:value-of select="$suffix"/>_semaphore);
<!---->					<!---->}

<!---->					<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, int split_index, float value) {
<!---->					<!---->	// no fetching, all we need to do is pass the delays to the threads
<!---->					<!---->	int thread;
<!---->					<!---->	int i;
<!---->					<!---->	int connection<xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>];
<!---->					<!---->	float delay<xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>];
<!---->					<!---->	readsdram(&amp;connectionSD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;connection<xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_dst"/>);
<!---->					<!---->	readsdram(&amp;delaySD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;delay<xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_dst"/>);
<!---->					<!---->	// run relevant OnEvent for each dst nrn
<!---->					<!---->	for (i = 0; i &lt; num_neurons_actual; i += 1)	{	
<!---->					<!---->		if (connection<xsl:value-of select="$suffix"/>[i] == 0) continue;	
<!---->					<!---->		thread = createthread(<xsl:value-of select="$suffix"/>, 1000, src_nrn_index, i, delay<xsl:value-of select="$suffix"/>[i], value);
<!---->					<!---->	}
<!---->					<!---->}
<!---->			</xsl:if>
			</xsl:when>
			<!-- FIXED PROB CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:FixedProbabilityConnection)=1">
				<xsl:variable name="n_prob" select="ceiling(6.0*number($n_dst)*number(SMLNL:FixedProbabilityConnection/@probability))"/>
<!---->			<!-- IF DELAYS ARE FIXED -->
				<xsl:if test="count(.//SMLNL:FixedValue)=1">
<!---->					<!-- THREAD FUNCTIONS -->
						<!---->// function threads
<!---->					<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int split_index, float value) {
<!---->					<!---->	int index;
<!---->					<!---->	int connection<xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_prob"/>];
<!---->					<!---->	readsdram(&amp;connectionSD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;connection<xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_prob"/>);
<!---->					<!---->	// run relevant OnEvent for each dst nrn
<!---->					<!---->	index=0;
<!---->					<!---->	while (connection<xsl:value-of select="$suffix"/>[index] != -1) {			
<!---->					<!---->		index+=1;
<!---->					<!---->		// semaphore:
<!---->					<!---->		wait(&amp;<xsl:value-of select="$suffix"/>_semaphore);
<!---->					<!----><xsl:text>		</xsl:text><xsl:value-of select="@dst_port"/><xsl:value-of select="$suffix_comp"/>[connection<xsl:value-of select="$suffix"/>[index]] += value;
<!---->					<!---->		signal(&amp;<xsl:value-of select="$suffix"/>_semaphore);
<!---->					<!---->	}						
<!---->					<!---->}

<!---->					<!---->void <xsl:value-of select="$suffix"/>_schedule(int nrn_index, float value) {
<!---->					<!---->	// no fetching, all we need to do is pass the delays to the threads
<!---->					<!---->	int thread;
<!---->					<!---->	delay(delay<xsl:value-of select="$suffix"/>*inv_dt);
<!---->					<!---->	thread = createthread(<xsl:value-of select="$suffix"/>, 1000, src_nrn_index, split_index, value);
<!---->					<!---->}
<!---->			</xsl:if>
				<!-- IF DELAYS ARE PROBABILISTIC -->
				<xsl:if test="count(.//SMLNL:UniformDistribution | .//SMLNL:NormalDistribution | .//SMLNL:PoissonDistribution)=1">
<!---->					<!-- THREAD FUNCTIONS -->
						<!---->// function threads
<!---->					<!---->void <xsl:value-of select="$suffix"/>(int src_nrn_index, int dst_nrn_index, float delay, float value) {
<!---->					<!---->	int i;
<!---->					<!---->	i = dst_nrn_index;
<!---->					<!---->	delay(delay*inv_dt);
<!---->					<!---->		// semaphore:
<!---->					<!---->		wait(&amp;<xsl:value-of select="$suffix"/>_semaphore);
<!---->					<!----><xsl:text>	</xsl:text><xsl:value-of select="@dst_port"/><xsl:value-of select="$suffix_comp"/>[i] += value;
<!---->					<!---->		signal(&amp;<xsl:value-of select="$suffix"/>_semaphore);
<!---->					<!---->	}						
<!---->					<!---->}
<!---->
<!---->					<!---->void <xsl:value-of select="$suffix"/>_schedule(int src_nrn_index, float value) {
<!---->					<!---->	// no fetching, all we need to do is pass the delays to the threads
<!---->					<!---->	int thread;
<!---->					<!---->	int index;
<!---->					<!---->	int connection<xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_prob"/>];
<!---->					<!---->	float delay<xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_prob"/>];
<!---->					<!---->	readsdram(&amp;connectionSD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;connection<xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_prob"/>);
<!---->					<!---->	readsdram(&amp;delaySD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;delay<xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_prob"/>);
<!---->					<!---->	// run threads for each dst nrn
<!---->					<!---->	index=0;
<!---->					<!---->	while (connection<xsl:value-of select="$suffix"/>[index] != -1) {			
<!---->					<!---->		index+=1;
<!---->					<!---->		thread = createthread(<xsl:value-of select="$suffix"/>, 1000, src_nrn_index, index, delay<xsl:value-of select="$suffix"/>[i], value);
<!---->					<!---->	}
<!---->					<!---->}
<!---->			</xsl:if>
			</xsl:when>
			<xsl:otherwise>
				<!---->!!! Error - connection type not recognised (<xsl:value-of select="../@name"/>, src=<xsl:value-of select="@src"/>, src_port=<xsl:value-of select="@src_port"/>, dst_port=<xsl:value-of select="@dst_port"/>)
			</xsl:otherwise>
		</xsl:choose>
	</xsl:if>	
	
	</xsl:if>
	
</xsl:template>


<!-- ####################################################
#														#
#			PostSynapse OR NRN INPUT FUNCTIONS					#
#														#
######################################################-->

<xsl:template match="SMLLOWNL:Input" mode="initExplicitConns">
	<xsl:param name="split_model_file"/>
	<xsl:param name="curr_dst"/>
	<xsl:param name="n_dst"/>
	<xsl:param name="n_src"/>
	

	<!-- FOR NOW JUST FIND THE NUMBER OF SPLITS IN THE SOURCE -->
	<xsl:variable name="num_split_src_pops">
		<xsl:variable name="src" select="@src"/>
		<xsl:value-of select="count($split_model_file//SMLLOWNL:Population/SMLLOWNL:Neuron[substring-before(@name,'_sub')=$src])"/>
	</xsl:variable>
	
	<!-- GET SPLIT INPUTS -->
	<xsl:variable name="input_src_name" select="@src"/>
	<xsl:variable name="src_port" select="@src_port"/>
	<xsl:variable name="dst_port" select="@dst_port"/>
	<xsl:variable name="inputs" select="$curr_dst/SMLLOWNL:Input[substring-before(@src,'_sub')=$input_src_name and @src_port=$src_port and @dst_port=$dst_port]"/>
			
	<!-- GENERATE SUFFIX NAME -->
	<xsl:variable name="suffix">
		<xsl:value-of select="translate(../@name,' -','__')"/><!-- NEW SUFFIX -->
		<!---->_input_<!---->
		<xsl:variable name="id" select="generate-id(.)"/>
		<xsl:for-each select="../SMLLOWNL:Input">
			<xsl:if test="generate-id(.)=$id">
				<xsl:value-of select="position()"/>
			</xsl:if>
		</xsl:for-each>
	</xsl:variable>
	
	<xsl:variable name="curr_input" select="."/>
	
<!---->// input to <xsl:value-of select="../@name"/>: <xsl:value-of select="$suffix"/> ///////////////
<!---->	

<!---->	<xsl:choose>
			<!-- ONE TO ONE CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:OneToOneConnection)=1">

			</xsl:when>
			<!-- ALL TO ALL CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->	
			<xsl:when test="count(SMLNL:AllToAllConnection)=1">

			</xsl:when>	
			<!-- EXPLICIT LIST CONNECTION TYPE ######################################### -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:ConnectionList)=1 and count($inputs)>0">
				<xsl:for-each select="SMLNL:ConnectionList">
					<!---->connectionSD<xsl:value-of select="$suffix"/> = {
<!---->				<xsl:for-each select="$inputs">
						<!---->{<!---->
						<xsl:call-template name="ConnectionExplicitVals">
							<xsl:with-param name="input" select="SMLNL:ConnectionList"/>
							<xsl:with-param name="maxRow" select="$n_dst"/>
							<xsl:with-param name="maxCol" select="$n_src"/>
						</xsl:call-template>
						<!---->}<!---->
						<xsl:if test="not(position()=$num_split_src_pops)">
							<!---->,
<!---->					</xsl:if>
					</xsl:for-each>
					<!-- ADD EXTRA LINES SO THAT THE DIMENSIONALITY IS CORRECT -->
					<xsl:variable name="line">
						<!---->{<!---->
						<xsl:call-template name="RepeatString">
						  <xsl:with-param name="string" select="'0'" />
						  <xsl:with-param name="divider" select="','" />
						  <xsl:with-param name="times"  select="$n_dst"/>
						</xsl:call-template>
						<!---->}<!---->
					</xsl:variable>
					<xsl:variable name="block">
						<!---->{<!---->
						<xsl:call-template name="RepeatString">
						  <xsl:with-param name="string" select="$line" />
						  <xsl:with-param name="divider" select="',&#10;'" />
						  <xsl:with-param name="times"  select="$n_src"/>
						</xsl:call-template>
						<!---->}<!---->
					</xsl:variable>
					<xsl:call-template name="RepeatString">
					  <xsl:with-param name="string" select="$block" />
					  <xsl:with-param name="divider" select="',&#10;'" />
					  <xsl:with-param name="times"  select="number($num_split_src_pops)-count($inputs)"/>
					</xsl:call-template>
					<!---->};
<!---->				<xsl:if test="count(SMLNL:Connection/@delay)>0">
						<!---->delaySD<xsl:value-of select="$suffix"/> = {
<!---->					<xsl:for-each select="$inputs">
							<!---->{<!---->
							<xsl:call-template name="DelayExplicitVals">
								<xsl:with-param name="input" select="SMLNL:ConnectionList"/>
								<xsl:with-param name="maxRow" select="$n_dst"/>
								<xsl:with-param name="maxCol" select="$n_src"/>
							</xsl:call-template>
							<!---->}<!---->
							<xsl:if test="not(position()=$num_split_src_pops)">
								<!---->,
<!---->						</xsl:if>
						</xsl:for-each>
												<!-- ADD EXTRA LINES SO THAT THE DIMENSIONALITY IS CORRECT -->
						<xsl:call-template name="RepeatString">
						  <xsl:with-param name="string" select="$block" />
						  <xsl:with-param name="divider" select="',&#10;'" />
						  <xsl:with-param name="times"  select="number($num_split_src_pops)-count($inputs)"/>
						</xsl:call-template>
						<!---->};
<!---->				</xsl:if>
				</xsl:for-each>
			</xsl:when>
			<!-- FIXED PROB CONNECTION TYPE ############################################ -->
			<!-- ####################################################################### -->
			<!-- ####################################################################### -->
			<xsl:when test="count(SMLNL:FixedProbabilityConnection)=1">

			</xsl:when>
			<xsl:when test="count(SMLNL:ConnectionList)=1 and not(count($inputs)>0)">
			</xsl:when>
			<xsl:otherwise>
				<!---->!!! Error - connection type not recognised (<xsl:value-of select="../@name"/>, src=<xsl:value-of select="@src"/>, src_port=<xsl:value-of select="@src_port"/>, dst_port=<xsl:value-of select="@dst_port"/>)
			</xsl:otherwise>
		</xsl:choose>
</xsl:template>

<xsl:template match="SMLLOWNL:Input" mode="serviceInputs">
<xsl:param name="suffix"/>
	<!-- fetch prototype for WeightUpdate -->
	<xsl:variable name="linked_proto" select="../@prototype"/>
	<xsl:variable name="linked_file" select="document(/SMLLOWNL:SpineML/SMLNL:ComponentPrototype[@name=$linked_proto]/@url)"/>
	<!-- select what the owner type is, and create the suffix -->
	<xsl:variable name="suffix_name">
		<xsl:value-of select="translate(../@name,' -','__')"/><!-- NEW SUFFIX -->
	</xsl:variable>
	<xsl:variable name="port_name" select="@dstPort"/>
		<xsl:for-each select="$linked_file//SMLCL:AnalogReceivePort[@name=$port_name]">
    // ############  ANALOG PORT ##########
    // assign port value:
    <xsl:value-of select="$port_name"/><xsl:value-of select="$suffix"/>[the_nrn_number] = the_packet_payload;
		</xsl:for-each>
		<xsl:for-each select="$linked_file//SMLCL:AnalogReducePort[@name=$port_name]">
    // ############  ANALOG PORT (REDUCE) ##########
    // assign port value:
 <xsl:if test="@reduce_op='+'">
    <xsl:value-of select="$port_name"/><xsl:value-of select="$suffix"/>[the_nrn_number] += the_packet_payload;
 </xsl:if>
 <xsl:if test="@reduce_op='*'">
    <xsl:value-of select="$port_name"/><xsl:value-of select="$suffix"/>[the_nrn_number] *= the_packet_payload;
 </xsl:if>
		</xsl:for-each>
		<xsl:for-each select="$linked_file//SMLCL:EventRecvPort[@name=$port_name]">
    // ############  EVENT PORT ##########
    // create WeightUpdate caches:
	<!-- we need an SDRAM region for each parameter, state_var, alias and analog port -->
	<xsl:apply-templates select="$linked_file//SMLCL:ComponentClass//SMLCL:Parameter | $linked_file//SMLCL:ComponentClass//SMLCL:StateVariable" mode="loadFromSDRAM"><xsl:with-param name="suffix" select="$suffix"/></xsl:apply-templates>

	// define aliases:
 	<xsl:apply-templates select="$linked_file//SMLCL:ComponentClass//SMLCL:Alias" mode="defineAlias"><xsl:with-param name="suffix" select="$suffix"/></xsl:apply-templates>

	for (i = 0; i &lt; NEURONS_PER_CORE; i += 1) {

	  // calculate Aliases
 	  <xsl:apply-templates select="$linked_file//SMLCL:ComponentClass//SMLCL:Alias" mode="allUpdates"><xsl:with-param name="suffix" select="$suffix"/></xsl:apply-templates>

      // do OnEvent for port
			 <xsl:for-each select="$linked_file//SMLCL:Regime">
				<xsl:apply-templates select="SMLCL:OnEvent[@src_port=$port_name]" mode="doEventInput"><xsl:with-param name="suffix" select="$suffix"/></xsl:apply-templates>
			 </xsl:for-each>
		</xsl:for-each>
	}
</xsl:template>



<xsl:template match="SMLLOWNL:Input | SMLNL:Connectivity" mode="createAlias">
<xsl:param name="component"/>
<xsl:param name="size"/>
<xsl:param name="suffix"/>
<xsl:param name="suffix"/>



</xsl:template>

<xsl:include href="SpineML_OnEvent.xsl"/>
<!--xsl:include href="SpineML_helpers.xsl"/-->

</xsl:stylesheet>
