<?xml version="1.0" encoding="ISO-8859-1"?><xsl:stylesheet version="1.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:SMLNL="http://www.shef.ac.uk/SpineMLNetworkLayer" xmlns:SMLLOWNL="http://www.shef.ac.uk/SpineMLLowLevelNetworkLayer" xmlns:SMLCL="http://www.shef.ac.uk/SpineMLComponentLayer"xmlns:fn="http://www.w3.org/2005/xpath-functions">
<xsl:output method="text" version="1.0" encoding="UTF-8" indent="yes"/>

<!-- ################################################
#													#
#		DEFINE ALL PARS / FUNCS FOR SYNAPSE			#
#													#
##################################################-->

<xsl:template match="SMLLOWNL:WeightUpdate" mode="defineAll">
<xsl:param name="curr_pop_name"/>
<xsl:param name="split_model_file"/>
<xsl:param name="number" select="number('0')"/>
<xsl:param name="curr_split_pop" select="''"/>
<xsl:param name="split_pops" select="'none'"/>
<xsl:param name="n_dst"/>
<xsl:param name="n_src"/>

<xsl:variable name="curr_pop_name_sanitised" select="translate($curr_pop_name, ' -', 'oo')"/>
<xsl:variable name="curr_component_name" select="translate(document(/SMLLOWNL:Neuron[@name=$curr_pop_name]/@url)/SMLCL:ComponentClass/@name, ' -', 'oo')"/>
<xsl:variable name="curr_synapse" select="."/>
<!-- SUFFIX IS EASY AS COMPONENT MUST BE A SYNAPSE -->
<xsl:variable name="suffix">
	<xsl:value-of select="translate($curr_synapse/@name,' -','__')"/><!-- NEW SUFFIX -->
</xsl:variable>

<!-- IF THIS IS NOT THE FIRST CALL OF THE TEMPLATE -->
<xsl:if test="not($split_pops='none')">
	<xsl:variable name="curr_syn_name" select="@name"/>
	<xsl:choose>
		<xsl:when test="count($split_pops)>0">
			<!-- GET THE SPLIT INDEX OF THE CURRENT SPLIT POPULATION -->
			<xsl:variable name="curr_split_pop_num" select="substring-after($split_pops[1]/SMLLOWNL:Neuron/@name, '_sub')"/>
			<!-- CREATE A VARIABLE WITH THE NUMBER OF INPUTS TO THE CURRENT SPLIT POP BY:
				1) finding the projections in the split file that target the current split population
				2) making sure that these are derived from the current synapse	
				3) this is enough to ensure that we only have the synapses from the rigt projection that target the right population		
			 -->
			<xsl:variable name="num_curr_pop_proj_inputs" select="count($split_model_file//SMLLOWNL:Projection[@dst_population=$split_pops[1]/SMLLOWNL:Neuron/@name]/SMLLOWNL:Synapse/SMLLOWNL:WeightUpdate[substring-before(@name, '_sub')=$curr_syn_name])"/>
		 	<xsl:choose>
		 		<!-- IF THE NUMBER OF INPUTS IS GREATER THAN THE PREVIOUS HIGHEST -->
		 		<xsl:when test="$num_curr_pop_proj_inputs>$number">
		 			<!-- CALL TEMPLATE WITH NEW POP AND NEW NUMBER OF INPUTS -->
		 			<xsl:apply-templates select="$curr_syn_name/.." mode="defineAll">
						<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
						<xsl:with-param name="split_model_file" select="$split_model_file"/>
						<xsl:with-param name="number" select="$num_curr_pop_proj_inputs"/>
						<xsl:with-param name="curr_split_pop" select="$split_pops[1]"/>
						<xsl:with-param name="split_pops" select="$split_pops[position()&gt;1]"/>
						<xsl:with-param name="n_dst" select="$n_dst"/>
						<xsl:with-param name="n_src" select="$n_src"/>
					</xsl:apply-templates>
		 		</xsl:when>
		 		<!-- OTHERWISE -->
		 		<xsl:otherwise>
		 			<!-- PASS ON CURRENT POP AND NUMBER -->
		 			<xsl:apply-templates select="$curr_syn_name/.." mode="defineAll">
						<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
						<xsl:with-param name="split_model_file" select="$split_model_file"/>
						<xsl:with-param name="number" select="$number"/>
						<xsl:with-param name="curr_split_pop" select="$curr_split_pop"/>
						<xsl:with-param name="split_pops" select="$split_pops[position()&gt;1]"/>
						<xsl:with-param name="n_dst" select="$n_dst"/>
						<xsl:with-param name="n_src" select="$n_src"/>
					</xsl:apply-templates>
		 		</xsl:otherwise>
		 	</xsl:choose>	 		
		</xsl:when>
		<xsl:otherwise>
			<!-- LOOPING COMPLETE - WRITE OUT THE DEFINITIONS FOR THE SYNAPSES -->
			<!-- NOTE: INTO SPLIT FILE -->
			<!--xsl:for-each select="$split_model_file//SMLLOWNL:Projection[@dst_population=$curr_split_pop/SMLLOWNL:Neuron/@name]/SMLLOWNL:Synapse/SMLLOWNL:WeightUpdate[substring-before(@name, '_sub')=$curr_syn_name]"-->
				<xsl:text>
</xsl:text>
				<!-- HEADING -->// <xsl:value-of select="@name"/> ##########################
<!---->							
				<!-- AL COMPONENT FOR THIS SYNAPSE -->
				<xsl:variable name="linked_syn_file" select="document(@url)"/>
				<!-- ok, quick comment on numbering - I chose to give every Synapse a unique number - this means that numbers are less meaningful, but then they were never particularly meaningful anyway! -->
				<xsl:variable name="suffix_name">
					<xsl:value-of select="translate($curr_synapse/@name,' -','__')"/><!-- NEW SUFFIX -->
				</xsl:variable>
					
				<xsl:for-each select="$linked_syn_file//SMLCL:ComponentClass">
				
					<!-- ADJUSTED VARIABLES DEPENDING UPON THE CONNECTIVITY  -->
					<xsl:choose>
						<!-- IF ONE TO ONE CONNECTION ###############################-->
						<xsl:when test="count($curr_synapse/../SMLNL:OneToOneConnection)=1">
							<!-- PARAMETERS -->
							<!---->// parameters:
<!---->						<!-- choose whether we use an array or not:
							if (we have a random value tag or and explicit list) 
							THEN it is an array and we should add the array notation.		            	
							 -->
							<xsl:for-each select="SMLCL:Parameter">
								<xsl:variable name="par_name" select="@name"/>
								<xsl:choose>
									<!-- WHEN PROBABILITY OR LIST -->
									<xsl:when test="(count($curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:NormalDistribution)=1 or count($curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1)">
										<!-- ARRAY -->
										<!---->float <xsl:value-of select="@name"/><xsl:value-of select="$suffix_name"/>[<xsl:value-of select="$number"/>][<xsl:value-of select="$n_dst"/>]; // (number of dst neurons per core)
<!---->								</xsl:when>
									<!-- WHEN FIXED -->
									<xsl:otherwise test="count($curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:FixedValue)=1">
										<!-- NO ARRAY -->
										<!---->float <xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>
										<!-- ADD ASSIGNED VALUE -->
										<xsl:value-of select="concat(' = ', format-number($curr_synapse/SMLNL:Property[@name=$par_name]/SMLNL:FixedValue/@value, '0.0########################'))"/>;
<!---->								</xsl:otherwise>
								</xsl:choose>
							</xsl:for-each>
							<!--STATE VARIABLES -->
							<xsl:for-each select="$linked_syn_file//SMLCL:StateVariable">
								<!---->// state variables:
<!---->							<!---->float <xsl:value-of select="@name"/><xsl:value-of select="$suffix_name"/>[<xsl:value-of select="$number"/>][<xsl:value-of select="$n_dst"/>]; // (number of dst neurons per core)
<!---->						</xsl:for-each>
							<!-- ADD THE ESSENTIAL VARIABLES -->
							<!-- WE DON'T NEED A REGIME VARIABLE IF WE HAVE ONLY ONE REGIME -->
							<xsl:if test="count(//SMLCL:Regime)>1">
								<!---->// regime variable:
<!---->							<!---->int regimeCurrSD<xsl:value-of select="$suffix_name"/>[<xsl:value-of select="$number"/>][<xsl:value-of select="$n_dst"/>]; // (number of dst neurons per core)
<!---->						</xsl:if>
							<!-- NO CONNECTIVITY REQUIRED FOR ONE TO ONE -->
							<!---->//delays
<!---->						<!-- IF DELAYS ARE FIXED -->
							<xsl:if test="count($curr_synapse/../SMLNL:OneToOneConnection//SMLNL:FixedValue)=1">
<!---->							<!---->float mainDelay<xsl:value-of name="suffix" select="$suffix_name"/> = <xsl:value-of select="$curr_synapse/../SMLNL:OneToOneConnection//SMLNL:FixedValue/@value"/>;<!---->							<!-- THREAD FUNCTION PROTOTYPES -->
								<!---->// function thread prototypes
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>(int nrn_index, int split_index);
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>_schedule(int nrn_index, int split_index);
<!---->						</xsl:if>
							<!-- IF DELAYS ARE PROBABALISTIC -->
							<xsl:if test="count($curr_synapse/../SMLNL:OneToOneConnection//SMLNL:UniformDistribution | $curr_synapse/../SMLNL:OneToOneConnection//SMLNL:NormalDistribution | $curr_synapse/../SMLNL:OneToOneConnection//SMLNL:PoissonDistribution)=1">
<!---->							<!---->float mainDelay<xsl:value-of name="suffix" select="$suffix_name"/>[<xsl:value-of select="$number"/>][<xsl:value-of select="$n_dst"/>];
<!---->							<!-- THREAD FUNCTION PROTOTYPES -->
								<!---->// function thread prototypes
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>(int nrn_index, int split_index, float delay);
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>_schedule(int nrn_index, int split_index);
<!---->						</xsl:if>	
							<!-- IF DELAYS ARE EXPLICIT (THEY ARE NOT SUPPORTED FOR 1-2-1 CONNECTIONS) -->
							<xsl:if test="count($curr_synapse/../SMLNL:FixedProbabilityConnection//SMLNL:ValueList)=1">
							!!! Error - Explicit delays are not supported for onetoone connections (<xsl:value-of select="$curr_synapse/@name"/>) !!!
	<!---->					</xsl:if>


<!---->					</xsl:when>
						<!-- IF ALL TO ALL CONNECTION ###############################-->
						<xsl:when test="count($curr_synapse/../SMLNL:AllToAllConnection)=1">
							<!-- PARAMETERS -->
							<!---->// parameters:
<!---->						<!-- choose whether we use an array or not:
							if (we have a random value tag or and explicit list) 
							THEN it is an array and we should add the array notation.		            	
							 -->
							<xsl:for-each select="SMLCL:Parameter">
								<xsl:variable name="par_name" select="@name"/>
								<xsl:choose>
									<!-- WHEN PROBABILITY OR LIST -->
									<xsl:when test="(count($curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:NormalDistribution)=1 or count($curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1)">
										<!-- ARRAY -->
										<!---->extern float <xsl:value-of select="@name"/>SD<xsl:value-of select="$suffix_name"/>[<xsl:value-of select="$number"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>]; // (number of src neurons per core)(number of dst neurons per core)
<!---->								</xsl:when>
									<!-- WHEN FIXED -->
									<xsl:otherwise test="count($curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:FixedValue)=1">
										<!-- NO ARRAY -->
										<!---->float <xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>
										<!-- ADD ASSIGNED VALUE -->
										<xsl:value-of select="concat(' = ', format-number($curr_synapse/SMLNL:Property[@name=$par_name]/SMLNL:FixedValue/@value,'0.0###############'))"/>;
<!---->								</xsl:otherwise>
								</xsl:choose>
							</xsl:for-each>
							<!--STATE VARIABLES -->
							<xsl:for-each select="$linked_syn_file//SMLCL:StateVariable">
								<!---->// state variables:
<!---->							<!---->extern float <xsl:value-of select="@name"/>SD<xsl:value-of select="$suffix_name"/>[<xsl:value-of select="$number"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>]; // (number of src neurons per core)(number of dst neurons per core)
<!---->						</xsl:for-each>
							<!-- ADD THE ESSENTIAL VARIABLES -->
							<!-- WE DON'T NEED A REGIME VARIABLE IF WE HAVE ONLY ONE REGIME -->
							<xsl:if test="count(//SMLCL:Regime)>1">
								<!---->// regime variable:
<!---->							<!---->int regimeCurrSD<xsl:value-of select="$suffix_name"/>[<xsl:value-of select="$number"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>]; // (number of src neurons per core)(number of dst neurons per core)
<!---->						</xsl:if>
							<!-- CONNECTION NOT NEEDED FOR A2A -->

							<!---->// delays	
<!---->						<!-- IF DELAYS ARE FIXED -->
							<xsl:if test="count($curr_synapse/../SMLNL:AllToAllConnection//SMLNL:FixedValue)=1">
<!---->							<!---->float mainDelay<xsl:value-of name="suffix" select="$suffix_name"/> = <xsl:value-of select="$curr_synapse/../SMLNL:AllToAllConnection//SMLNL:FixedValue/@value"/>;
<!---->							<!-- THREAD FUNCTION PROTOTYPES -->
								<!---->// function thread prototypes
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>(int src_nrn_index, int split_index);
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>_schedule(int src_nrn_index, int split_index);					
<!---->						</xsl:if>
							<!-- IF DELAYS ARE PROBABALISTIC -->
							<xsl:if test="count($curr_synapse/../SMLNL:AllToAllConnection//SMLNL:UniformDistribution | $curr_synapse/../SMLNL:AllToAllConnection//SMLNL:NormalDistribution | $curr_synapse/../SMLNL:AllToAllConnection//SMLNL:PoissonDistribution)=1">
<!---->							<!---->extern float mainDelaySD<xsl:value-of name="suffix" select="$suffix_name"/>[<xsl:value-of select="$number"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>];
<!---->							<!-- THREAD FUNCTION PROTOTYPES -->
								<!---->// function thread prototypes
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>(int src_nrn_index, int split_index, int dst_nrn_index, <!---->
								<xsl:for-each select="$linked_syn_file//SMLCL:Parameter">
									<!---->float <!---->
									<xsl:value-of select="@name"/>
									<!---->, <!---->
								</xsl:for-each>
								<xsl:for-each select="$linked_syn_file//SMLCL:StateVariable">
									<!---->float <!---->
									<xsl:value-of select="@name"/>
									<!---->, <!---->
								</xsl:for-each>
								<!---->float delay);
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>_schedule(int src_nrn_index, int split_index);
<!---->						</xsl:if>
							<!-- IF DELAYS ARE EXPLICIT (THEY ARE NOT SUPPORTED FOR A2A CONNECTIONS) -->
							<xsl:if test="count($curr_synapse/../SMLNL:AllToAllConnection//SMLNL:ValueList)=1">
								<!---->	!!! Error - Explicit delays are not supported for all to all connections (<xsl:value-of select="$curr_synapse/@name"/>) !!!
<!---->						</xsl:if>
						
						</xsl:when>
						<!-- IF FIXED PROBABILITY CONNECTION ###############################-->
						<xsl:when test="count($curr_synapse/../SMLNL:FixedProbabilityConnection)=1">
							<!-- PARAMETERS -->
							<!---->// parameters:
<!---->						<!-- choose whether we use an array or not:
							if (we have a random value tag or and explicit list) 
							THEN it is an array and we should add the array notation.		            	
							 -->
							<xsl:for-each select="SMLCL:Parameter">
								<xsl:variable name="par_name" select="@name"/>
								<xsl:choose>
									<!-- WHEN PROBABILITY BUT NOT LIST -->
									<xsl:when test="(count($curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:NormalDistribution)=1)">
										<!-- ARRAY -->
										<!---->extern float <xsl:value-of select="@name"/>SD<xsl:value-of select="$suffix_name"/>[<xsl:value-of select="$number"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="ceiling(6.0*number($n_dst)*number($curr_synapse/../SMLNL:FixedProbabilityConnection/@probability))"/>]; // (number of src neurons per core)(ceil(number of dst neurons per core * 6 * probability))
<!---->								</xsl:when>
									<!-- WHEN FIXED -->
									<xsl:when test="count($curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:FixedValue)=1">
										<!-- NO ARRAY -->
										<!---->float <xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>
										<!-- ADD ASSIGNED VALUE -->
										<xsl:value-of select="concat(' = ', format-number($curr_synapse/SMLNL:Property[@name=$par_name]/SMLNL:FixedValue/@value,'0.0###############'))"/>;
<!---->								</xsl:when>
									<xsl:otherwise>
										<!---->!!! Error - Explicit parameters are not supported for fixed probability connections (<xsl:value-of select="$curr_synapse/@name"/>) !!!
<!---->								</xsl:otherwise>
								</xsl:choose>
							</xsl:for-each>
							<!--STATE VARIABLES -->
							<xsl:for-each select="$linked_syn_file//SMLCL:StateVariable">
								<!---->// state variables:
<!---->							<!---->extern float <xsl:value-of select="@name"/>SD<xsl:value-of select="$suffix_name"/>[<xsl:value-of select="$number"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="ceiling(6.0*number($n_dst)*number($curr_synapse/../SMLNL:FixedProbabilityConnection/@probability))"/>]; // (number of src neurons per core)(ceil(number of dst neurons per core * 6 * probability))
<!---->						</xsl:for-each>
							<!-- ADD THE ESSENTIAL VARIABLES -->
							<!-- WE DON'T NEED A REGIME VARIABLE IF WE HAVE ONLY ONE REGIME -->
							<xsl:if test="count(//SMLCL:Regime)>1">
								<!---->// regime variable:<!---->
<!---->							<!---->int regimeCurrSD<xsl:value-of select="$suffix_name"/>[<xsl:value-of select="$number"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="ceiling(6.0*number($n_dst)*number($curr_synapse/../SMLNL:FixedProbabilityConnection/@probability))"/>]; // (number of src neurons per core)(ceil(number of dst neurons per core * 6 * probability))
<!---->						</xsl:if>
							<!---->// connectivity
<!---->						<!---->extern int mainConnectionSD<xsl:value-of select="$suffix_name"/>[<xsl:value-of select="$number"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="ceiling(6.0*number($n_dst)*number($curr_synapse/../SMLNL:FixedProbabilityConnection/@probability))"/>];						
<!---->						<!---->// delays
<!---->						<!-- IF DELAYS ARE FIXED -->
							<xsl:if test="count($curr_synapse/../SMLNL:FixedProbabilityConnection//SMLNL:FixedValue)=1">
<!---->							<!---->float mainDelay<xsl:value-of name="suffix" select="$suffix_name"/> = <xsl:value-of select="$curr_synapse/../SMLNL:FixedProbabilityConnection//SMLNL:FixedValue/@value"/>;	
<!---->							<!-- THREAD FUNCTION PROTOTYPES -->
								<!---->// function thread prototypes
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>(int src_nrn_index, int split_index);
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>_schedule(int src_nrn_index, int split_index);					
<!---->						</xsl:if>
							<!-- IF DELAYS ARE PROBABALISTIC -->
							<xsl:if test="count($curr_synapse/../SMLNL:FixedProbabilityConnection//SMLNL:UniformDistribution | $curr_synapse/../SMLNL:FixedProbabilityConnection//SMLNL:NormalDistribution | $curr_synapse/../SMLNL:FixedProbabilityConnection//SMLNL:PoissonDistribution)=1">
<!---->							<!---->extern float mainDelaySD<xsl:value-of name="suffix" select="$suffix_name"/>[<xsl:value-of select="$number"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="ceiling(6.0*number($n_dst)*number($curr_synapse/../SMLNL:FixedProbabilityConnection/@probability))"/>]; // (number of src neurons per core)(ceil(number of dst neurons per core * 6 * probability))
<!---->							<!-- THREAD FUNCTION PROTOTYPES -->
								<!---->// function thread prototypes
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>(int src_nrn_index, int split_index, int dst_nrn_index, <!---->
								<xsl:for-each select="$linked_syn_file//SMLCL:Parameter">
									<!---->float <!---->
									<xsl:value-of select="@name"/>
									<!---->, <!---->
								</xsl:for-each>
								<xsl:for-each select="$linked_syn_file//SMLCL:StateVariable">
									<!---->float <!---->
									<xsl:value-of select="@name"/>
									<!---->, <!---->
								</xsl:for-each>
								<!---->float delay);
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>_schedule(int src_nrn_index, int split_index);
<!---->						</xsl:if>
							<!-- IF DELAYS ARE EXPLICIT (THEY ARE NOT SUPPORTED FOR FIXED PROB CONNECTIONS) -->
							<xsl:if test="count($curr_synapse/../SMLNL:FixedProbabilityConnection//SMLNL:ValueList)=1">
							!!! Error - Explicit delays are not supported for fixed probability connections (<xsl:value-of select="$curr_synapse/@name"/>) !!!
<!---->						</xsl:if>
						
						
						</xsl:when>
						<!-- IF EXPLICIT LIST CONNECTION ###############################-->
						<xsl:when test="count($curr_synapse/../SMLNL:ConnectionList)=1">
							<!-- PARAMETERS -->
							<!---->// parameters:
<!---->						<!-- choose whether we use an array or not:
							if (we have a random value tag or and explicit list) 
							THEN it is an array and we should add the array notation.		            	
							 -->
							<xsl:for-each select="SMLCL:Parameter">
								<xsl:variable name="par_name" select="@name"/>
								<xsl:choose>
									<!-- WHEN PROBABILITY OR LIST -->
									<xsl:when test="(count($curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:NormalDistribution)=1) or count($curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
										<!-- ARRAY -->
										<!---->extern float <xsl:value-of select="@name"/>SD<xsl:value-of select="$suffix_name"/>[<xsl:value-of select="$number"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>]; // (number of src neurons per core)(number of dst neurons per core)
<!---->								</xsl:when>
									<!-- WHEN FIXED -->
									<xsl:otherwise test="count($curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:FixedValue)=1">
										<!-- NO ARRAY -->
										<!---->float <xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>
										<!-- ADD ASSIGNED VALUE -->
										<xsl:value-of select="concat(' = ', format-number($curr_synapse/SMLNL:Property[@name=$par_name]/SMLNL:FixedValue/@value,'0.0###############'))"/>;
<!---->								</xsl:otherwise>
								</xsl:choose>
							</xsl:for-each>
							<!--STATE VARIABLES -->
							<xsl:for-each select="$linked_syn_file//SMLCL:StateVariable">
								<!---->// state variables:
<!---->							<!---->extern float <xsl:value-of select="@name"/>SD<xsl:value-of select="$suffix_name"/>[<xsl:value-of select="$number"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>]; // (number of src neurons per core)(number of dst neurons per core)
<!---->						</xsl:for-each>
							<!-- ADD THE ESSENTIAL VARIABLES -->
							<!-- WE DON'T NEED A REGIME VARIABLE IF WE HAVE ONLY ONE REGIME -->
							<xsl:if test="count(//SMLCL:Regime)>1">
								<!---->// regime variable:
<!---->							<!---->int regimeCurrSD<xsl:value-of select="$suffix_name"/>[<xsl:value-of select="$number"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>]; // (number of src neurons per core)(number of dst neurons per core)
<!---->						</xsl:if>
							<!---->//connectivity
<!---->						<!---->extern int mainConnectionSD<xsl:value-of select="$suffix_name"/>[<xsl:value-of select="$number"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>]; // (number of src neurons per core)(number of dst neurons per core)
<!---->							
							<!---->// delays
<!---->						<!-- IF DELAYS ARE FIXED -->
							<xsl:if test="count($curr_synapse/../SMLNL:ConnectionList//SMLNL:FixedValue)=1">
<!---->							<!---->float mainDelay<xsl:value-of name="suffix" select="$suffix_name"/> = <xsl:value-of select="$curr_synapse/../SMLNL:ConnectionList//SMLNL:FixedValue/@value"/>;
<!---->							<!-- THREAD FUNCTION PROTOTYPES -->
								<!---->// function thread prototypes
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>(int src_nrn_index, int split_index);
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>_schedule(int src_nrn_index, int split_index);						
<!---->						</xsl:if>
							<!-- IF DELAYS ARE PROBABALISTIC -->
							<xsl:if test="count($curr_synapse/../SMLNL:ConnectionList//SMLNL:UniformDistribution | $curr_synapse/../SMLNL:ConnectionList//SMLNL:NormalDistribution | $curr_synapse/../SMLNL:ConnectionList//SMLNL:PoissonDistribution)=1">
<!---->							<!---->extern float mainDelaySD<xsl:value-of name="suffix" select="$suffix_name"/>[<xsl:value-of select="$number"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>]; // array (number of src neurons per core)(number of dst neurons per core) (Probabilistic Delays)
<!---->							<!-- THREAD FUNCTION PROTOTYPES -->
								<!---->// function thread prototypes
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>(int src_nrn_index, int split_index, int dst_nrn_index, <!---->
								<xsl:for-each select="$linked_syn_file//SMLCL:Parameter">
									<!---->float <!---->
									<xsl:value-of select="@name"/>
									<!---->, <!---->
								</xsl:for-each>
								<xsl:for-each select="$linked_syn_file//SMLCL:StateVariable">
									<!---->float <!---->
									<xsl:value-of select="@name"/>
									<!---->, <!---->
								</xsl:for-each>
								<!---->float delay);
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>_schedule(int src_nrn_index, int split_index);
<!---->						</xsl:if>
						
							<!-- IF DELAYS ARE EXPLICIT -->
							<xsl:if test="count($curr_synapse/../SMLNL:ConnectionList//SMLNL:Connection/@delay)>0">
								<!---->extern float mainDelaySD<xsl:value-of name="suffix" select="$suffix_name"/>[<xsl:value-of select="$number"/>][<xsl:value-of select="$n_src"/>][<xsl:value-of select="$n_dst"/>]; // array (number of src neurons per core)(number of dst neurons per core) (Explicit Delays)
<!---->							<!-- THREAD FUNCTION PROTOTYPES -->
								<!---->// function thread prototypes
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>(int src_nrn_index, int split_index, int dst_nrn_index, <!---->
								<xsl:for-each select="$linked_syn_file//SMLCL:Parameter">
									<!---->float <!---->
									<xsl:value-of select="@name"/>
									<!---->, <!---->
								</xsl:for-each>
								<xsl:for-each select="$linked_syn_file//SMLCL:StateVariable">
									<!---->float <!---->
									<xsl:value-of select="@name"/>
									<!---->, <!---->
								</xsl:for-each>
								<!---->float delay);
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>_schedule(int src_nrn_index, int split_index);
<!---->						</xsl:if>
						</xsl:when>
						<xsl:otherwise>
							<!---->!!! Error - connection type not recognised (<xsl:value-of select="$curr_synapse/@name"/>) !!!
<!---->					</xsl:otherwise>
					</xsl:choose>
				</xsl:for-each>

		</xsl:otherwise>
	</xsl:choose>

</xsl:if>

<!-- IF WE HAVE NO LIST BECAUSE THIS IS THE FIRST CALL OF THE TEMPLATE -->
<xsl:if test="$split_pops='none'">
	<!-- FIND ALL THE POPULATIONS IN THE SPLIT FILE WITH THE SAME BASE NAME AS THE UNSPLIT POPULATION -->
	<xsl:variable name="split_pops_new" select="$split_model_file//SMLLOWNL:Population[substring-before(SMLLOWNL:Neuron/@name, '_sub')=$curr_pop_name]"/>
	<!-- RE-CALL TEMPLATE WITH THIS LIST -->
	<xsl:apply-templates select="." mode="defineAll">
		<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
		<xsl:with-param name="split_model_file" select="$split_model_file"/>
		<xsl:with-param name="split_pops" select="$split_pops_new"/>
		<xsl:with-param name="n_dst" select="$n_dst"/>
		<xsl:with-param name="n_src" select="$n_src"/>
	</xsl:apply-templates>
</xsl:if>

</xsl:template>

<!-- ################################################
#													#
#		INITIALISE ALL DATA FOR SYNAPSE 			#
#													#
##################################################-->

<xsl:template match="SMLLOWNL:WeightUpdate" mode="initData">

<xsl:param name="curr_pop_name"/>
<xsl:param name="split_model_file"/>
<xsl:param name="number" select="number('0')"/>
<xsl:param name="curr_split_pop" select="''"/>
<xsl:param name="split_pops" select="'none'"/>
<xsl:param name="n_dst"/>
<xsl:param name="n_src"/>

<xsl:variable name="curr_pop_name_sanitised" select="translate($curr_pop_name, ' -', 'oo')"/>
<xsl:variable name="curr_component_name" select="translate(document(/SMLLOWNL:Neuron[@name=$curr_pop_name]/@url)/SMLCL:ComponentClass/@name, ' -', 'oo')"/>
<xsl:variable name="curr_synapse" select="."/>
<!-- SUFFIX IS EASY AS COMPONENT MUST BE A SYNAPSE -->
<xsl:variable name="suffix">
	<xsl:value-of select="translate($curr_synapse/@name,' -','__')"/><!-- NEW SUFFIX -->
</xsl:variable>

<!-- IF THIS IS NOT THE FIRST CALL OF THE TEMPLATE -->
<xsl:if test="not($split_pops='none')">
	<xsl:variable name="curr_syn_name" select="@name"/>
	<xsl:choose>
		<xsl:when test="count($split_pops)>0">
			<!-- GET THE SPLIT INDEX OF THE CURRENT SPLIT POPULATION -->
			<xsl:variable name="curr_split_pop_num" select="substring-after($split_pops[1]/SMLLOWNL:Neuron/@name, '_sub')"/>
			<!-- CREATE A VARIABLE WITH THE NUMBER OF INPUTS TO THE CURRENT SPLIT POP BY:
				1) finding the projections in the split file that target the current split population
				2) making sure that these are derived from the current synapse	
				3) this is enough to ensure that we only have the synapses from the rigt projection that target the right population		
			 -->
			<xsl:variable name="num_curr_pop_proj_inputs" select="count($split_model_file//SMLLOWNL:Projection[@dst_population=$split_pops[1]/SMLLOWNL:Neuron/@name]/SMLLOWNL:Synapse/SMLLOWNL:WeightUpdate[substring-before(@name, '_sub')=$curr_syn_name])"/>
		 	<xsl:choose>
		 		<!-- IF THE NUMBER OF INPUTS IS GREATER THAN THE PREVIOUS HIGHEST -->
		 		<xsl:when test="$num_curr_pop_proj_inputs>$number">
		 			<!-- CALL TEMPLATE WITH NEW POP AND NEW NUMBER OF INPUTS -->
		 			<xsl:apply-templates select="$curr_syn_name/.." mode="initData">
						<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
						<xsl:with-param name="split_model_file" select="$split_model_file"/>
						<xsl:with-param name="number" select="$num_curr_pop_proj_inputs"/>
						<xsl:with-param name="curr_split_pop" select="$split_pops[1]"/>
						<xsl:with-param name="split_pops" select="$split_pops[position()&gt;1]"/>
						<xsl:with-param name="n_dst" select="$n_dst"/>
						<xsl:with-param name="n_src" select="$n_src"/>
					</xsl:apply-templates>
		 		</xsl:when>
		 		<!-- OTHERWISE -->
		 		<xsl:otherwise>
		 			<!-- PASS ON CURRENT POP AND NUMBER -->
		 			<xsl:apply-templates select="$curr_syn_name/.." mode="initData">
						<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
						<xsl:with-param name="split_model_file" select="$split_model_file"/>
						<xsl:with-param name="number" select="$number"/>
						<xsl:with-param name="curr_split_pop" select="$curr_split_pop"/>
						<xsl:with-param name="split_pops" select="$split_pops[position()&gt;1]"/>
						<xsl:with-param name="n_dst" select="$n_dst"/>
						<xsl:with-param name="n_src" select="$n_src"/>
					</xsl:apply-templates>
		 		</xsl:otherwise>
		 	</xsl:choose>	 		
		</xsl:when>
		<xsl:otherwise>
			<!-- LOOPING COMPLETE - WRITE OUT THE DEFINITIONS FOR THE SYNAPSES -->
			<!-- NOTE: INTO SPLIT FILE -->
			<!--xsl:for-each select="$split_model_file//SMLLOWNL:Projection[@dst_population=$curr_split_pop/SMLLOWNL:Neuron/@name]/SMLLOWNL:Synapse/SMLLOWNL:WeightUpdate[substring-before(@name, '_sub')=$curr_syn_name]"-->
				<xsl:text>
</xsl:text>
				<!-- HEADING -->// <xsl:value-of select="@name"/> ##########################
<!---->							
				<!-- AL COMPONENT FOR THIS SYNAPSE -->
				<xsl:variable name="linked_syn_file" select="document(@url)"/>
				<!-- ok, quick comment on numbering - I chose to give every Synapse a unique number - this means that numbers are less meaningful, but then they were never particularly meaningful anyway! -->
				<xsl:variable name="suffix_name">
					<xsl:value-of select="translate($curr_synapse/@name,' -','__')"/><!-- NEW SUFFIX -->
				</xsl:variable>
					
				<xsl:for-each select="$linked_syn_file//SMLCL:ComponentClass">
				
					<!-- ADJUSTED VARIABLES DEPENDING UPON THE CONNECTIVITY  -->
					<xsl:choose>
						<!-- IF ONE TO ONE CONNECTION ###############################-->
						<xsl:when test="count($curr_synapse/../SMLNL:OneToOneConnection)=1">
							<!-- PARAMETERS -->
							<!---->	// parameters:
<!---->						<!-- choose whether we use an array or not:
							if (we have a random value tag or and explicit list) 
							THEN it is an array and we should add the array notation.		            	
							 -->
							<xsl:for-each select="SMLCL:Parameter">
								<xsl:variable name="par_name" select="@name"/>
								<!-- WHEN PROBABILITY -->
								<xsl:for-each select="$curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution">
								<!---->	for (i=0; i &lt; <xsl:value-of select="$number"/>; i += 1) {
<!---->							<!---->		for (j=0; j &lt; <xsl:value-of select="$n_dst"/>; j += 1) {
<!---->							<!----><xsl:text>			</xsl:text><xsl:value-of select="../@name"/><xsl:value-of select="$suffix"/>[i][j] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->							<!---->		}
<!---->							<!---->	}
<!---->							</xsl:for-each>
							</xsl:for-each>
							<!--STATE VARIABLES -->
							<xsl:for-each select="$linked_syn_file//SMLCL:StateVariable">
								<xsl:variable name="par_name" select="@name"/>
								<!---->	// state variables:
<!---->							<!-- WHEN FIXED -->
								<xsl:for-each select="$curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:FixedValue">
								<!---->	for (i=0; i &lt; <xsl:value-of select="$number"/>; i += 1) {
<!---->							<!---->		for (j=0; j &lt; <xsl:value-of select="$n_dst"/>; j += 1) {
<!---->							<!----><xsl:text>			</xsl:text><xsl:value-of select="../@name"/><xsl:value-of select="$suffix"/>[i][j] = <xsl:value-of select="@value"/>;
<!---->							<!---->		}
<!---->							<!---->	}
<!---->							</xsl:for-each>
<!---->							<!-- WHEN PROB -->
								<xsl:for-each select="$curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution">
								<!---->	for (i=0; i &lt; <xsl:value-of select="$number"/>; i += 1) {
<!---->							<!---->		for (j=0; j &lt; <xsl:value-of select="$n_dst"/>; j += 1) {
<!---->							<!----><xsl:text>			</xsl:text><xsl:value-of select="../@name"/><xsl:value-of select="$suffix"/>[i][j] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->							<!---->		}
<!---->							<!---->	}
<!---->							</xsl:for-each>
							</xsl:for-each>
							<!-- ADD THE ESSENTIAL VARIABLES -->
							<!-- WE DON'T NEED A REGIME VARIABLE IF WE HAVE ONLY ONE REGIME -->
							<xsl:if test="count(//SMLCL:Regime)>1">
								<!---->// regime variable:
<!---->							<!-- WHEN FIXED -->
								<xsl:for-each select="$curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:FixedValue">
								<!---->	for (i=0; i &lt; <xsl:value-of select="$number"/>; i += 1) {
<!---->							<!---->		for (j=0; j &lt; <xsl:value-of select="$n_dst"/>; j += 1) {
<!---->							<!---->			regimeCurrSD<xsl:value-of select="$suffix"/>[i][j] = 0;
<!---->							<!---->		}
<!---->							<!---->	}
<!---->							</xsl:for-each>
<!---->						</xsl:if>
							<!-- NO CONNECTIVITY REQUIRED FOR ONE TO ONE -->
							<!---->	//delays
<!---->						<!-- IF DELAYS ARE PROBABALISTIC -->
							<xsl:for-each select="$curr_synapse/../SMLNL:OneToOneConnection//SMLNL:UniformDistribution">
								<!---->	for (i=0; i &lt; <xsl:value-of select="$number"/>; i += 1) {
<!---->							<!---->		for (j=0; j &lt; <xsl:value-of select="$n_dst"/>; j += 1) {
<!---->							<!---->			mainDelay<xsl:value-of select="$suffix"/>[i][j] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->							<!---->		}
<!---->							<!---->	}
<!---->						</xsl:for-each>	
							<!-- IF DELAYS ARE EXPLICIT (THEY ARE NOT SUPPORTED FOR 1-2-1 CONNECTIONS) -->
							<xsl:if test="count($curr_synapse/../SMLNL:FixedProbabilityConnection//SMLNL:ValueList)=1">
								<!---->!!! Error - Explicit delays are not supported for onetoone connections (<xsl:value-of select="$curr_synapse/@name"/>) !!!
<!---->						</xsl:if>
						</xsl:when>


						<!-- IF ALL TO ALL CONNECTION ###############################-->
						<xsl:when test="count($curr_synapse/../SMLNL:AllToAllConnection)=1">
							<!-- PARAMETERS -->
							<!---->	// parameters:
<!---->						<!-- choose whether we use an array or not:
							if (we have a random value tag or and explicit list) 
							THEN it is an array and we should add the array notation.		            	
							 -->
							<xsl:for-each select="SMLCL:Parameter">
								<xsl:variable name="par_name" select="@name"/>
								<!-- WHEN PROBABILITY -->
								<xsl:for-each select="$curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution">
								<!---->	for (i=0; i &lt; <xsl:value-of select="$number"/>; i += 1) {
<!---->							<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->							<!---->			for (k=0; k &lt; <xsl:value-of select="$n_dst"/>; k += 1) {
<!---->							<!---->				tempFloats[k] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->							<!---->			}
<!---->							<!---->			writesdram(&amp;<xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/>[i][j][0], &amp;tempFloats, <xsl:value-of select="$n_dst"/>);
<!---->							<!---->		}
<!---->							<!---->	}
<!---->							</xsl:for-each>
							</xsl:for-each>
							<!--STATE VARIABLES -->
							<xsl:for-each select="$linked_syn_file//SMLCL:StateVariable">
								<xsl:variable name="par_name" select="@name"/>
								<!---->	// state variables:
<!---->							<!-- WHEN FIXED -->
								<xsl:for-each select="$curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:FixedValue">
								<!---->	for (i=0; i &lt; <xsl:value-of select="$number"/>; i += 1) {
<!---->							<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->							<!---->			for (k=0; k &lt; <xsl:value-of select="$n_dst"/>; k += 1) {
<!---->							<!---->				tempFloats[k] = <xsl:value-of select="@value"/>;
<!---->							<!---->			}
<!---->							<!---->			writesdram(&amp;<xsl:value-of select="$par_name"/>SD<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempFloats, <xsl:value-of select="$n_dst"/>);
<!---->							<!---->		}
<!---->							<!---->	}
<!---->							</xsl:for-each>
								<!-- WHEN PROBABILITY -->
								<xsl:for-each select="$curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution">
								<!---->	for (i=0; i &lt; <xsl:value-of select="$number"/>; i += 1) {
<!---->							<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->							<!---->			for (k=0; k &lt; <xsl:value-of select="$n_dst"/>; k += 1) {
<!---->							<!---->				tempFloats[k] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->							<!---->			}
<!---->							<!---->			writesdram(&amp;<xsl:value-of select="$par_name"/>SD<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempFloats, <xsl:value-of select="$n_dst"/>);
<!---->							<!---->		}
<!---->							<!---->	}
<!---->							</xsl:for-each>
<!---->						</xsl:for-each>
							<!-- ADD THE ESSENTIAL VARIABLES -->
							<!-- WE DON'T NEED A REGIME VARIABLE IF WE HAVE ONLY ONE REGIME -->
							<xsl:if test="count(//SMLCL:Regime)>1">
								<!---->	// regime variable:
								<!---->	for (i=0; i &lt; <xsl:value-of select="$number"/>; i += 1) {
<!---->							<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->							<!---->			for (k=0; k &lt; <xsl:value-of select="$n_dst"/>; k += 1) {
<!---->							<!---->				tempInts[k] = 0;
<!---->							<!---->			}
<!---->							<!---->			writesdram(&amp;regimeCurrSD<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempInts, <xsl:value-of select="$n_dst"/>);
<!---->							<!---->		}
<!---->							<!---->	}
<!---->						</xsl:if>
							<!-- CONNECTION INIT NOT NEEDED FOR A2A -->
							<!---->	// delays	
<!---->						<!-- IF DELAYS ARE PROBABALISTIC -->
							<xsl:for-each select="$curr_synapse/../SMLNL:AllToAllConnection//SMLNL:UniformDistribution">
								<!---->	for (i=0; i &lt; <xsl:value-of select="$number"/>; i += 1) {
<!---->							<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->							<!---->			for (k=0; k &lt; <xsl:value-of select="$n_dst"/>; k += 1) {
<!---->							<!---->				tempFloats[k] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->							<!---->			}
<!---->							<!---->			writesdram(&amp;mainDelaySD<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempFloats, <xsl:value-of select="$n_dst"/>);
<!---->							<!---->		}
<!---->							<!---->	}
<!---->						</xsl:for-each>
					
						</xsl:when>
						<!-- IF FIXED PROBABILITY CONNECTION ###############################-->
						<xsl:when test="count($curr_synapse/../SMLNL:FixedProbabilityConnection)=1">
							<xsl:variable name="n_prob" select="ceiling(6.0*number($n_dst)*number($curr_synapse/../SMLNL:FixedProbabilityConnection/@probability))"/>
							<!-- THIS ONE IS A BIT DIFFERENT, AS WE NEED TO FILL IN ALL THE SD 
							RAM ARRAYS WITH THE NUMBER OF ITEMS GENERATED FOR THE CONNECTION -->
							<!-- CONNECTIVITY -->
							<!---->	// connectivity
<!---->							<!---->	for (i=0; i &lt; <xsl:value-of select="$number"/>; i += 1) {
<!---->							<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->							<!---->			counter = 0;
<!---->							<!---->			for (k=0; k &lt; num_neurons_actual; k += 1) {
<!---->							<!---->				if (uniformRNG(1.0, 0.0) &lt; <xsl:value-of select="$curr_synapse/../SMLNL:FixedProbabilityConnection/@probability"/>) {	
<!---->							<!---->					tempInts[counter] = k;
<!---->							<!---->					counter += 1;
<!---->							<!---->					if (counter > <xsl:value-of select="number($n_prob)-2"/>) {printf("Error - too many connectivity values for FP conn\n"); exit(0);}
<!---->							<!---->				}
<!---->							<!---->			}
<!---->							<!---->			connRowLengths[j] = counter; // for use in pars init
<!---->							<!---->			tempInts[counter] = -1; // terminate list
<!---->							<!---->			writesdram(&amp;mainConnectionSD<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempInts, <xsl:value-of select="$n_prob"/>);
<!---->							<!---->		}
<!---->							<!---->	}				
<!---->						<!-- PARAMETERS -->
							<!---->	// parameters:
<!---->						<xsl:for-each select="SMLCL:Parameter">
								<xsl:variable name="par_name" select="@name"/>
								<!-- WHEN PROBABILITY -->
								<xsl:for-each select="$curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution">
								<!---->	for (i=0; i &lt; <xsl:value-of select="$number"/>; i += 1) {
<!---->							<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->							<!---->			for (k=0; k &lt; connRowLengths[j]; k += 1) {
<!---->							<!---->				tempFloats[k] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->							<!---->			}
<!---->							<!---->			writesdram(&amp;<xsl:value-of select="$par_name"/>SD<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempFloats, <xsl:value-of select="$n_prob"/>);
<!---->							<!---->		}
<!---->							<!---->	}
<!---->							</xsl:for-each>
							</xsl:for-each>
							<!--STATE VARIABLES -->
							<xsl:for-each select="$linked_syn_file//SMLCL:StateVariable">
								<xsl:variable name="par_name" select="@name"/>
								<!---->	// state variables:
<!---->							<!-- WHEN FIXED -->
								<xsl:for-each select="$curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:FixedValue">
								<!---->	for (i=0; i &lt; <xsl:value-of select="$number"/>; i += 1) {
<!---->							<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->							<!---->			for (k=0; k &lt; connRowLengths[j]; k += 1) {
<!---->							<!---->				tempFloats[k] = <xsl:value-of select="@value"/>;
<!---->							<!---->			}
<!---->							<!---->			writesdram(&amp;<xsl:value-of select="$par_name"/>SD<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempFloats, <xsl:value-of select="$n_prob"/>);
<!---->							<!---->		}
<!---->							<!---->	}
<!---->							</xsl:for-each>
								<!-- WHEN PROBABILITY -->
								<xsl:for-each select="$curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution">
								<!---->	for (i=0; i &lt; <xsl:value-of select="$number"/>; i += 1) {
<!---->							<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->							<!---->			for (k=0; k &lt; connRowLengths[j]; k += 1) {
<!---->							<!---->				tempFloats[k] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->							<!---->			}
<!---->							<!---->			writesdram(&amp;<xsl:value-of select="$par_name"/>SD<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempFloats, <xsl:value-of select="$n_prob"/>);
<!---->							<!---->		}
<!---->							<!---->	}
<!---->							</xsl:for-each>
<!---->						</xsl:for-each>
							<!-- ADD THE ESSENTIAL VARIABLES -->
							<!-- WE DON'T NEED A REGIME VARIABLE IF WE HAVE ONLY ONE REGIME -->
							<xsl:if test="count(//SMLCL:Regime)>1">
								<!---->	// regime variable:<!---->
								<!---->	for (i=0; i &lt; <xsl:value-of select="$number"/>; i += 1) {
<!---->							<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->							<!---->			for (k=0; k &lt; connRowLengths[j]; k += 1) {
<!---->							<!---->				tempFloats[k] = 0;
<!---->							<!---->			}
<!---->							<!---->			writesdram(&amp;regimeCurrSD<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempFloats, <xsl:value-of select="$n_prob"/>);
<!---->							<!---->		}
<!---->							<!---->	}
<!---->						</xsl:if>

<!---->						<!---->	// delays
<!---->						<!-- IF DELAYS ARE PROBABALISTIC -->
<!---->						<xsl:for-each select="$curr_synapse/../SMLNL:FixedProbabilityConnection//SMLNL:UniformDistribution">
								<!---->	for (i=0; i &lt; <xsl:value-of select="$number"/>; i += 1) {
<!---->							<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->							<!---->			for (k=0; k &lt; connRowLengths[j]; k += 1) {
<!---->							<!---->				tempFloats[k] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->							<!---->			}
<!---->							<!---->			writesdram(&amp;mainDelaySD<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempFloats, <xsl:value-of select="$n_prob"/>);
<!---->							<!---->		}
<!---->							<!---->	}
<!---->						</xsl:for-each>
						</xsl:when>
						<!-- IF EXPLICIT LIST CONNECTION ###############################-->
						<xsl:when test="count($curr_synapse/../SMLNL:ConnectionList)=1">
							<!-- FOR THIS TYPE WE MUST FETCH ROWS FROM SDRAM TO FILL IN ONLY
							THE PARS WITH CORRESPONDING CONNECTIONS -->
							<!-- PARAMETERS -->
							<!---->	// parameters:
<!---->						<xsl:for-each select="SMLCL:Parameter">
								<xsl:variable name="par_name" select="@name"/>
								<!-- WHEN PROBABILITY-->
								<xsl:for-each select="$curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution">
								<!---->	for (i=0; i &lt; <xsl:value-of select="$number"/>; i += 1) {
<!---->							<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->							<!---->			// fetch connectivity
<!---->							<!---->			readsdram(&amp;mainConnectionSD<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempInts, <xsl:value-of select="$n_dst"/>);
<!---->							<!---->			for (k=0; k &lt; <xsl:value-of select="$n_dst"/>; k += 1) {
<!---->							<!---->				if (tempInts[k]==1) {
<!---->							<!---->					tempFloats[k] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->							<!---->				}
<!---->							<!---->			}
<!---->							<!---->			writesdram(&amp;<xsl:value-of select="$par_name"/>SD<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempFloats, <xsl:value-of select="$n_dst"/>);
<!---->							<!---->		}
<!---->							<!---->	}
<!---->							</xsl:for-each>
							</xsl:for-each>
							<!--STATE VARIABLES -->
							<xsl:for-each select="$linked_syn_file//SMLCL:StateVariable">
								<xsl:variable name="par_name" select="@name"/>
								<!---->	// state variables:
<!---->							<!-- WHEN FIXED -->
								<xsl:for-each select="$curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:FixedValue">
								<!---->	for (i=0; i &lt; <xsl:value-of select="$number"/>; i += 1) {
<!---->							<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->							<!---->			for (k=0; k &lt; <xsl:value-of select="$n_dst"/>; k += 1) {
<!---->							<!---->				tempFloats[k] = <xsl:value-of select="@value"/>;
<!---->							<!---->			}
<!---->							<!---->			writesdram(&amp;<xsl:value-of select="$par_name"/>SD<xsl:value-of select="$suffix"/>[i][j], &amp;tempFloats, <xsl:value-of select="$n_dst"/>);
<!---->							<!---->		}
<!---->							<!---->	}
<!---->							</xsl:for-each>
<!---->							<!-- WHEN PROBABILITY-->
								<xsl:for-each select="$curr_synapse/SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution">
								<!---->	for (i=0; i &lt; <xsl:value-of select="$number"/>; i += 1) {
<!---->							<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->							<!---->			// fetch connectivity
<!---->							<!---->			readsdram(&amp;mainConnectionSD<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempInts, <xsl:value-of select="$n_dst"/>);
<!---->							<!---->			for (k=0; k &lt; <xsl:value-of select="$n_dst"/>; k += 1) {
<!---->							<!---->				if (tempInts[k]==1) {
<!---->							<!---->					tempFloats[k] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->							<!---->				}
<!---->							<!---->			}
<!---->							<!---->			writesdram(&amp;<xsl:value-of select="$par_name"/>SD<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempFloats, <xsl:value-of select="$n_dst"/>);
<!---->							<!---->		}
<!---->							<!---->	}
<!---->							</xsl:for-each>
<!---->						</xsl:for-each>
							<!-- ADD THE ESSENTIAL VARIABLES -->
							<!-- WE DON'T NEED A REGIME VARIABLE IF WE HAVE ONLY ONE REGIME -->
							<xsl:if test="count(//SMLCL:Regime)>1">
								<!---->	// regime variable:
								<!---->	for (i=0; i &lt; <xsl:value-of select="$number"/>; i += 1) {
<!---->							<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->							<!---->			for (k=0; k &lt; <xsl:value-of select="$n_dst"/>; k += 1) {
<!---->							<!---->				tempFloats[k] = 0;
<!---->							<!---->			}
<!---->							<!---->			writesdram(&amp;regimeCurrSD<xsl:value-of select="$suffix"/>[i][j], &amp;tempFloats, <xsl:value-of select="$n_dst"/>);
<!---->							<!---->		}
<!---->							<!---->	}
<!---->						</xsl:if>	
							<!---->	// delays
<!---->						<!-- IF DELAYS ARE PROBABALISTIC -->
							<xsl:for-each select="$curr_synapse/../SMLNL:ConnectionList//SMLNL:UniformDistribution">
								<!---->	for (i=0; i &lt; <xsl:value-of select="$number"/>; i += 1) {
<!---->							<!---->		for (j=0; j &lt; <xsl:value-of select="$n_src"/>; j += 1) {
<!---->							<!---->			// fetch connectivity
<!---->							<!---->			readsdram(&amp;mainConnectionSD<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempInts, <xsl:value-of select="$n_dst"/>);
<!---->							<!---->			for (k=0; k &lt; <xsl:value-of select="$n_dst"/>; k += 1) {
<!---->							<!---->				if (tempInts[k]==1) {
<!---->							<!---->					tempFloats[k] = uniformRNG((float) <xsl:value-of select="@maximum"/>, (float) <xsl:value-of select="@minimum"/>);
<!---->							<!---->				}
<!---->							<!---->			}
<!---->							<!---->			writesdram(&amp;mainDelaySD<xsl:value-of select="$suffix"/>[i][j][0], &amp;tempFloats, <xsl:value-of select="$n_dst"/>);
<!---->							<!---->		}
<!---->							<!---->	}
<!---->						</xsl:for-each>
						</xsl:when>
						<xsl:otherwise>
							<!---->!!! Error - connection type not recognised (<xsl:value-of select="$curr_synapse/@name"/>) !!!
<!---->					</xsl:otherwise>
					</xsl:choose>
				</xsl:for-each>

		</xsl:otherwise>
	</xsl:choose>

</xsl:if>

<!-- IF WE HAVE NO LIST BECAUSE THIS IS THE FIRST CALL OF THE TEMPLATE -->
<xsl:if test="$split_pops='none'">
	<!-- FIND ALL THE POPULATIONS IN THE SPLIT FILE WITH THE SAME BASE NAME AS THE UNSPLIT POPULATION -->
	<xsl:variable name="split_pops_new" select="$split_model_file//SMLLOWNL:Population[substring-before(SMLLOWNL:Neuron/@name, '_sub')=$curr_pop_name]"/>
	<!-- RE-CALL TEMPLATE WITH THIS LIST -->
	<xsl:apply-templates select="." mode="initData">
		<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
		<xsl:with-param name="split_model_file" select="$split_model_file"/>
		<xsl:with-param name="split_pops" select="$split_pops_new"/>
		<xsl:with-param name="n_dst" select="$n_dst"/>
		<xsl:with-param name="n_src" select="$n_src"/>
	</xsl:apply-templates>
</xsl:if>

</xsl:template>

<!-- ################################################
#													#
#		INITIALISE EXPLICIT DATA FOR SYNAPSE 		#
#													#
##################################################-->

<xsl:template match="SMLLOWNL:WeightUpdate" mode="initExplicitData">

<xsl:param name="curr_pop_name"/>
<xsl:param name="curr_split_pop_name"/>
<xsl:param name="split_model_file"/>
<xsl:param name="number" select="number('0')"/>
<xsl:param name="curr_split_pop" select="''"/>
<xsl:param name="split_pops" select="'none'"/>
<xsl:param name="n_dst"/>
<xsl:param name="n_src"/>

<xsl:variable name="curr_pop_name_sanitised" select="translate($curr_pop_name, ' -', 'oo')"/>
<xsl:variable name="curr_component_name" select="translate(document(/SMLLOWNL:Neuron[@name=$curr_pop_name]/@url)/SMLCL:ComponentClass/@name, ' -', 'oo')"/>
<xsl:variable name="curr_synapse" select="."/>
<!-- SUFFIX IS EASY AS COMPONENT MUST BE A SYNAPSE -->
<xsl:variable name="suffix">
	<xsl:value-of select="translate($curr_synapse/@name,' -','__')"/><!-- NEW SUFFIX -->
</xsl:variable>

<!-- IF THIS IS NOT THE FIRST CALL OF THE TEMPLATE -->
<xsl:if test="not($split_pops='none')">
	<xsl:variable name="curr_syn_name" select="@name"/>
	<xsl:choose>
		<xsl:when test="count($split_pops)>0">
			<!-- GET THE SPLIT INDEX OF THE CURRENT SPLIT POPULATION -->
			<xsl:variable name="curr_split_pop_num" select="substring-after($split_pops[1]/SMLLOWNL:Neuron/@name, '_sub')"/>
			<!-- CREATE A VARIABLE WITH THE NUMBER OF INPUTS TO THE CURRENT SPLIT POP BY:
				1) finding the projections in the split file that target the current split population
				2) making sure that these are derived from the current synapse	
				3) this is enough to ensure that we only have the synapses from the rigt projection that target the right population		
			 -->
			<xsl:variable name="num_curr_pop_proj_inputs" select="count($split_model_file//SMLLOWNL:Projection[@dst_population=$split_pops[1]/SMLLOWNL:Neuron/@name]/SMLLOWNL:Synapse/SMLLOWNL:WeightUpdate[substring-before(@name, '_sub')=$curr_syn_name])"/>
		 	<xsl:choose>
		 		<!-- IF THE NUMBER OF INPUTS IS GREATER THAN THE PREVIOUS HIGHEST -->
		 		<xsl:when test="$num_curr_pop_proj_inputs>$number">
		 			<!-- CALL TEMPLATE WITH NEW POP AND NEW NUMBER OF INPUTS -->
		 			<xsl:apply-templates select="$curr_syn_name/.." mode="initExplicitData">
						<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
						<xsl:with-param name="split_model_file" select="$split_model_file"/>
						<xsl:with-param name="number" select="$num_curr_pop_proj_inputs"/>
						<xsl:with-param name="curr_split_pop" select="$split_pops[1]"/>
						<xsl:with-param name="split_pops" select="$split_pops[position()&gt;1]"/>
						<xsl:with-param name="curr_split_pop_name" select="$curr_split_pop_name"/>
						<xsl:with-param name="n_dst" select="$n_dst"/>
						<xsl:with-param name="n_src" select="$n_src"/>
					</xsl:apply-templates>
		 		</xsl:when>
		 		<!-- OTHERWISE -->
		 		<xsl:otherwise>
		 			<!-- PASS ON CURRENT POP AND NUMBER -->
		 			<xsl:apply-templates select="$curr_syn_name/.." mode="initExplicitData">
						<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
						<xsl:with-param name="split_model_file" select="$split_model_file"/>
						<xsl:with-param name="number" select="$number"/>
						<xsl:with-param name="curr_split_pop" select="$curr_split_pop"/>
						<xsl:with-param name="split_pops" select="$split_pops[position()&gt;1]"/>
						<xsl:with-param name="curr_split_pop_name" select="$curr_split_pop_name"/>
						<xsl:with-param name="n_dst" select="$n_dst"/>
						<xsl:with-param name="n_src" select="$n_src"/>
					</xsl:apply-templates>
		 		</xsl:otherwise>
		 	</xsl:choose>	 		
		</xsl:when>
		<xsl:otherwise>
			<!-- LOOPING COMPLETE - WRITE OUT THE LISTS FOR THE SYNAPSES -->
			<xsl:text>
</xsl:text>		<!-- HEADING -->// <xsl:value-of select="@name"/> ##########################
<!---->							
			<!-- AL COMPONENT FOR THIS SYNAPSE -->
			<xsl:variable name="linked_syn_file" select="document(@url)"/>
			<!-- ok, quick comment on numbering - I chose to give every Synapse a unique number - this means that numbers are less meaningful, but then they were never particularly meaningful anyway! -->
				<xsl:variable name="suffix_name">
					<xsl:value-of select="translate($curr_synapse/@name,' -','__')"/><!-- NEW SUFFIX -->
				</xsl:variable>
				
			<xsl:for-each select="$linked_syn_file//SMLCL:ComponentClass">
			
				<!-- ADJUSTED VARIABLES DEPENDING UPON THE CONNECTIVITY  -->
				<xsl:choose>
					<!-- IF ONE TO ONE CONNECTION ###############################-->
					<xsl:when test="count($curr_synapse/../SMLNL:OneToOneConnection)=1">
						<!-- FOR EACH UNSPLIT PROPERTY THAT USES AN EXPLICIT LIST -->
						<xsl:for-each select="$curr_synapse/SMLNL:Property[count(SMLNL:ValueList)=1]">
							<xsl:variable name="curr_prop_name" select="@name"/>
							<xsl:value-of select="@name"/><xsl:value-of select="$suffix"/> = {
<!---->						<!-- FOR EACH SPLIT OF THIS PROPERTY IN THE SPLIT MODEL -->
							<xsl:for-each select="$split_model_file//SMLLOWNL:Projection[@dst_population=$curr_split_pop_name]//SMLLOWNL:WeightUpdate[substring-before(@name, '_sub')=$curr_synapse/@name]/SMLNL:Property[@name=$curr_prop_name]">
								<xsl:variable name="curr_prop" select="."/>
								<!---->{<!---->
									<xsl:call-template name="PropertyExplicitVals121">
										<xsl:with-param name="input" select="."/>
										<xsl:with-param name="length" select="$n_dst"/>
									</xsl:call-template>
								<!---->}<!---->
								<xsl:if test="not(position()=$number)">
									<!---->,
<!---->							</xsl:if>
							</xsl:for-each>
							<!-- ADD EXTRA LINES SO THAT THE DIMENSIONALITY IS CORRECT -->
							<xsl:variable name="line">
								<!---->{<!---->
								<xsl:call-template name="RepeatString">
								  <xsl:with-param name="string" select="'0'" />
								  <xsl:with-param name="divider" select="', '" />
								  <xsl:with-param name="times"  select="$n_dst"/>
								</xsl:call-template>
								<!---->}<!---->
							</xsl:variable>
							<xsl:call-template name="RepeatString">
							  <xsl:with-param name="string" select="$line" />
							  <xsl:with-param name="divider" select="',&#13;'" />
							  <xsl:with-param name="times"  select="$number - count($curr_synapse/SMLNL:Property[count(SMLNL:ValueList)=1])"/>
							</xsl:call-template>
							<!---->};
<!---->					</xsl:for-each>
					</xsl:when>
					<!-- IF ALL TO ALL CONNECTION ###############################-->
					<xsl:when test="count($curr_synapse/../SMLNL:AllToAllConnection)=1">
						<!-- FOR EACH UNSPLIT PROPERTY THAT USES AN EXPLICIT LIST -->
						<xsl:for-each select="$curr_synapse/SMLNL:Property[count(SMLNL:ValueList)=1]">
							<xsl:variable name="curr_prop_name" select="@name"/>
							<xsl:value-of select="@name"/><xsl:value-of select="$suffix"/> = {
<!---->						<!-- FOR EACH SPLIT OF THIS PROPERTY IN THE SPLIT MODEL -->
							<xsl:for-each select="$split_model_file//SMLLOWNL:Projection[@dst_population=$curr_split_pop_name]//SMLLOWNL:WeightUpdate[substring-before(@name, '_sub')=$curr_synapse/@name]/SMLNL:Property[@name=$curr_prop_name]">
								<xsl:variable name="curr_prop" select="."/>
								<!---->{<!---->
									<xsl:call-template name="PropertyExplicitValsA2A">
										<xsl:with-param name="input" select="."/>
										<xsl:with-param name="maxRow" select="n_src"/>
										<xsl:with-param name="maxCol" select="$n_dst"/>
									</xsl:call-template>
								<!---->}<!---->
								<xsl:if test="not(position()=$number)">
									<!---->,
<!---->							</xsl:if>
							</xsl:for-each>
							<!-- ADD EXTRA LINES SO THAT THE DIMENSIONALITY IS CORRECT -->
							<xsl:variable name="line">
								<!---->{<!---->
								<xsl:call-template name="RepeatString">
								  <xsl:with-param name="string" select="'0'" />
								  <xsl:with-param name="divider" select="', '" />
								  <xsl:with-param name="times"  select="$n_dst"/>
								</xsl:call-template>
								<!---->}<!---->
							</xsl:variable>
							<xsl:variable name="block">
								<!---->{<!---->
								<xsl:call-template name="RepeatString">
								  <xsl:with-param name="string" select="$line" />
								  <xsl:with-param name="divider" select="', '" />
								  <xsl:with-param name="times"  select="$n_src"/>
								</xsl:call-template>
								<!---->}<!---->
							</xsl:variable>
							<xsl:call-template name="RepeatString">
							  <xsl:with-param name="string" select="$block" />
							  <xsl:with-param name="divider" select="',&#13;'" />
							  <xsl:with-param name="times"  select="$number - count($curr_synapse/SMLNL:Property[count(SMLNL:ValueList)=1])"/>
							</xsl:call-template>
							<!---->};
<!---->					</xsl:for-each>
					</xsl:when>
					<!-- IF FIXED PROBABILITY CONNECTION ###############################-->
					<xsl:when test="count($curr_synapse/../SMLNL:FixedProbabilityConnection)=1">
						<!-- EXPLICIT LIST PROPERTIES ARE NOT POSSIBLE FOR THIS CONNECTION TYPE! -->
					</xsl:when>
					<!-- IF EXPLICIT LIST CONNECTION ###############################-->
					<xsl:when test="count($curr_synapse/../SMLNL:ConnectionList)=1">
						<!-- FOR EACH UNSPLIT PROPERTY THAT USES AN EXPLICIT LIST -->
						<xsl:for-each select="$curr_synapse/SMLNL:Property[count(SMLNL:ValueList)=1]">
							<xsl:variable name="curr_prop_name" select="@name"/>
							<xsl:value-of select="@name"/><xsl:value-of select="$suffix"/> = {
<!---->						<!-- FOR EACH SPLIT OF THIS PROPERTY IN THE SPLIT MODEL -->
							<xsl:for-each select="$split_model_file//SMLLOWNL:Projection[@dst_population=$curr_split_pop_name]//SMLLOWNL:WeightUpdate[substring-before(@name, '_sub')=$curr_synapse/@name]/SMLNL:Property[@name=$curr_prop_name]">
								<xsl:variable name="curr_prop" select="."/>
								<!---->{<!---->
									<xsl:call-template name="PropertyExplicitValsA2A">
										<xsl:with-param name="input" select="."/>
										<xsl:with-param name="maxRow" select="$split_model_file//SMLLOWNL:Neuron[@name=$curr_prop/../../../@dst_population]/@size"/>
										<xsl:with-param name="maxCol" select="$curr_prop/../../../../SMLLOWNL:Neuron/@size"/>
									</xsl:call-template>
								<!---->}<!---->
								<xsl:if test="not(position()=$number)">
									<!---->,
<!---->							</xsl:if>
							</xsl:for-each>
							<!-- ADD EXTRA LINES SO THAT THE DIMENSIONALITY IS CORRECT -->
							<xsl:variable name="line">
								<!---->{<!---->
								<xsl:call-template name="RepeatString">
								  <xsl:with-param name="string" select="'0'" />
								  <xsl:with-param name="divider" select="', '" />
								  <xsl:with-param name="times"  select="$split_model_file//SMLLOWNL:Neuron[@name=$split_model_file//SMLLOWNL:WeightUpdate[substring-before(@name, '_sub')=$curr_synapse/@name]/../../@dst_population]/@size"/>
								</xsl:call-template>
								<!---->}<!---->
							</xsl:variable>
							<xsl:variable name="block">
								<!---->{<!---->
								<xsl:call-template name="RepeatString">
								  <xsl:with-param name="string" select="$line" />
								  <xsl:with-param name="divider" select="', '" />
								  <xsl:with-param name="times"  select="$split_model_file//SMLLOWNL:WeightUpdate[substring-before(@name, '_sub')=$curr_synapse/@name]/../../../SMLLOWNL:Neuron/@size"/>
								</xsl:call-template>
								<!---->}<!---->
							</xsl:variable>
							<xsl:call-template name="RepeatString">
							  <xsl:with-param name="string" select="$block" />
							  <xsl:with-param name="divider" select="',&#13;'" />
							  <xsl:with-param name="times"  select="$number - count($curr_synapse/SMLNL:Property[count(SMLNL:ValueList)=1])"/>
							</xsl:call-template>
							<!---->};
<!---->					</xsl:for-each>
					</xsl:when>
					<xsl:otherwise>
						<!---->!!! Error - connection type not recognised (<xsl:value-of select="$curr_synapse/@name"/>) !!!
<!---->					</xsl:otherwise>
				</xsl:choose>
			</xsl:for-each>

		</xsl:otherwise>
	</xsl:choose>

</xsl:if>

<!-- IF WE HAVE NO LIST BECAUSE THIS IS THE FIRST CALL OF THE TEMPLATE -->
<xsl:if test="$split_pops='none'">
	<!-- FIND ALL THE POPULATIONS IN THE SPLIT FILE WITH THE SAME BASE NAME AS THE UNSPLIT POPULATION -->
	<xsl:variable name="split_pops_new" select="$split_model_file//SMLLOWNL:Population[substring-before(SMLLOWNL:Neuron/@name, '_sub')=$curr_pop_name]"/>
	<!-- RE-CALL TEMPLATE WITH THIS LIST -->
	<xsl:apply-templates select="." mode="initExplicitData">
		<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
		<xsl:with-param name="split_model_file" select="$split_model_file"/>
		<xsl:with-param name="split_pops" select="$split_pops_new"/>
		<xsl:with-param name="curr_split_pop_name" select="$curr_split_pop_name"/>
		<xsl:with-param name="n_dst" select="$n_dst"/>
		<xsl:with-param name="n_src" select="$n_src"/>
	</xsl:apply-templates>
</xsl:if>

</xsl:template>

<!-- ################################################
#													#
#   INITIALISE EXPLICIT CONNECTIONS FOR SYNAPSE 	#
#													#
##################################################-->

<xsl:template match="SMLLOWNL:WeightUpdate" mode="initExplicitConns">

<xsl:param name="curr_pop_name"/>
<xsl:param name="curr_split_pop_name"/>
<xsl:param name="split_model_file"/>
<xsl:param name="number" select="number('0')"/>
<xsl:param name="curr_split_pop" select="''"/>
<xsl:param name="split_pops" select="'none'"/>
<xsl:param name="n_dst"/>
<xsl:param name="n_src"/>

<xsl:variable name="curr_pop_name_sanitised" select="translate($curr_pop_name, ' -', 'oo')"/>
<xsl:variable name="curr_component_name" select="translate(document(/SMLLOWNL:Neuron[@name=$curr_pop_name]/@url)/SMLCL:ComponentClass/@name, ' -', 'oo')"/>
<xsl:variable name="curr_synapse" select="."/>
<!-- SUFFIX IS EASY AS COMPONENT MUST BE A SYNAPSE -->
<xsl:variable name="suffix">
	<xsl:value-of select="translate($curr_synapse/@name,' -','__')"/><!-- NEW SUFFIX -->
</xsl:variable>

<!-- IF THIS IS NOT THE FIRST CALL OF THE TEMPLATE -->
<xsl:if test="not($split_pops='none')">
	<xsl:variable name="curr_syn_name" select="@name"/>
	<xsl:choose>
		<xsl:when test="count($split_pops)>0">
			<!-- GET THE SPLIT INDEX OF THE CURRENT SPLIT POPULATION -->
			<xsl:variable name="curr_split_pop_num" select="substring-after($split_pops[1]/SMLLOWNL:Neuron/@name, '_sub')"/>
			<!-- CREATE A VARIABLE WITH THE NUMBER OF INPUTS TO THE CURRENT SPLIT POP BY:
				1) finding the projections in the split file that target the current split population
				2) making sure that these are derived from the current synapse	
				3) this is enough to ensure that we only have the synapses from the rigt projection that target the right population		
			 -->
			<xsl:variable name="num_curr_pop_proj_inputs" select="count($split_model_file//SMLLOWNL:Projection[@dst_population=$split_pops[1]/SMLLOWNL:Neuron/@name]/SMLLOWNL:Synapse/SMLLOWNL:WeightUpdate[substring-before(@name, '_sub')=$curr_syn_name])"/>
		 	<xsl:choose>
		 		<!-- IF THE NUMBER OF INPUTS IS GREATER THAN THE PREVIOUS HIGHEST -->
		 		<xsl:when test="$num_curr_pop_proj_inputs>$number">
		 			<!-- CALL TEMPLATE WITH NEW POP AND NEW NUMBER OF INPUTS -->
		 			<xsl:apply-templates select="$curr_syn_name/.." mode="initExplicitConns">
		 				<xsl:with-param name="curr_split_pop_name" select="$curr_split_pop_name"/>
						<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
						<xsl:with-param name="split_model_file" select="$split_model_file"/>
						<xsl:with-param name="number" select="$num_curr_pop_proj_inputs"/>
						<xsl:with-param name="curr_split_pop" select="$split_pops[1]"/>
						<xsl:with-param name="split_pops" select="$split_pops[position()&gt;1]"/>
						<xsl:with-param name="n_dst" select="$n_dst"/>
						<xsl:with-param name="n_src" select="$n_src"/>
					</xsl:apply-templates>
		 		</xsl:when>
		 		<!-- OTHERWISE -->
		 		<xsl:otherwise>
		 			<!-- PASS ON CURRENT POP AND NUMBER -->
		 			<xsl:apply-templates select="$curr_syn_name/.." mode="initExplicitConns">
		 				<xsl:with-param name="curr_split_pop_name" select="$curr_split_pop_name"/>
						<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
						<xsl:with-param name="split_model_file" select="$split_model_file"/>
						<xsl:with-param name="number" select="$number"/>
						<xsl:with-param name="curr_split_pop" select="$curr_split_pop"/>
						<xsl:with-param name="split_pops" select="$split_pops[position()&gt;1]"/>
						<xsl:with-param name="n_dst" select="$n_dst"/>
						<xsl:with-param name="n_src" select="$n_src"/>
					</xsl:apply-templates>
		 		</xsl:otherwise>
		 	</xsl:choose>	 		
		</xsl:when>
		<xsl:otherwise>
			<!-- LOOPING COMPLETE - WRITE OUT THE LISTS FOR THE SYNAPSES -->
			<xsl:text>
</xsl:text>		<!-- HEADING -->// <xsl:value-of select="@name"/> ##########################
<!---->							
			<!-- AL COMPONENT FOR THIS SYNAPSE -->
			<xsl:variable name="linked_syn_file" select="document(@url)"/>
			<!-- ok, quick comment on numbering - I chose to give every Synapse a unique number - this means that numbers are less meaningful, but then they were never particularly meaningful anyway! -->
				<xsl:variable name="suffix_name">
					<xsl:value-of select="translate($curr_synapse/@name,' -','__')"/><!-- NEW SUFFIX -->
				</xsl:variable>
				
			<xsl:for-each select="$linked_syn_file//SMLCL:ComponentClass">
			
				<!-- ADJUSTED VARIABLES DEPENDING UPON THE CONNECTIVITY  -->
				<xsl:choose>
					<!-- IF ONE TO ONE CONNECTION ###############################-->
					<xsl:when test="count($curr_synapse/../SMLNL:OneToOneConnection)=1">
						<!-- NOT USED -->
					</xsl:when>
					<!-- IF ALL TO ALL CONNECTION ###############################-->
					<xsl:when test="count($curr_synapse/../SMLNL:AllToAllConnection)=1">
						<!-- NOT USED -->
					</xsl:when>
					<!-- IF FIXED PROBABILITY CONNECTION ###############################-->
					<xsl:when test="count($curr_synapse/../SMLNL:FixedProbabilityConnection)=1">
						<!-- NOT USED -->
					</xsl:when>
					<!-- IF EXPLICIT LIST CONNECTION ###############################-->
					<xsl:when test="count($curr_synapse/../SMLNL:ConnectionList)=1">
						<!-- FOR EACH UNSPLIT PROPERTY THAT USES AN EXPLICIT LIST -->
							<!---->mainConnectionSD<xsl:value-of select="$suffix"/> = {
<!---->						<!-- FOR EACH SPLIT OF THIS PROPERTY IN THE SPLIT MODEL -->
							<xsl:for-each select="$split_model_file//SMLLOWNL:WeightUpdate[substring-before(@name, '_sub')=$curr_synapse/@name and ../../@dst_population=$curr_split_pop_name]/../SMLNL:ConnectionList">
								<xsl:variable name="curr_conn" select="."/>
								<!---->{<!---->
									<xsl:call-template name="ConnectionExplicitVals">
										<xsl:with-param name="input" select="."/>
										<xsl:with-param name="maxRow" select="$n_dst"/>
										<xsl:with-param name="maxCol" select="$n_src"/>
									</xsl:call-template>
								<!---->}<!---->
								<xsl:if test="not(position()=$number)">
									<!---->,
<!---->							</xsl:if>
							</xsl:for-each>
							<!-- ADD EXTRA LINES SO THAT THE DIMENSIONALITY IS CORRECT -->
							<xsl:variable name="line">
								<!---->{<!---->
								<xsl:call-template name="RepeatString">
								  <xsl:with-param name="string" select="'0'" />
								  <xsl:with-param name="divider" select="','" />
								  <xsl:with-param name="times"  select="$n_dst"/>
								</xsl:call-template>
								<!---->}<!---->
							</xsl:variable>
							<xsl:variable name="block">
								<!---->{<!---->
								<xsl:call-template name="RepeatString">
								  <xsl:with-param name="string" select="$line" />
								  <xsl:with-param name="divider" select="',&#13;'" />
								  <xsl:with-param name="times"  select="$n_src"/>
								</xsl:call-template>
								<!---->}<!---->
							</xsl:variable>
							<xsl:call-template name="RepeatString">
							  <xsl:with-param name="string" select="$block" />
							  <xsl:with-param name="divider" select="',&#13;'" />
							  <xsl:with-param name="times"  select="$number - count($split_model_file//SMLLOWNL:WeightUpdate[substring-before(@name, '_sub')=$curr_synapse/@name and ../../@dst_population=$curr_split_pop_name]/../SMLNL:ConnectionList)"/>
							</xsl:call-template>
							<!---->};
<!---->						<xsl:if test="count($curr_synapse/../SMLNL:ConnectionList//@delay)>0">
								<!---->mainDelaySD<xsl:value-of select="$suffix"/> = {
<!---->							<!-- FOR EACH SPLIT OF THIS PROPERTY IN THE SPLIT MODEL -->
								<xsl:for-each select="$split_model_file//SMLLOWNL:WeightUpdate[substring-before(@name, '_sub')=$curr_synapse/@name and ../../@dst_population=$curr_split_pop_name]/../SMLNL:ConnectionList">
									<xsl:variable name="curr_conn" select="."/>
									<!---->{<!---->
										<xsl:call-template name="DelayExplicitVals">
											<xsl:with-param name="input" select="."/>
											<xsl:with-param name="maxRow" select="$n_dst"/>
											<xsl:with-param name="maxCol" select="$n_src"/>
										</xsl:call-template>
									<!---->}<!---->
									<xsl:if test="not(position()=$number)">
										<!---->,
<!---->								</xsl:if>
								</xsl:for-each>
								<!-- ADD EXTRA LINES SO THAT THE DIMENSIONALITY IS CORRECT -->
								<xsl:call-template name="RepeatString">
								  <xsl:with-param name="string" select="$block" />
								  <xsl:with-param name="divider" select="',&#13;'" />
								  <xsl:with-param name="times"  select="$number - count($split_model_file//SMLLOWNL:WeightUpdate[substring-before(@name, '_sub')=$curr_synapse/@name and ../../@dst_population=$curr_split_pop_name]/../SMLNL:ConnectionList)"/>
								</xsl:call-template>
								<!---->};
							</xsl:if> 
					</xsl:when>
					<xsl:otherwise>
						<!---->!!! Error - connection type not recognised (<xsl:value-of select="$curr_synapse/@name"/>) !!!
<!---->					</xsl:otherwise>
				</xsl:choose>
			</xsl:for-each>

		</xsl:otherwise>
	</xsl:choose>

</xsl:if>

<!-- IF WE HAVE NO LIST BECAUSE THIS IS THE FIRST CALL OF THE TEMPLATE -->
<xsl:if test="$split_pops='none'">
	<!-- FIND ALL THE POPULATIONS IN THE SPLIT FILE WITH THE SAME BASE NAME AS THE UNSPLIT POPULATION -->
	<xsl:variable name="split_pops_new" select="$split_model_file//SMLLOWNL:Population[substring-before(SMLLOWNL:Neuron/@name, '_sub')=$curr_pop_name]"/>
	<!-- RE-CALL TEMPLATE WITH THIS LIST -->
	<xsl:apply-templates select="." mode="initExplicitConns">
		 <xsl:with-param name="curr_split_pop_name" select="$curr_split_pop_name"/>
		<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
		<xsl:with-param name="split_model_file" select="$split_model_file"/>
		<xsl:with-param name="split_pops" select="$split_pops_new"/>
		<xsl:with-param name="n_dst" select="$n_dst"/>
		<xsl:with-param name="n_src" select="$n_src"/>
	</xsl:apply-templates>
</xsl:if>

</xsl:template>

<!-- ################################################
#													#
#		FULL THREAD FUNCTIONS						#
#													#
##################################################-->

<xsl:template match="SMLLOWNL:WeightUpdate" mode="threadFuncs">
<xsl:param name="curr_pop_name"/>
<xsl:param name="split_model_file"/>
<xsl:param name="number" select="number('0')"/>
<xsl:param name="curr_split_pop" select="''"/>
<xsl:param name="split_pops" select="'none'"/>
<xsl:param name="n_dst"/>
<xsl:param name="n_src"/>

<xsl:variable name="curr_pop_name_sanitised" select="translate($curr_pop_name, ' -', 'oo')"/>
<xsl:variable name="curr_component_name" select="translate(document(/SMLLOWNL:Neuron[@name=$curr_pop_name]/@url)/SMLCL:ComponentClass/@name, ' -', 'oo')"/>
<xsl:variable name="curr_synapse" select="."/>
<!-- SUFFIX IS EASY AS COMPONENT MUST BE A SYNAPSE -->
<xsl:variable name="suffix">
	<xsl:value-of select="translate($curr_synapse/@name,' -','__')"/><!-- NEW SUFFIX -->
</xsl:variable>

<!-- IF THIS IS NOT THE FIRST CALL OF THE TEMPLATE -->
<xsl:if test="not($split_pops='none')">
	<xsl:variable name="curr_syn_name" select="@name"/>
	<xsl:choose>
		<xsl:when test="count($split_pops)>0">
			<!-- GET THE SPLIT INDEX OF THE CURRENT SPLIT POPULATION -->
			<xsl:variable name="curr_split_pop_num" select="substring-after($split_pops[1]/SMLLOWNL:Neuron/@name, '_sub')"/>
			<!-- CREATE A VARIABLE WITH THE NUMBER OF INPUTS TO THE CURRENT SPLIT POP BY:
				1) finding the projections in the split file that target the current split population
				2) making sure that these are derived from the current synapse	
				3) this is enough to ensure that we only have the synapses from the rigt projection that target the right population		
			 -->
			<xsl:variable name="num_curr_pop_proj_inputs" select="count($split_model_file//SMLLOWNL:Projection[@dst_population=$split_pops[1]/SMLLOWNL:Neuron/@name]/SMLLOWNL:Synapse/SMLLOWNL:WeightUpdate[substring-before(@name, '_sub')=$curr_syn_name])"/>
		 	<xsl:choose>
		 		<!-- IF THE NUMBER OF INPUTS IS GREATER THAN THE PREVIOUS HIGHEST -->
		 		<xsl:when test="$num_curr_pop_proj_inputs>$number">
		 			<!-- CALL TEMPLATE WITH NEW POP AND NEW NUMBER OF INPUTS -->
		 			<xsl:apply-templates select="$curr_syn_name/.." mode="threadFuncs">
						<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
						<xsl:with-param name="split_model_file" select="$split_model_file"/>
						<xsl:with-param name="number" select="$num_curr_pop_proj_inputs"/>
						<xsl:with-param name="curr_split_pop" select="$split_pops[1]"/>
						<xsl:with-param name="split_pops" select="$split_pops[position()&gt;1]"/>
						<xsl:with-param name="n_dst" select="$n_dst"/>
						<xsl:with-param name="n_src" select="$n_src"/>
					</xsl:apply-templates>
		 		</xsl:when>
		 		<!-- OTHERWISE -->
		 		<xsl:otherwise>
		 			<!-- PASS ON CURRENT POP AND NUMBER -->
		 			<xsl:apply-templates select="$curr_syn_name/.." mode="threadFuncs">
						<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
						<xsl:with-param name="split_model_file" select="$split_model_file"/>
						<xsl:with-param name="number" select="$number"/>
						<xsl:with-param name="curr_split_pop" select="$curr_split_pop"/>
						<xsl:with-param name="split_pops" select="$split_pops[position()&gt;1]"/>
						<xsl:with-param name="n_dst" select="$n_dst"/>
						<xsl:with-param name="n_src" select="$n_src"/>
					</xsl:apply-templates>
		 		</xsl:otherwise>
		 	</xsl:choose>	 		
		</xsl:when>
		<xsl:otherwise>
			<!-- LOOPING COMPLETE - WRITE OUT THE DEFINITIONS FOR THE SYNAPSES -->
			<!-- NOTE: INTO SPLIT FILE -->
			<!--xsl:for-each select="$split_model_file//SMLLOWNL:Projection[@dst_population=$curr_split_pop/SMLLOWNL:Neuron/@name]/SMLLOWNL:Synapse/SMLLOWNL:WeightUpdate[substring-before(@name, '_sub')=$curr_syn_name]"-->
				<xsl:text>
</xsl:text>
				<!-- HEADING -->// <xsl:value-of select="@name"/> ##########################
<!---->							
				<!-- AL COMPONENT FOR THIS SYNAPSE -->
				<xsl:variable name="linked_syn_file" select="document(@url)"/>
				<!-- ok, quick comment on numbering - I chose to give every Synapse a unique number - this means that numbers are less meaningful, but then they were never particularly meaningful anyway! -->
				
				<xsl:for-each select="$linked_syn_file//SMLCL:ComponentClass">
				
					<!-- ADJUSTED VARIABLES DEPENDING UPON THE CONNECTIVITY  -->
					<xsl:choose>
						<!-- IF ONE TO ONE CONNECTION ###############################-->
						<xsl:when test="count($curr_synapse/../SMLNL:OneToOneConnection)=1">
<!---->						<!-- IF DELAYS ARE FIXED -->
							<xsl:if test="count($curr_synapse/../SMLNL:OneToOneConnection//SMLNL:FixedValue)=1">
<!---->							<!-- THREAD FUNCTIONS -->
								<!---->// function threads
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>(int nrn_index, int split_index) {

<!---->							<!---->	// declare aliases for ports
<!---->							<xsl:for-each select="document($curr_synapse/@url)////SMLCL:EventSendPort | document($curr_synapse/@url)////SMLCL:AnalogSendPort">
									<xsl:variable name="portTestName" select="@name"/>
									<xsl:for-each select="//SMLCL:Alias[@name=$portTestName]">
										<xsl:apply-templates select="." mode="declareAliasSYN">
											<xsl:with-param name="suffix" select="concat($suffix,'[split_index][nrn_index]')"/>
										</xsl:apply-templates>
									</xsl:for-each>
								</xsl:for-each>
								
<!---->							<!---->	// declare fixed value parameters as local vars (work-around to stop having to write very complicated template for substituting into MathInLines)
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:if test="count($curr_synapse/SMLNL:Property[@name=$par_name]/SMLNL:FixedValue)=1">
<!---->									<!---->	float <xsl:value-of select="@name"/> = <xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>;
<!---->								</xsl:if>
								</xsl:for-each>							
								
<!---->							<!---->	// run relevant OnEvent
<!---->							<xsl:variable name="currPortName" select="@name"/>
								<xsl:for-each select="//SMLCL:OnEvent[@src_port=$curr_synapse/@input_dst_port]">
									<xsl:apply-templates select="." mode="doOnEventSYN">
										<xsl:with-param name="suffix" select="concat($suffix,'[split_index][nrn_index]')"/>
									</xsl:apply-templates>
								</xsl:for-each>
								
								<!-- EVALUATE ANY ONCONDITIONS THAT EXIST -->
<!---->								<xsl:for-each select="document($curr_synapse/@url)">
									<xsl:if test="count(//SMLCL:OnCondition)>0">
<!---->									<!---->	// run OnConditions	
<!---->									<!---->	while (transition == 1) {
<!---->									<!---->	transition = 0;
<!---->									<xsl:apply-templates select="//SMLCL:OnCondition" mode="doTrans">
											<xsl:with-param name="suffix" select="$suffix"/>
   		  								</xsl:apply-templates>
<!---->									<!---->	}
<!---->	  							</xsl:if>
   		  						</xsl:for-each>			
	
<!---->							<!----><xsl:text>
</xsl:text><!---->				<!---->// do aliases for ports
	<!---->						<xsl:for-each select="document($curr_synapse/@url)////SMLCL:EventSendPort | document($curr_synapse/@url)////SMLCL:AnalogSendPort | document($curr_synapse/@url)////SMLCL:ImpulseSendPort">
									<xsl:variable name="portTestName" select="@name"/>
									<xsl:for-each select="//SMLCL:Alias[@name=$portTestName]">
										<xsl:apply-templates select="." mode="doAliasSYN">
											<xsl:with-param name="suffix" select="concat($suffix,'[split_index][nrn_index]')"/>
											<xsl:with-param name="props" select="$curr_synapse/SMLNL:Property"/>
										</xsl:apply-templates>
									</xsl:for-each>
								</xsl:for-each>
								
<!---->							<xsl:variable name="suffix_out">
									<xsl:value-of select="translate($curr_synapse/../SMLLOWNL:PostSynapse/@name, ' -', '__')"/><!-- NEW SUFFIX -->
								</xsl:variable>
		
<!---->							<!---->	// if there is an impulse out that is the WU->PS connection
								<xsl:for-each select="//SMLCL:OnEvent[@src_port=$curr_synapse/@input_dst_port]/SMLCL:ImpulseOut[@port=$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port]">
									<xsl:variable name="srcPort" select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/>
<!---->								<!---->	// call the handler function for the OnImpulse
<!---->								<!---->	HANDLE_IMPULSE_<xsl:value-of select="$suffix_out"/>_<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_dst_port"/>(nrn_index, <!---->
										<xsl:if test="count(document($curr_synapse/@url)//SMLCL:Parameter[@name=$srcPort])=1 and count($curr_synapse/SMLNL:Property[@name=$srcPort]/SMLNL:FixedValue)=1">
											<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/>
										</xsl:if>
										<xsl:if test="count(document($curr_synapse/@url)//SMLCL:Parameter[@name=$srcPort])=0 or count($curr_synapse/SMLNL:Property[@name=$srcPort]/SMLNL:FixedValue)=0">
											<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/><xsl:value-of select="$suffix"/>_temp[nrn_index];
										</xsl:if>
										<!---->);
<!---->							</xsl:for-each>			
		
								
								<!---->}

<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>_schedule(int nrn_index, int split_index) {
<!---->							<!---->	// no fetching, all we need to do is pass the delays to the threads
<!---->							<!---->	int thread;
<!---->							<!---->	delay(mainDelay<xsl:value-of select="$suffix"/>*inv_dt*0.1);
<!---->							<!---->	thread = createthread(<xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>, 100, nrn_index, split_index);
<!---->							<!---->}
<!---->						</xsl:if>
							<!-- IF DELAYS ARE PROBABALISTIC -->
							<xsl:if test="count($curr_synapse/../SMLNL:OneToOneConnection//SMLNL:UniformDistribution | $curr_synapse/../SMLNL:OneToOneConnection//SMLNL:NormalDistribution | $curr_synapse/../SMLNL:OneToOneConnection//SMLNL:PoissonDistribution)=1">
<!---->							<!-- THREAD FUNCTIONS -->
								<!---->// function threads
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>(int nrn_index, int split_index, float delay) {

<!---->							<!---->	// declare aliases for ports
<!---->							<xsl:for-each select="document($curr_synapse/@url)////SMLCL:EventSendPort | document($curr_synapse/@url)////SMLCL:AnalogSendPort">
									<xsl:variable name="portTestName" select="@name"/>
									<xsl:for-each select="//SMLCL:Alias[@name=$portTestName]">
										<xsl:apply-templates select="." mode="declareAliasSYN">
											<xsl:with-param name="suffix" select="concat($suffix,'[split_index][nrn_index]')"/>
										</xsl:apply-templates>
									</xsl:for-each>
								</xsl:for-each>
								
<!---->							<!---->	// delay
<!---->							<!---->	delay(delay*inv_dt*0.1);
<!---->							<!---->	// run relevant OnEvent
<!---->							<xsl:variable name="currPortName" select="@name"/>
								<xsl:for-each select="//SMLCL:OnEvent[@src_port=$curr_synapse/@input_dst_port]">
									<xsl:apply-templates select="." mode="doOnEventSYN">
										<xsl:with-param name="suffix" select="concat($suffix,'[split_index][nrn_index]')"/>
									</xsl:apply-templates>
								</xsl:for-each>
									
								<!-- EVALUATE ANY ONCONDITIONS THAT EXIST -->
<!---->							<xsl:for-each select="document($curr_synapse/@url)">
									<xsl:if test="count(//SMLCL:OnCondition)>0">
<!---->									<!---->	// run OnConditions	
<!---->									<!---->	while (transition == 1) {
<!---->									<!---->	transition = 0;
<!---->									<xsl:apply-templates select="//SMLCL:OnCondition" mode="doTrans">
											<xsl:with-param name="suffix" select="$suffix"/>
   		  								</xsl:apply-templates>
<!---->									<!---->	}
<!---->								</xsl:if>
   		  						</xsl:for-each>			
	
<!---->							<!----><xsl:text>
</xsl:text><!---->				<!---->// do aliases for ports
	<!---->						<xsl:for-each select="document($curr_synapse/@url)////SMLCL:EventSendPort | document($curr_synapse/@url)////SMLCL:AnalogSendPort">
									<xsl:variable name="portTestName" select="@name"/>
									<xsl:for-each select="//SMLCL:Alias[@name=$portTestName]">
										<xsl:apply-templates select="." mode="doAliasSYN">
											<xsl:with-param name="suffix" select="concat($suffix,'[split_index][nrn_index]')"/>
										</xsl:apply-templates>
									</xsl:for-each>
								</xsl:for-each>
								
<!---->							<xsl:variable name="suffix_out">
									<xsl:value-of select="translate($curr_synapse/../SMLLOWNL:PostSynapse/@name, ' -', '__')"/><!-- NEW SUFFIX -->
								</xsl:variable>
		
<!---->							<!---->	// if there is an impulse out that is the WU->PS connection
								<xsl:for-each select="//SMLCL:OnEvent[@src_port=$curr_synapse/@input_dst_port]/SMLCL:ImpulseOut[@port=$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port]">
									<xsl:variable name="srcPort" select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/>
<!---->								<!---->	// call the handler function for the OnImpulse
<!---->								<!---->	HANDLE_IMPULSE_<xsl:value-of select="$suffix_out"/>_<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_dst_port"/>(dst_nrn_index, <!---->
										<xsl:if test="count(document($curr_synapse/@url)//SMLCL:Parameter[@name=$srcPort])=1 and count($curr_synapse/SMLNL:Property[@name=$srcPort]/SMLNL:FixedValue)=1">
											<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/><xsl:value-of select="$suffix"/>
										</xsl:if>
										<xsl:if test="count(document($curr_synapse/@url)//SMLCL:Parameter[@name=$srcPort])=0 or count($curr_synapse/SMLNL:Property[@name=$srcPort]/SMLNL:FixedValue)=0">
											<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/><xsl:value-of select="$suffix"/>_temp[dst_nrn_index];
										</xsl:if>
										<!---->);
<!---->							</xsl:for-each>		
<!---->							<!---->}

<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>_schedule(int nrn_index, int split_index) {
<!---->							<!---->	// no fetching, all we need to do is pass the delays to the threads
<!---->							<!---->	int thread;
<!---->							<!---->	thread = createthread(<xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>, 100, nrn_index, split_index, mainDelay<xsl:value-of select="$suffix"/>[nrn_index]);
<!---->							<!---->}
<!---->						</xsl:if>	
<!---->					</xsl:when>
						<!-- IF ALL TO ALL CONNECTION ###############################-->
						<xsl:when test="count($curr_synapse/../SMLNL:AllToAllConnection)=1">
<!---->						<!-- IF DELAYS ARE FIXED -->
							<xsl:if test="count($curr_synapse/../SMLNL:AllToAllConnection//SMLNL:FixedValue)=1">
<!---->							<!-- THREAD FUNCTION PROTOTYPES -->
								<!---->// function thread prototypes
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>(int src_nrn_index, int split_index){
<!---->							<!---->	int i;		
<!---->							<!---->	int dst_nrn_index;	

<!---->							<!---->	// declare aliases for ports
<!---->							<xsl:for-each select="document($curr_synapse/@url)////SMLCL:EventSendPort | document($curr_synapse/@url)////SMLCL:AnalogSendPort">
									<xsl:variable name="portTestName" select="@name"/>
									<xsl:for-each select="//SMLCL:Alias[@name=$portTestName]">
										<xsl:apply-templates select="." mode="declareAliasSYN">
											<xsl:with-param name="suffix" select="concat($suffix,'[split_index][nrn_index]')"/>
										</xsl:apply-templates>
									</xsl:for-each>
								</xsl:for-each>
	
<!---->							<!---->	// create temporary storage for pars
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:if test="count($curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
<!---->									<!---->	float <xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/>_temp[<xsl:value-of select="$n_dst"/>];
<!---->									<!---->	float <xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/>;
<!---->								</xsl:if>
								</xsl:for-each>
								
<!---->							<!---->	// create temporary storage for SVs
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!---->	float <xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>_temp[<xsl:value-of select="$n_dst"/>];
<!---->								<!---->	float <xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>;
<!---->							</xsl:for-each>
																
<!---->							<!---->	// load pars from SDRAM
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:if test="count($curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
<!---->							<!---->	readsdram(&amp;<xsl:value-of select="$par_name"/>SD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;<xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/>_temp, <xsl:value-of select="$n_dst"/>);
<!---->								</xsl:if>
								</xsl:for-each>
								
<!---->							<!---->	// load State Variables from SDRAM
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!---->	readsdram(&amp;<xsl:value-of select="$sv_name"/>SD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;<xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>_temp, <xsl:value-of select="$n_dst"/>);
<!---->							</xsl:for-each>
								
<!---->							<!---->	for (dst_nrn_index = 0; dst_nrn_index &lt; num_neurons_actual; dst_nrn_index += 1) {
<!---->							<!---->	// assign non-fixed pars and SVs
<!---->							<!-- PARAMETERS -->	
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:if test="count($curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
<!---->								<!----><xsl:text>	</xsl:text><xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/> = <xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/>_temp[dst_nrn_index];
<!---->								</xsl:if>
								</xsl:for-each>
								<!-- STATE VARIABLES -->
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!----><xsl:text>	</xsl:text><xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/> = <xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>_temp[dst_nrn_index];
<!---->							</xsl:for-each>

<!---->							<!---->	// run relevant OnEvent
<!---->							<xsl:variable name="currPortName" select="@name"/>
								<xsl:for-each select="//SMLCL:OnEvent[@src_port=$curr_synapse/@input_dst_port]">
									<xsl:apply-templates select="." mode="doOnEventSYN">
										<xsl:with-param name="suffix" select="$suffix"/>
									</xsl:apply-templates>
								</xsl:for-each>
									
								<!-- EVALUATE ANY ONCONDITIONS THAT EXIST -->
<!---->							<xsl:for-each select="document($curr_synapse/@url)">
									<xsl:if test="count(//SMLCL:OnCondition)>0">
<!---->									<!---->		// run OnConditions	
<!---->									<!---->		while (transition == 1) {
<!---->									<!---->		transition = 0;
<!---->									<xsl:apply-templates select="//SMLCL:OnCondition" mode="doTransSYN">
											<xsl:with-param name="suffix" select="$suffix"/>
   		  								</xsl:apply-templates>
<!---->									<!---->		}
<!---->								</xsl:if>
   		  						</xsl:for-each>		
	
<!---->							<!----><xsl:text>
</xsl:text><!---->				<!---->// do aliases for ports
	<!---->						<xsl:for-each select="document($curr_synapse/@url)////SMLCL:EventSendPort | document($curr_synapse/@url)////SMLCL:AnalogSendPort">
									<xsl:variable name="portTestName" select="@name"/>
									<xsl:for-each select="//SMLCL:Alias[@name=$portTestName]">
										<xsl:apply-templates select="." mode="doAliasSYN">
											<xsl:with-param name="suffix" select="$suffix"/>
										</xsl:apply-templates>
									</xsl:for-each>
								</xsl:for-each>
								
<!---->							<!---->	// write back SVs
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!----><xsl:text>	</xsl:text><xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>_temp[dst_nrn_index] = <xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>;
<!---->							</xsl:for-each>
								
<!---->							<xsl:variable name="suffix_out">
									<xsl:value-of select="translate($curr_synapse/../SMLLOWNL:PostSynapse/@name, ' -', '__')"/><!-- NEW SUFFIX -->
								</xsl:variable>
		
<!---->							<!---->	// if there is an impulse out that is the WU->PS connection
								<xsl:for-each select="//SMLCL:OnEvent[@src_port=$curr_synapse/@input_dst_port]/SMLCL:ImpulseOut[@port=$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port]">
									<xsl:variable name="srcPort" select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/>
<!---->								<!---->	// call the handler function for the OnImpulse
<!---->								<!---->	HANDLE_IMPULSE_<xsl:value-of select="$suffix_out"/>_<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_dst_port"/>(dst_nrn_index, <!---->
										<xsl:if test="count(document($curr_synapse/@url)//SMLCL:Parameter[@name=$srcPort])=1 and count($curr_synapse/SMLNL:Property[@name=$srcPort]/SMLNL:FixedValue)=1">
											<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/><xsl:value-of select="$suffix"/>
										</xsl:if>
										<xsl:if test="count(document($curr_synapse/@url)//SMLCL:Parameter[@name=$srcPort])=0 or count($curr_synapse/SMLNL:Property[@name=$srcPort]/SMLNL:FixedValue)=0">
											<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/><xsl:value-of select="$suffix"/>_temp[dst_nrn_index];
										</xsl:if>
										<!---->);
<!---->							</xsl:for-each>	
<!---->							<!---->	}									
<!---->							<!---->}
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>_schedule(int src_nrn_index, int split_index) {
<!---->							<!---->	int thread;
<!---->							<!---->	delay(mainDelay<xsl:value-of select="$suffix"/>*inv_dt*0.1);
<!---->							<!---->	thread = createthread(<xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>, 1000, src_nrn_index, split_index);
<!---->							<!---->}						
<!---->						</xsl:if>
							<!-- IF DELAYS ARE PROBABALISTIC -->
							<xsl:if test="count($curr_synapse/../SMLNL:AllToAllConnection//SMLNL:UniformDistribution | $curr_synapse/../SMLNL:AllToAllConnection//SMLNL:NormalDistribution | $curr_synapse/../SMLNL:AllToAllConnection//SMLNL:PoissonDistribution)=1">
<!---->							<!-- THREAD FUNCTION PROTOTYPES -->
								<!---->// function thread prototypes
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>(int src_nrn_index, int split_index, int dst_nrn_index, <!---->
								<xsl:for-each select="$linked_syn_file//SMLCL:Parameter">
									<!---->float <!---->
									<xsl:value-of select="@name"/>
									<!---->, <!---->
								</xsl:for-each>
								<xsl:for-each select="$linked_syn_file//SMLCL:StateVariable">
									<!---->float <!---->
									<xsl:value-of select="@name"/>
									<!---->, <!---->
								</xsl:for-each>
								<!---->float delay) {
								
<!---->							<!---->	// declare aliases for ports
<!---->							<xsl:for-each select="document($curr_synapse/@url)////SMLCL:EventSendPort | document($curr_synapse/@url)////SMLCL:AnalogSendPort">
									<xsl:variable name="portTestName" select="@name"/>
									<xsl:for-each select="//SMLCL:Alias[@name=$portTestName]">
										<xsl:apply-templates select="." mode="declareAliasSYN">
											<xsl:with-param name="suffix" select="concat($suffix,'[split_index][nrn_index]')"/>
										</xsl:apply-templates>
									</xsl:for-each>
								</xsl:for-each>
								
<!---->							<!---->	delay(delay*inv_dt*0.1);
								
<!---->							<!---->	// run relevant OnEvent
<!---->							<xsl:variable name="currPortName" select="@name"/>
								<xsl:for-each select="//SMLCL:OnEvent[@src_port=$curr_synapse/@input_dst_port]">
									<xsl:apply-templates select="." mode="doOnEventSYN">
										<xsl:with-param name="suffix" select="''"/>
									</xsl:apply-templates>
								</xsl:for-each>
									
								<!-- EVALUATE ANY ONCONDITIONS THAT EXIST -->
<!---->							<xsl:for-each select="document($curr_synapse/@url)">
									<xsl:if test="count(//SMLCL:OnCondition)>0">
<!---->									<!---->		// run OnConditions	
<!---->									<!---->		while (transition == 1) {
<!---->									<!---->		transition = 0;
<!---->									<xsl:apply-templates select="//SMLCL:OnCondition" mode="doTrans">
											<xsl:with-param name="suffix" select="''"/>
   		  								</xsl:apply-templates>
<!---->									<!---->		}
<!---->								</xsl:if>
   		  						</xsl:for-each>		
	
<!---->							<!----><xsl:text>
</xsl:text><!---->				<!---->// do aliases for ports
	<!---->						<xsl:for-each select="document($curr_synapse/@url)////SMLCL:EventSendPort | document($curr_synapse/@url)////SMLCL:AnalogSendPort">
									<xsl:variable name="portTestName" select="@name"/>
									<xsl:for-each select="//SMLCL:Alias[@name=$portTestName]">
										<xsl:apply-templates select="." mode="doAliasSYN">
											<xsl:with-param name="suffix" select="''"/>
										</xsl:apply-templates>
									</xsl:for-each>
								</xsl:for-each>
								
<!---->							<!---->	// return State Variables to SDRAM
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!---->	writesdram(&amp;<xsl:value-of select="$sv_name"/>SD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][dst_nrn_index], &amp;<xsl:value-of select="$sv_name"/>, 1);
<!---->							</xsl:for-each>
								
<!---->							<xsl:variable name="suffix_out">
									<xsl:value-of select="translate($curr_synapse/../SMLLOWNL:PostSynapse/@name, ' -', '__')"/><!-- NEW SUFFIX -->
								</xsl:variable>
		
<!---->							<!---->	// if there is an impulse out that is the WU->PS connection
								<xsl:for-each select="//SMLCL:OnEvent[@src_port=$curr_synapse/@input_dst_port]/SMLCL:ImpulseOut[@port=$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port]">
<!---->								<!---->	// call the handler function for the OnImpulse
<!---->								<!---->	HANDLE_IMPULSE_<xsl:value-of select="$suffix_out"/>_<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/input_dst_port"/>(dst_nrn_number, <xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/>);
<!---->							</xsl:for-each>		
<!---->							<!---->}
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>_schedule(int src_nrn_index, int split_index)  {
<!---->							<!---->	int dst_nrn_index;
<!---->							<!---->	int thread;
<!---->							<!---->	float mainDelay<xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>];
<!---->							<!---->	// create temporary storage for pars
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:if test="count($curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
<!---->									<!---->	float <xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>];
<!---->								</xsl:if>
								</xsl:for-each>
								
<!---->							<!---->	// create temporary storage for SVs
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!---->	float <xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>];
<!---->							</xsl:for-each>

<!---->							<!---->	readsdram(&amp;mainDelaySD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;mainDelay<xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_dst"/>);
<!---->																
<!---->							<!---->	// load pars from SDRAM
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:if test="count($curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
<!---->							<!---->	readsdram(&amp;<xsl:value-of select="$par_name"/>SD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;<xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_dst"/>);
<!---->								</xsl:if>
								</xsl:for-each>
								
<!---->							<!---->	// load State Variables from SDRAM
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!---->	readsdram(&amp;<xsl:value-of select="$sv_name"/>SD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;<xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_dst"/>);
<!---->							</xsl:for-each>
<!---->							<!---->	// launch threads
<!---->							<!---->	for (dst_nrn_index = 0; dst_nrn_index &lt; num_neurons_actual; dst_nrn_index += 1) {	
<!---->							<!---->		thread = createthread(<xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>, 1000, src_nrn_index, split_index, dst_nrn_index, <!---->
								<xsl:for-each select="$linked_syn_file//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>
									<xsl:if test="count($curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
										<!---->[dst_nrn_index]<!---->
									</xsl:if>
									<!---->, <!---->
								</xsl:for-each>
								<xsl:for-each select="$linked_syn_file//SMLCL:StateVariable">
									<xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>
									<!---->[dst_nrn_index], <!---->
								</xsl:for-each>
								<!----> mainDelay<xsl:value-of select="$suffix"/>[dst_nrn_index]);
<!---->							<!---->	}
<!---->							<!---->}
<!---->						</xsl:if>
						
						</xsl:when>
						<!-- IF FIXED PROBABILITY CONNECTION ###############################-->
						<xsl:when test="count($curr_synapse/../SMLNL:FixedProbabilityConnection)=1">	
							<xsl:variable name="n_prob" select="ceiling(6.0*number($n_dst)*number($curr_synapse/../SMLNL:FixedProbabilityConnection/@probability))"/>
<!---->						<!-- IF DELAYS ARE FIXED -->
							<xsl:if test="count($curr_synapse/../SMLNL:FixedProbabilityConnection//SMLNL:FixedValue)=1">
<!---->							<!-- THREAD FUNCTION PROTOTYPES -->
								<!---->// function thread prototypes
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>(int src_nrn_index, int split_index){
<!---->							<!---->	int i;		
<!---->							<!---->	int dst_nrn_index;	

<!---->							<!---->	// declare aliases for ports
<!---->							<xsl:for-each select="document($curr_synapse/@url)////SMLCL:EventSendPort | document($curr_synapse/@url)////SMLCL:AnalogSendPort">
									<xsl:variable name="portTestName" select="@name"/>
									<xsl:for-each select="//SMLCL:Alias[@name=$portTestName]">
										<xsl:apply-templates select="." mode="declareAliasSYN">
											<xsl:with-param name="suffix" select="concat($suffix,'[split_index][nrn_index]')"/>
										</xsl:apply-templates>
									</xsl:for-each>
								</xsl:for-each>
	
<!---->							<!---->	// create temporary storage for connectivity
<!---->							<!---->	int connection<xsl:value-of select="$suffix"/>_temp[<xsl:value-of select="$n_prob"/>];
<!---->							<!---->	// create temporary storage for pars
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:if test="count($curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
<!---->									<!---->	float <xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/>_temp[<xsl:value-of select="$n_prob"/>];
<!---->									<!---->	float <xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/>;
<!---->								</xsl:if>
								</xsl:for-each>
								
<!---->							<!---->	// create temporary storage for SVs
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!---->	float <xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>_temp[<xsl:value-of select="$n_prob"/>];
<!---->								<!---->	float <xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>;
<!---->							</xsl:for-each>
									
<!---->							<!---->	// load connectivity from SDRAM									
<!---->							<!---->	readsdram(&amp;mainConnectionSD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;connection<xsl:value-of select="$suffix"/>_temp, <xsl:value-of select="$n_prob"/>);
<!---->				
<!---->							<!---->	// load pars from SDRAM
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:if test="count($curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
<!---->							<!---->	readsdram(&amp;<xsl:value-of select="$par_name"/>SD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;<xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/>_temp, <xsl:value-of select="$n_prob"/>);
<!---->								</xsl:if>
								</xsl:for-each>
								
<!---->							<!---->	// load State Variables from SDRAM
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!---->	readsdram(&amp;<xsl:value-of select="$sv_name"/>SD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;<xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>_temp, <xsl:value-of select="$n_prob"/>);
<!---->							</xsl:for-each>

<!---->							<!---->	i = 0;								
<!---->							<!---->	while (connection<xsl:value-of select="$suffix"/>_temp[i] != -1) {
<!---->							<!---->		dst_nrn_index = connection<xsl:value-of select="$suffix"/>_temp[i];
<!---->							<!---->		// assign non-fixed pars and SVs
<!---->							<!-- PARAMETERS -->	
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:if test="count($curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
<!---->								<!----><xsl:text>		</xsl:text><xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/> = <xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/>_temp[i];
<!---->								</xsl:if>
								</xsl:for-each>
								<!-- STATE VARIABLES -->
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!----><xsl:text>		</xsl:text><xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/> = <xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>_temp[i];
<!---->							</xsl:for-each>

<!---->							<!---->		// run relevant OnEvent
<!---->							<xsl:variable name="currPortName" select="@name"/>
								<xsl:for-each select="//SMLCL:OnEvent[@src_port=$curr_synapse/@input_dst_port]">
									<xsl:apply-templates select="." mode="doOnEventSYN">
										<xsl:with-param name="suffix" select="$suffix"/>
									</xsl:apply-templates>
								</xsl:for-each>
									
								<!-- EVALUATE ANY ONCONDITIONS THAT EXIST -->
<!---->							<xsl:for-each select="document($curr_synapse/@url)">
									<xsl:if test="count(//SMLCL:OnCondition)>0">
<!---->									<!---->		// run OnConditions	
<!---->									<!---->		while (transition == 1) {
<!---->									<!---->		transition = 0;
<!---->									<xsl:apply-templates select="//SMLCL:OnCondition" mode="doTrans">
											<xsl:with-param name="suffix" select="$suffix"/>
   		  								</xsl:apply-templates>
<!---->									<!---->		}
<!---->								</xsl:if>
   		  						</xsl:for-each>		
	
<!---->							<!----><xsl:text>
</xsl:text><!---->				<!---->// do aliases for ports
<!---->							<xsl:for-each select="document($curr_synapse/@url)////SMLCL:EventSendPort | document($curr_synapse/@url)////SMLCL:AnalogSendPort">
									<xsl:variable name="portTestName" select="@name"/>
									<xsl:for-each select="//SMLCL:Alias[@name=$portTestName]">
										<xsl:apply-templates select="." mode="doAliasSYN">
											<xsl:with-param name="suffix" select="$suffix"/>
										</xsl:apply-templates>
									</xsl:for-each>
								</xsl:for-each>
								
<!---->							<!---->		// write back SVs
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!----><xsl:text>		</xsl:text><xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>_temp[i] = <xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>;
<!---->							</xsl:for-each>
								
<!---->							<xsl:variable name="suffix_out">
									<xsl:value-of select="translate($curr_synapse/../SMLLOWNL:PostSynapse/@name, ' -', '__')"/><!-- NEW SUFFIX -->
								</xsl:variable>
		
<!---->							<!---->	// if there is an impulse out that is the WU->PS connection
								<xsl:for-each select="//SMLCL:OnEvent[@src_port=$curr_synapse/@input_dst_port]/SMLCL:ImpulseOut[@port=$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port]">
									<xsl:variable name="srcPort" select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/>
<!---->								<!---->	// call the handler function for the OnImpulse
<!---->								<!---->	HANDLE_IMPULSE_<xsl:value-of select="$suffix_out"/>_<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_dst_port"/>(dst_nrn_index, <!---->
										<xsl:if test="count(document($curr_synapse/@url)//SMLCL:Parameter[@name=$srcPort])=1 and count($curr_synapse/SMLNL:Property[@name=$srcPort]/SMLNL:FixedValue)=1">
											<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/><xsl:value-of select="$suffix"/>
										</xsl:if>
										<xsl:if test="count(document($curr_synapse/@url)//SMLCL:Parameter[@name=$srcPort])=0 or count($curr_synapse/SMLNL:Property[@name=$srcPort]/SMLNL:FixedValue)=0">
											<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/><xsl:value-of select="$suffix"/>_temp[dst_nrn_index];
										</xsl:if>
										<!---->);
<!---->							</xsl:for-each>										
<!---->							<!---->		i += 1;
<!---->							<!---->	}
<!---->							<!---->	// return State Variables to SDRAM
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!---->	writesdram(&amp;<xsl:value-of select="$sv_name"/>SD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;<xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>_temp, <xsl:value-of select="$n_prob"/>);
<!---->							</xsl:for-each>
<!---->							<!---->}
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>_schedule(int src_nrn_index, int split_index) {
<!---->							<!---->	int thread;
<!---->							<!---->	delay(mainDelay<xsl:value-of select="$suffix"/>*inv_dt*0.1);
<!---->							<!---->	thread = createthread(<xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>, 1000, src_nrn_index, split_index);
<!---->							<!---->}
<!---->						</xsl:if>
							<!-- IF DELAYS ARE PROBABALISTIC -->
							<xsl:if test="count($curr_synapse/../SMLNL:FixedProbabilityConnection//SMLNL:UniformDistribution | $curr_synapse/../SMLNL:FixedProbabilityConnection//SMLNL:NormalDistribution | $curr_synapse/../SMLNL:FixedProbabilityConnection//SMLNL:PoissonDistribution)=1">
<!---->							<!-- THREAD FUNCTION PROTOTYPES -->
								<!---->// function thread prototypes
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>(int src_nrn_index, int split_index, int dst_nrn_index, <!---->
								<xsl:for-each select="$linked_syn_file//SMLCL:Parameter">
									<!---->float <!---->
									<xsl:value-of select="@name"/>
									<!---->, <!---->
								</xsl:for-each>
								<xsl:for-each select="$linked_syn_file//SMLCL:StateVariable">
									<!---->float <!---->
									<xsl:value-of select="@name"/>
									<!---->, <!---->
								</xsl:for-each>
								<!---->float delay) {
								
<!---->							<!---->	// declare aliases for ports
<!---->							<xsl:for-each select="document($curr_synapse/@url)////SMLCL:EventSendPort | document($curr_synapse/@url)////SMLCL:AnalogSendPort">
									<xsl:variable name="portTestName" select="@name"/>
									<xsl:for-each select="//SMLCL:Alias[@name=$portTestName]">
										<xsl:apply-templates select="." mode="declareAliasSYN">
											<xsl:with-param name="suffix" select="concat($suffix,'[split_index][nrn_index]')"/>
										</xsl:apply-templates>
									</xsl:for-each>
								</xsl:for-each>
								
<!---->							<!---->	delay(delay*inv_dt*0.1);
								
<!---->							<!---->	// run relevant OnEvent
<!---->							<xsl:variable name="currPortName" select="@name"/>
								<xsl:for-each select="//SMLCL:OnEvent[@src_port=$curr_synapse/@input_dst_port]">
									<xsl:apply-templates select="." mode="doOnEventSYN">
										<xsl:with-param name="suffix" select="''"/>
									</xsl:apply-templates>
								</xsl:for-each>
									
								<!-- EVALUATE ANY ONCONDITIONS THAT EXIST -->
<!---->							<xsl:for-each select="document($curr_synapse/@url)">
									<xsl:if test="count(//SMLCL:OnCondition)>0">
<!---->									<!---->		// run OnConditions	
<!---->									<!---->		while (transition == 1) {
<!---->									<!---->		transition = 0;
<!---->									<xsl:apply-templates select="//SMLCL:OnCondition" mode="doTrans">
											<xsl:with-param name="suffix" select="''"/>
   		  								</xsl:apply-templates>
<!---->									<!---->		}
<!---->								</xsl:if>
   		  						</xsl:for-each>		
	
<!---->							<!----><xsl:text>
</xsl:text><!---->				<!---->// do aliases for ports
	<!---->						<xsl:for-each select="document($curr_synapse/@url)////SMLCL:EventSendPort | document($curr_synapse/@url)////SMLCL:AnalogSendPort">
									<xsl:variable name="portTestName" select="@name"/>
									<xsl:for-each select="//SMLCL:Alias[@name=$portTestName]">
										<xsl:apply-templates select="." mode="doAliasSYN">
											<xsl:with-param name="suffix" select="''"/>
										</xsl:apply-templates>
									</xsl:for-each>
								</xsl:for-each>
								
<!---->							<!---->	// return State Variables to SDRAM
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!---->	writesdram(&amp;<xsl:value-of select="$sv_name"/>SD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][dst_nrn_index][0], &amp;<xsl:value-of select="$sv_name"/>, 1);
<!---->							</xsl:for-each>
								
<!---->							<xsl:variable name="suffix_out">
									<xsl:value-of select="translate($curr_synapse/../SMLLOWNL:PostSynapse/@name, ' -', '__')"/><!-- NEW SUFFIX -->
								</xsl:variable>
		
<!---->							<!---->	// if there is an impulse out that is the WU->PS connection
								<xsl:for-each select="//SMLCL:OnEvent[@src_port=$curr_synapse/@input_dst_port]/SMLCL:ImpulseOut[@port=$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port]">
									<xsl:variable name="srcPort" select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/>
<!---->								<!---->	// call the handler function for the OnImpulse
<!---->								<!---->	HANDLE_IMPULSE_<xsl:value-of select="$suffix_out"/>_<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_dst_port"/>(dst_nrn_index, <!---->
										<xsl:if test="count(document($curr_synapse/@url)//SMLCL:Parameter[@name=$srcPort])=1 and count($curr_synapse/SMLNL:Property[@name=$srcPort]/SMLNL:FixedValue)=1">
											<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/><xsl:value-of select="$suffix"/>
										</xsl:if>
										<xsl:if test="count(document($curr_synapse/@url)//SMLCL:Parameter[@name=$srcPort])=0 or count($curr_synapse/SMLNL:Property[@name=$srcPort]/SMLNL:FixedValue)=0">
											<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/><xsl:value-of select="$suffix"/>_temp[dst_nrn_index];
										</xsl:if>
										<!---->);
<!---->							</xsl:for-each>		
<!---->							<!---->}
<!---->	
								<!-- ################# -->
								<!-- SCHEDULE FUNCTION -->
								<!-- ################# -->
								
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>_schedule(int src_nrn_index, int split_index)  {
<!---->							<!---->	int i;
<!---->							<!---->	int thread;
<!---->							<!---->	int dst_nrn_index;
<!---->							<!---->	float mainDelay<xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>];
<!---->							<!---->	// create temporary storage for connectivity
<!---->							<!---->	int connection<xsl:value-of select="$suffix"/>_temp[<xsl:value-of select="$n_prob"/>];
<!---->							<!---->	// create temporary storage for pars
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:if test="count($curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
<!---->									<!---->	float <xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_prob"/>];
<!---->								</xsl:if>
								</xsl:for-each>
								
<!---->							<!---->	// create temporary storage for SVs
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!---->	float <xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_prob"/>];
<!---->							</xsl:for-each>

<!---->							<!---->	// load delay from SDRAM	
<!---->							<!---->	readsdram(&amp;mainDelaySD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;mainDelay<xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_prob"/>);
<!---->												
<!---->							<!---->	// load connectivity from SDRAM									
readsdram(&amp;mainConnectionSD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;connection<xsl:value-of select="$suffix"/>_temp, <xsl:value-of select="$n_prob"/>);
<!---->				
<!---->							<!---->	// load pars from SDRAM
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:if test="count($curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
<!---->							<!---->	readsdram(&amp;<xsl:value-of select="$par_name"/>SD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;<xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_prob"/>);
<!---->								</xsl:if>
								</xsl:for-each>
								
<!---->							<!---->	// load State Variables from SDRAM
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!---->	readsdram(&amp;<xsl:value-of select="$sv_name"/>SD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;<xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_prob"/>);
<!---->							</xsl:for-each>
<!---->							<!---->	// launch threads
<!---->							<!---->	i = 0;								
<!---->							<!---->	while (connection<xsl:value-of select="$suffix"/>_temp[i] != -1) {
<!---->							<!---->		dst_nrn_index = connection<xsl:value-of select="$suffix"/>_temp[i];
<!---->							<!---->		thread = createthread(<xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>, 1000, src_nrn_index, split_index, dst_nrn_index, <!---->
								<xsl:for-each select="$linked_syn_file//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>
									<xsl:if test="count($curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
										<!---->[i]<!---->
									</xsl:if>
									<!---->, <!---->
								</xsl:for-each>
								<xsl:for-each select="$linked_syn_file//SMLCL:StateVariable">
									<xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>
									<!---->[i], <!---->
								</xsl:for-each>
								<!----> mainDelay<xsl:value-of select="$suffix"/>[i]);
<!---->							<!---->		i += 1;
<!---->							<!---->	}
<!---->							<!---->}
<!---->						</xsl:if>
						
						</xsl:when>
						<!-- IF EXPLICIT LIST CONNECTION ###############################-->
						<xsl:when test="count($curr_synapse/../SMLNL:ConnectionList)=1">
<!---->						<!-- IF DELAYS ARE FIXED -->
							<xsl:if test="count($curr_synapse/../SMLNL:ConnectionList//SMLNL:FixedValue)=1">
<!---->							<!-- THREAD FUNCTION PROTOTYPES -->
								<!---->// function thread prototypes
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>(int src_nrn_index, int split_index){
<!---->							<!---->	int i;		
<!---->							<!---->	int dst_nrn_index;	

<!---->							<!---->	// declare aliases for ports
<!---->							<xsl:for-each select="document($curr_synapse/@url)////SMLCL:EventSendPort | document($curr_synapse/@url)////SMLCL:AnalogSendPort">
									<xsl:variable name="portTestName" select="@name"/>
									<xsl:for-each select="//SMLCL:Alias[@name=$portTestName]">
										<xsl:apply-templates select="." mode="declareAliasSYN">
											<xsl:with-param name="suffix" select="concat($suffix,'[split_index][nrn_index]')"/>
										</xsl:apply-templates>
									</xsl:for-each>
								</xsl:for-each>
	
<!---->							<!---->	// create temporary storage for connectivity
<!---->							<!---->	int connection<xsl:value-of select="$suffix"/>_temp[<xsl:value-of select="$n_prob"/>];
<!---->							<!---->	// create temporary storage for pars
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:if test="count($curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
<!---->									<!---->	float <xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/>_temp[<xsl:value-of select="$n_dst"/>];
<!---->									<!---->	float <xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/>;
<!---->								</xsl:if>
								</xsl:for-each>
								
<!---->							<!---->	// create temporary storage for SVs
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!---->	float <xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>_temp[<xsl:value-of select="$n_dst"/>];
<!---->								<!---->	float <xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>;
<!---->							</xsl:for-each>
									
<!---->							<!---->	// load connectivity from SDRAM									
<!---->							<!---->	readsdram(&amp;mainConnectionSD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;connection<xsl:value-of select="$suffix"/>_temp, <xsl:value-of select="$n_dst"/>);
<!---->				
<!---->							<!---->	// load pars from SDRAM
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:if test="count($curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
<!---->							<!---->	readsdram(&amp;<xsl:value-of select="$par_name"/>SD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;<xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/>_temp, <xsl:value-of select="$n_dst"/>);
<!---->								</xsl:if>
								</xsl:for-each>
								
<!---->							<!---->	// load State Variables from SDRAM
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!---->	readsdram(&amp;<xsl:value-of select="$sv_name"/>SD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;<xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>_temp, <xsl:value-of select="$n_dst"/>);
<!---->							</xsl:for-each>
								
<!---->							<!---->	for (dst_nrn_index = 0; dst_nrn_index &lt; num_neurons_actual; dst_nrn_index += 1) {
<!---->							<!---->	// if no connection then skip...
<!---->							<!---->	if (connection<xsl:value-of select="$suffix"/>_temp[dst_nrn_index]==0) continue;
<!---->							<!---->	// assign non-fixed pars and SVs
<!---->							<!-- PARAMETERS -->	
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:if test="count($curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
<!---->								<!----><xsl:text>	</xsl:text><xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/> = <xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/>_temp[i];
<!---->								</xsl:if>
								</xsl:for-each>
								<!-- STATE VARIABLES -->
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!----><xsl:text>	</xsl:text><xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/> = <xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>_temp[i];
<!---->							</xsl:for-each>

<!---->							<!---->	// run relevant OnEvent
<!---->							<xsl:variable name="currPortName" select="@name"/>
								<xsl:for-each select="//SMLCL:OnEvent[@src_port=$curr_synapse/@input_dst_port]">
									<xsl:apply-templates select="." mode="doOnEventSYN">
										<xsl:with-param name="suffix" select="$suffix"/>
									</xsl:apply-templates>
								</xsl:for-each>
									
								<!-- EVALUATE ANY ONCONDITIONS THAT EXIST -->
<!---->							<xsl:for-each select="document($curr_synapse/@url)">
									<xsl:if test="count(//SMLCL:OnCondition)>0">
<!---->									<!---->		// run OnConditions	
<!---->									<!---->		while (transition == 1) {
<!---->									<!---->		transition = 0;
<!---->									<xsl:apply-templates select="//SMLCL:OnCondition" mode="doTrans">
											<xsl:with-param name="suffix" select="$suffix"/>
   		  								</xsl:apply-templates>
<!---->									<!---->		}
<!---->								</xsl:if>
   		  						</xsl:for-each>		
	
<!---->							<!----><xsl:text>
</xsl:text><!---->				<!---->// do aliases for ports
	<!---->						<xsl:for-each select="document($curr_synapse/@url)////SMLCL:EventSendPort | document($curr_synapse/@url)////SMLCL:AnalogSendPort">
									<xsl:variable name="portTestName" select="@name"/>
									<xsl:for-each select="//SMLCL:Alias[@name=$portTestName]">
										<xsl:apply-templates select="." mode="doAliasSYN">
											<xsl:with-param name="suffix" select="$suffix"/>
										</xsl:apply-templates>
									</xsl:for-each>
								</xsl:for-each>
								
<!---->							<!---->	// write back SVs
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!----><xsl:text>	</xsl:text><xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>_temp[i] = <xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>;
<!---->							</xsl:for-each>
								
<!---->							<xsl:variable name="suffix_out">
									<xsl:value-of select="translate($curr_synapse/../SMLLOWNL:PostSynapse/@name, ' -', '__')"/><!-- NEW SUFFIX -->
								</xsl:variable>
		
<!---->							<!---->	// if there is an impulse out that is the WU->PS connection
								<xsl:for-each select="//SMLCL:OnEvent[@src_port=$curr_synapse/@input_dst_port]/SMLCL:ImpulseOut[@port=$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port]">
									<xsl:variable name="srcPort" select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/>
<!---->								<!---->	// call the handler function for the OnImpulse
<!---->								<!---->	HANDLE_IMPULSE_<xsl:value-of select="$suffix_out"/>_<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_dst_port"/>(dst_nrn_index, <!---->
										<xsl:if test="count(document($curr_synapse/@url)//SMLCL:Parameter[@name=$srcPort])=1 and count($curr_synapse/SMLNL:Property[@name=$srcPort]/SMLNL:FixedValue)=1">
											<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/>
										</xsl:if>
										<xsl:if test="count(document($curr_synapse/@url)//SMLCL:Parameter[@name=$srcPort])=0 or count($curr_synapse/SMLNL:Property[@name=$srcPort]/SMLNL:FixedValue)=0">
											<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/><xsl:value-of select="$suffix"/>_temp[dst_nrn_index];
										</xsl:if>
										<!---->);
<!---->							</xsl:for-each>		
<!---->							<!---->	}
<!---->							<!---->	// return State Variables to SDRAM
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!---->	writesdram(&amp;<xsl:value-of select="$sv_name"/>SD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;<xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_dst"/>);
<!---->							</xsl:for-each>
<!---->							<!---->}
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>_schedule(int src_nrn_index, int split_index) {
<!---->							<!---->	int thread;
<!---->							<!---->	delay(mainDelay<xsl:value-of select="$suffix"/>*inv_dt*0.1);
<!---->							<!---->	thread = createthread(<xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>, 1000, src_nrn_index, split_index);
<!---->							<!---->}
<!---->						</xsl:if>
							<!-- IF DELAYS ARE PROBABALISTIC -->
							<xsl:if test="count($curr_synapse/../SMLNL:ConnectionList//SMLNL:UniformDistribution | $curr_synapse/../SMLNL:ConnectionList//SMLNL:NormalDistribution | $curr_synapse/../SMLNL:ConnectionList//SMLNL:PoissonDistribution)=1">
<!---->							<!-- THREAD FUNCTION PROTOTYPES -->
								<!---->// function thread prototypes
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>(int src_nrn_index, int split_index, int dst_nrn_index, <!---->
								<xsl:for-each select="$linked_syn_file//SMLCL:Parameter">
									<!---->float <!---->
									<xsl:value-of select="@name"/>
									<!---->, <!---->
								</xsl:for-each>
								<xsl:for-each select="$linked_syn_file//SMLCL:StateVariable">
									<!---->float <!---->
									<xsl:value-of select="@name"/>
									<!---->, <!---->
								</xsl:for-each>
								<!---->float delay) {
								
<!---->							<!---->	// declare aliases for ports
<!---->							<xsl:for-each select="document($curr_synapse/@url)////SMLCL:EventSendPort | document($curr_synapse/@url)////SMLCL:AnalogSendPort">
									<xsl:variable name="portTestName" select="@name"/>
									<xsl:for-each select="//SMLCL:Alias[@name=$portTestName]">
										<xsl:apply-templates select="." mode="declareAliasSYN">
											<xsl:with-param name="suffix" select="concat($suffix,'[split_index][nrn_index]')"/>
										</xsl:apply-templates>
									</xsl:for-each>
								</xsl:for-each>
								
<!---->							<!---->	delay(delay*inv_dt*0.1);
								
<!---->							<!---->	// run relevant OnEvent
<!---->							<xsl:variable name="currPortName" select="@name"/>
								<xsl:for-each select="//SMLCL:OnEvent[@src_port=$curr_synapse/@input_dst_port]">
									<xsl:apply-templates select="." mode="doOnEventSYN">
										<xsl:with-param name="suffix" select="''"/>
									</xsl:apply-templates>
								</xsl:for-each>
									
								<!-- EVALUATE ANY ONCONDITIONS THAT EXIST -->
<!---->							<xsl:for-each select="document($curr_synapse/@url)">
									<xsl:if test="count(//SMLCL:OnCondition)>0">
<!---->									<!---->		// run OnConditions	
<!---->									<!---->		while (transition == 1) {
<!---->									<!---->		transition = 0;
<!---->									<xsl:apply-templates select="//SMLCL:OnCondition" mode="doTrans">
											<xsl:with-param name="suffix" select="''"/>
   		  								</xsl:apply-templates>
<!---->									<!---->		}
<!---->								</xsl:if>
   		  						</xsl:for-each>		
	
<!---->							<!----><xsl:text>
</xsl:text><!---->				<!---->// do aliases for ports
	<!---->						<xsl:for-each select="document($curr_synapse/@url)////SMLCL:EventSendPort | document($curr_synapse/@url)////SMLCL:AnalogSendPort">
									<xsl:variable name="portTestName" select="@name"/>
									<xsl:for-each select="//SMLCL:Alias[@name=$portTestName]">
										<xsl:apply-templates select="." mode="doAliasSYN">
											<xsl:with-param name="suffix" select="''"/>
										</xsl:apply-templates>
									</xsl:for-each>
								</xsl:for-each>
								
<!---->							<!---->	// return State Variables to SDRAM
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!---->	writesdram(&amp;<xsl:value-of select="$sv_name"/>SD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][dst_nrn_index][0], &amp;<xsl:value-of select="$sv_name"/>, 1);
<!---->							</xsl:for-each>
								
<!---->							<xsl:variable name="suffix_out">
									<xsl:value-of select="translate($curr_synapse/../SMLLOWNL:PostSynapse/@name, ' -', '__')"/><!-- NEW SUFFIX -->
								</xsl:variable>
		
<!---->							<!---->	// if there is an impulse out that is the WU->PS connection
								<xsl:for-each select="//SMLCL:OnEvent[@src_port=$curr_synapse/@input_dst_port]/SMLCL:ImpulseOut[@port=$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port]">
									<xsl:variable name="srcPort" select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/>
<!---->								<!---->	// call the handler function for the OnImpulse
<!---->								<!---->	HANDLE_IMPULSE_<xsl:value-of select="$suffix_out"/>_<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_dst_port"/>(dst_nrn_index, <!---->
										<xsl:if test="count(document($curr_synapse/@url)//SMLCL:Parameter[@name=$srcPort])=1 and count($curr_synapse/SMLNL:Property[@name=$srcPort]/SMLNL:FixedValue)=1">
											<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/>
										</xsl:if>
										<xsl:if test="count(document($curr_synapse/@url)//SMLCL:Parameter[@name=$srcPort])=0 or count($curr_synapse/SMLNL:Property[@name=$srcPort]/SMLNL:FixedValue)=0">
											<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/><xsl:value-of select="$suffix"/>_temp[dst_nrn_index];
										</xsl:if>
										<!---->);
<!---->							</xsl:for-each>		
<!---->							<!---->	}
<!---->							<!---->}
<!---->	
								<!-- ################# -->
								<!-- SCHEDULE FUNCTION -->
								<!-- ################# -->
								
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>_schedule(int src_nrn_index, int split_index);  {
<!---->							<!---->	int i;
<!---->							<!---->	int dst_nrn_index;
<!---->							<!---->	float mainDelay<xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>];
<!---->							<!---->	// create temporary storage for connectivity
<!---->							<!---->	int connection<xsl:value-of select="$suffix"/>_temp[<xsl:value-of select="$n_dst"/>];
<!---->							<!---->	// create temporary storage for pars
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:if test="count($curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
<!---->									<!---->	float <xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>];
<!---->								</xsl:if>
								</xsl:for-each>
								
<!---->							<!---->	// create temporary storage for SVs
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!---->	float <xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>];
<!---->							</xsl:for-each>

<!---->							<!---->	// load delay from SDRAM	
<!---->							<!---->	readsdram(&amp;mainDelaySD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;mainDelay<xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_dst"/>);
<!---->												
<!---->							<!---->	// load connectivity from SDRAM									
readsdram(&amp;mainConnectionSD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;connection<xsl:value-of select="$suffix"/>_temp, <xsl:value-of select="$n_dst"/>);
<!---->				
<!---->							<!---->	// load pars from SDRAM
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:if test="count($curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
<!---->							<!---->	readsdram(&amp;<xsl:value-of select="$par_name"/>SD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;<xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_dst"/>);
<!---->								</xsl:if>
								</xsl:for-each>
								
<!---->							<!---->	// load State Variables from SDRAM
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!---->	readsdram(&amp;<xsl:value-of select="$sv_name"/>SD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;<xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_dst"/>);
<!---->							</xsl:for-each>
<!---->							<!---->	// launch threads							
<!---->							<!---->	for (dst_nrn_index=0; dst_nrn_index &lt; num_neurons_actual; dst_nrn_index += 1) {
<!---->							<!---->		if (connection<xsl:value-of select="$suffix"/>_temp[dst_nrn_index]==0) continue;
<!---->							<!---->		thread = createthread(<xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>, 1000, src_nrn_index, split_index, dst_nrn_index, <!---->
								<xsl:for-each select="$linked_syn_file//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>
									<xsl:if test="count($curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
										<!---->[i]<!---->
									</xsl:if>
									<!---->, <!---->
								</xsl:for-each>
								<xsl:for-each select="$linked_syn_file//SMLCL:StateVariable">
									<xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>
									<!---->[i], <!---->
								</xsl:for-each>
								<!----> mainDelay<xsl:value-of select="$suffix"/>[i]);
<!---->							<!---->	}
<!---->							<!---->}
<!---->						</xsl:if>
						
							<!-- IF DELAYS ARE EXPLICIT -->
							<xsl:if test="count($curr_synapse/../SMLNL:ConnectionList//SMLNL:Connection/@delay)>0">
<!---->							<!-- THREAD FUNCTION PROTOTYPES -->
								<!---->// function thread prototypes
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>(int src_nrn_index, int split_index, int dst_nrn_index, <!---->
								<xsl:for-each select="$linked_syn_file//SMLCL:Parameter">
									<!---->float <!---->
									<xsl:value-of select="@name"/>
									<!---->, <!---->
								</xsl:for-each>
								<xsl:for-each select="$linked_syn_file//SMLCL:StateVariable">
									<!---->float <!---->
									<xsl:value-of select="@name"/>
									<!---->, <!---->
								</xsl:for-each>
								<!---->float delay) {
								
<!---->							<!---->	// declare aliases for ports
<!---->							<xsl:for-each select="document($curr_synapse/@url)////SMLCL:EventSendPort | document($curr_synapse/@url)////SMLCL:AnalogSendPort">
									<xsl:variable name="portTestName" select="@name"/>
									<xsl:for-each select="//SMLCL:Alias[@name=$portTestName]">
										<xsl:apply-templates select="." mode="declareAliasSYN">
											<xsl:with-param name="suffix" select="concat($suffix,'[split_index][nrn_index]')"/>
										</xsl:apply-templates>
									</xsl:for-each>
								</xsl:for-each>
								
<!---->							<!---->	delay(delay*inv_dt*0.1);
								
<!---->							<!---->	// run relevant OnEvent
<!---->							<xsl:variable name="currPortName" select="@name"/>
								<xsl:for-each select="//SMLCL:OnEvent[@src_port=$curr_synapse/@input_dst_port]">
									<xsl:apply-templates select="." mode="doOnEventSYN">
										<xsl:with-param name="suffix" select="''"/>
									</xsl:apply-templates>
								</xsl:for-each>
									
								<!-- EVALUATE ANY ONCONDITIONS THAT EXIST -->
<!---->							<xsl:for-each select="document($curr_synapse/@url)">
									<xsl:if test="count(//SMLCL:OnCondition)>0">
<!---->									<!---->		// run OnConditions	
<!---->									<!---->		while (transition == 1) {
<!---->									<!---->		transition = 0;
<!---->									<xsl:apply-templates select="//SMLCL:OnCondition" mode="doTrans">
											<xsl:with-param name="suffix" select="''"/>
   		  								</xsl:apply-templates>
<!---->									<!---->		}
<!---->								</xsl:if>
   		  						</xsl:for-each>		
	
<!---->							<!----><xsl:text>
</xsl:text><!---->				<!---->// do aliases for ports
	<!---->						<xsl:for-each select="document($curr_synapse/@url)////SMLCL:EventSendPort | document($curr_synapse/@url)////SMLCL:AnalogSendPort">
									<xsl:variable name="portTestName" select="@name"/>
									<xsl:for-each select="//SMLCL:Alias[@name=$portTestName]">
										<xsl:apply-templates select="." mode="doAliasSYN">
											<xsl:with-param name="suffix" select="''"/>
										</xsl:apply-templates>
									</xsl:for-each>
								</xsl:for-each>
								
<!---->							<!---->	// return State Variables to SDRAM
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!---->	writesdram(&amp;<xsl:value-of select="$sv_name"/>SD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][dst_nrn_index][0], &amp;<xsl:value-of select="$sv_name"/>, 1);
<!---->							</xsl:for-each>
								
<!---->							<xsl:variable name="suffix_out">
									<xsl:value-of select="translate($curr_synapse/../SMLLOWNL:PostSynapse/@name, ' -', '__')"/><!-- NEW SUFFIX -->
								</xsl:variable>
		
<!---->							<!---->	// if there is an impulse out that is the WU->PS connection
								<xsl:for-each select="//SMLCL:OnEvent[@src_port=$curr_synapse/@input_dst_port]/SMLCL:ImpulseOut[@port=$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port]">
									<xsl:variable name="srcPort" select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/>
<!---->								<!---->	// call the handler function for the OnImpulse
<!---->								<!---->	HANDLE_IMPULSE_<xsl:value-of select="$suffix_out"/>_<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_dst_port"/>(dst_nrn_index, <!---->
										<xsl:if test="count(document($curr_synapse/@url)//SMLCL:Parameter[@name=$srcPort])=1 and count($curr_synapse/SMLNL:Property[@name=$srcPort]/SMLNL:FixedValue)=1">
											<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/>
										</xsl:if>
										<xsl:if test="count(document($curr_synapse/@url)//SMLCL:Parameter[@name=$srcPort])=0 or count($curr_synapse/SMLNL:Property[@name=$srcPort]/SMLNL:FixedValue)=0">
											<xsl:value-of select="$curr_synapse/../SMLLOWNL:PostSynapse/@input_src_port"/><xsl:value-of select="$suffix"/>_temp[dst_nrn_index];
										</xsl:if>
										<!---->);
<!---->							</xsl:for-each>		
<!---->							<!---->}
<!---->	
								<!-- ################# -->
								<!-- SCHEDULE FUNCTION -->
								<!-- ################# -->
								
<!---->							<!---->void <xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>_schedule(int src_nrn_index, int split_index) {
<!---->							<!---->	int thread;
<!---->							<!---->	int dst_nrn_index;
<!---->							<!---->	float mainDelay<xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>];
<!---->							<!---->	// create temporary storage for connectivity
<!---->							<!---->	int connection<xsl:value-of select="$suffix"/>_temp[<xsl:value-of select="$n_dst"/>];
<!---->							<!---->	// create temporary storage for pars
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:if test="count($curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
<!---->									<!---->	float <xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>];
<!---->								</xsl:if>
								</xsl:for-each>
								
<!---->							<!---->	// create temporary storage for SVs
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!---->	float <xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>[<xsl:value-of select="$n_dst"/>];
<!---->							</xsl:for-each>

<!---->							<!---->	// load delay from SDRAM	
<!---->							<!---->	readsdram(&amp;mainDelaySD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;mainDelay<xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_dst"/>);
<!---->												
<!---->							<!---->	// load connectivity from SDRAM									
readsdram(&amp;mainConnectionSD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;connection<xsl:value-of select="$suffix"/>_temp, <xsl:value-of select="$n_dst"/>);
<!---->				
<!---->							<!---->	// load pars from SDRAM
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:if test="count($curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
<!---->							<!---->	readsdram(&amp;<xsl:value-of select="$par_name"/>SD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;<xsl:value-of select="$par_name"/><xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_dst"/>);
<!---->								</xsl:if>
								</xsl:for-each>
								
<!---->							<!---->	// load State Variables from SDRAM
<!---->							<xsl:for-each select="document($curr_synapse/@url)//SMLCL:StateVariable">
									<xsl:variable name="sv_name" select="@name"/>
<!---->								<!---->	readsdram(&amp;<xsl:value-of select="$sv_name"/>SD<xsl:value-of select="$suffix"/>[split_index][src_nrn_index][0], &amp;<xsl:value-of select="$sv_name"/><xsl:value-of select="$suffix"/>, <xsl:value-of select="$n_dst"/>);
<!---->							</xsl:for-each>
<!---->							<!---->	// launch threads							
<!---->							<!---->	for (dst_nrn_index=0; dst_nrn_index &lt; num_neurons_actual; dst_nrn_index += 1) {
<!---->							<!---->		if (connection<xsl:value-of select="$suffix"/>_temp[dst_nrn_index]==0) continue;
<!---->							<!---->		thread = createthread(<xsl:value-of select="translate($curr_synapse/@name, ' -', 'oo')"/>, 1000, src_nrn_index, split_index, dst_nrn_index, <!---->
								<xsl:for-each select="$linked_syn_file//SMLCL:Parameter">
									<xsl:variable name="par_name" select="@name"/>
									<xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>
									<xsl:if test="count($curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:UniformDistribution | $curr_synapse//SMLNL:Property[@name=$par_name]//SMLNL:ValueList)=1">
										<!---->[dst_nrn_index]<!---->
									</xsl:if>
									<!---->, <!---->
								</xsl:for-each>
								<xsl:for-each select="$linked_syn_file//SMLCL:StateVariable">
									<xsl:value-of select="@name"/><xsl:value-of select="$suffix"/>
									<!---->[dst_nrn_index], <!---->
								</xsl:for-each>
								<!----> mainDelay<xsl:value-of select="$suffix"/>[dst_nrn_index]);
<!---->							<!---->	}
<!---->							<!---->}
<!---->						</xsl:if>
						</xsl:when>
						<xsl:otherwise>
							<!---->!!! Error - connection type not recognised (<xsl:value-of select="$curr_synapse/@name"/>) !!!
<!---->					</xsl:otherwise>
					</xsl:choose>
				</xsl:for-each>

		</xsl:otherwise>
	</xsl:choose>

</xsl:if>

<!-- IF WE HAVE NO LIST BECAUSE THIS IS THE FIRST CALL OF THE TEMPLATE -->
<xsl:if test="$split_pops='none'">
	<!-- FIND ALL THE POPULATIONS IN THE SPLIT FILE WITH THE SAME BASE NAME AS THE UNSPLIT POPULATION -->
	<xsl:variable name="split_pops_new" select="$split_model_file//SMLLOWNL:Population[substring-before(SMLLOWNL:Neuron/@name, '_sub')=$curr_pop_name]"/>
	<!-- RE-CALL TEMPLATE WITH THIS LIST -->
	<xsl:apply-templates select="." mode="threadFuncs">
		<xsl:with-param name="curr_pop_name" select="$curr_pop_name"/>
		<xsl:with-param name="split_model_file" select="$split_model_file"/>
		<xsl:with-param name="split_pops" select="$split_pops_new"/>
		<xsl:with-param name="n_dst" select="$n_dst"/>
		<xsl:with-param name="n_src" select="$n_src"/>
	</xsl:apply-templates>
</xsl:if>

</xsl:template>



<xsl:include href="SpineML_Component.xsl"/>
<xsl:include href="SpineML_Dynamics.xsl"/>
<xsl:include href="SpineML_Regime.xsl"/>
<xsl:include href="SpineML_StateVariable.xsl"/>
<xsl:include href="SpineML_Parameter.xsl"/>
<xsl:include href="SpineML_AnalogPort.xsl"/>
<xsl:include href="SpineML_EventPort.xsl"/>
<xsl:include href="SpineML_Alias.xsl"/>
<xsl:include href="SpineML_Input.xsl"/>
<xsl:include href="SpineML_Property.xsl"/>

</xsl:stylesheet>
